<?php

/* {{{ Size Constants */
define("_SIZEGB_",1073741824);
define("_SIZEMB_",1048576);
define("_SIZEKB_",1024);
/* }}} */

/* {{{ Function: fsize */
/**
 * Determine the size of a file
 *
 * @param integer $fid ID of file
 * @return string
 */
function fsize($fid)
{
  $size = 'UNKNOWN';

  $sql  = "SELECT uploaded_on,name,typeid,file_type ";
  $sql .= "FROM files ";
  $sql .= "WHERE fid=?";
  $stmt = $_ENV['dbi']->prepare($sql);
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($fid));
  list($date,$name,$id,$type) = $result->fetchRow(MDB2_FETCHMODE_ORDERED); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $result->free();
  $stmt->free();
  if (!empty($name)) {
    $filename = !empty($date) ? $filename = $date."-".$name : $name;
    if (file_exists(_FILES_."$type/$id/$filename")) {
      $size = filesize(_FILES_."$type/$id/$filename");

      if ($size >= _SIZEGB_) {
        $size = number_format(($size / _SIZEGB_),2) . " GB";
      } elseif ($size >= _SIZEMB_) {
        $size = number_format(($size / _SIZEMB_),2) . " MB";
      } elseif ($size >= _SIZEKB_) {
        $size = number_format(($size / _SIZEKB_),2) . " KB";
      } elseif ($size >= 0) {
        $size = $size . " bytes";
      } else {
        $size = "0 bytes";
      }
    }
  } else {
    $size = "N/A";
  }

  return $size;
}
/* }}} */

/* {{{ Function: can_download */
/**
 * Determines whether or not a user can download a file
 *
 * @param integer $fid ID of file to check
 * @return boolean
 */
function can_download($fid)
{
  if (is_admin() or is_employee()) {
    return TRUE;
  }

  $sql  = "SELECT typeid,file_type ";
  $sql .= "FROM files ";
  $sql .= "WHERE fid=?";
  $stmt = $_ENV['dbi']->prepare($sql);
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($fid));
  list($id,$type) = $result->fetchRow(MDB2_FETCHMODE_ORDERED); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
  $result->free();
  $stmt->free();
  if (!empty($id)) {
    switch ($type) {
      case "groups":
        if (user_in_group($gid)) {
          return TRUE;
        }
        break;
      case "issues":
        if (can_view_issue($id,$_SESSION['userid'])) {
          return TRUE;
        }
        break;
      default:
        $_ENV['error']->logger("Could not determine type for file $fid");
        return FALSE;
        break;
    }
  }

  return FALSE;
}
/* }}} */

/* {{{ Function: mime_check */
/**
 * Determines if a file is valid for upload/download
 *
 * @param string $filename is the full name of the file to be reviewed
 * @return boolean
 */
function mime_check($filename)
{
  if (empty($filename)) {
    return FALSE;
  } else {
    foreach ($_ENV['config']['bad_mime'] as $key => $val) {
      if (eregi("$val$",trim($filename))) {
        return FALSE;
      }
    }
  }

  return TRUE;
}
  /* }}} */

/* {{{ Function: upload */
/**
 * Upload a file
 *
 * @param integer $id ID of the issue or group to associate with
 * @param string $type Type of upload (issue|group)
 * @return integer
 */
function upload($id,$type = "issues")
{
  // TODO this function should not be parsing form data; factor this out
  // TODO into form-processing functionality, file management, and some issue
  // TODO member functions
  clearstatcache();

  // Make sure files directory exists
  if (!file_exists(_FILES_)) {
    if (!mkdir(_FILES_,0700)) {
      $_ENV['error']->push_error(_("Creation of main file upload directory failed!"), TRUE);
      return FALSE;
    }
    chown(_FILES_,_WEBUSR_);
    chgrp(_FILES_,_WEBGRP_);
	}

  // cycle through the $_FILES array
  foreach ($_FILES as $upload) {
    if (empty($upload['name']) or $upload['size'] < 0) {
      continue;
    }

    // make sure there is a directory for this type
    if (!file_exists(_FILES_.$type)) {
      if (!mkdir(_FILES_.$type,0700)) {
        $_ENV['error']->push_error(sprintf(_("Creation of %s file upload directory failed!"),$type), TRUE);
        return FALSE;
      }
      @chown(_FILES_.$type,_WEBUSR_);
      @chgrp(_FILES_.$type,_WEBGRP_);
    }
    
    if (!file_exists(_FILES_."$type/$id")) {
      if (!mkdir(_FILES_."$type/$id",0700)) {
        $_ENV['error']->push_error(_("Creation of issue file directory failed!"), TRUE);
        return FALSE;
      }
      @chown(_FILES_."$type/$id",_WEBUSR_);
      @chgrp(_FILES_."$type/$id",_WEBGRP_);
    } 

    // check if the file is a cgi file for older versions of apache that do not handle thie gracefully
    $date = time();

    if (!mime_check($upload['name'])) {
      $newname = $date."-".$upload['name'].".dl";
    } else {
      $newname = $date."-".$upload['name'];
    }
 
    // move file from tmp to the right directory
    if (move_uploaded_file($upload['tmp_name'],_FILES_."$type/$id/$newname")) {
      @chown(_FILES_."$type/$id/$newname",_WEBUSR_);
      @chgrp(_FILES_."$type/$id/$newname",_WEBGRP_);

      $insert = array();
      $types = array();
      $insert['file_type']  = $type;
      $types[] = 'text';
      $insert['typeid']     = $id;
      $types[] = 'integer';
      $insert['userid']     = $_SESSION['userid']; 
      $types[] = 'integer';
      $insert['uploaded_on']= $date;
      $types[] = 'integer';
      $insert['name']       = $upload['name'];
      $types[] = 'text';
      $insert['private']    = $_POST['eprivate'] == "on" ? "t" : "f";
      $types[] = 'text';

      $result = $_ENV['dbi']->extended->autoExecute("files",
        $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
      $fid = dbi_last_insert_id_fix("files", "fid");

      if (!empty($fid)) {
        if ($type == "groups") {
          $_ENV['error']->logger(username($_SESSION['userid'])." uploaded ".$upload['name']." to ".group_name($id),"uploads");
        } else {
          $_ENV['error']->logger(username($_SESSION['userid'])." uploaded ".$upload['name']." to issue $id.","uploads");
          issue_log($_GET['issueid'],sprintf(_("File uploaded: %s"),$upload['name']));
        }
        $files[] = $fid;
      } else {
        $_ENV['error']->push_error(sprintf(_("%s could not be uploaded."),$upload['name']), TRUE);
      }
    }
  }

  return $files;
}
/* }}} */

/* {{{ Function: download */
/**
 * Download a file by its fid
 *
 * @param integer $fid ID of file to be downloaded
 */
function download($fid)
{
  clearstatcache();

  // check to see if user can
  // download this file
  if (can_download($fid)) {
    $sql  = "SELECT name,uploaded_on,typeid,file_type ";
    $sql .= "FROM files ";
    $sql .= "WHERE fid=? ";
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($fid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    list($name,$date,$id,$type) = $result->fetchRow(MDB2_FETCHMODE_ORDERED); 
    $result->free();
    $stmt->free();
    if (!empty($id)) {
      if (!mime_check($name)) {
        $name .= ".dl";
      }

      // make sure to get correct file
      if ($date != "" and file_exists(_FILES_."$type/$id/$date-$name")) {
        $name = $date."-".$name;
      }

      if (!empty($name) and file_exists(_FILES_."$type/$id/$name")) {
        if(!file_exists(_PATH_."/download/".$_SESSION['userid'])){
          mkdir(_PATH_."/download/".$_SESSION['userid'],0700);
        }

        $fp = fopen(_PATH_."/download/".$_SESSION['userid']."/.htaccess","w");
        fwrite($fp,"Order deny,allow\n");
        fwrite($fp,"Deny from all\n");
        fwrite($fp,"Allow from ".$_SERVER['REMOTE_ADDR']."\n");
        fclose($fp);
        symlink(_FILES_."/$type/$id/$name",_PATH_."/download/".$_SESSION['userid']."/$name");
        $_ENV['error']->logger(username($_SESSION['userid'])." downloading file $fid ($name).","downloads");
        redirect(_URL_."download/".$_SESSION['userid']."/$name");
      }
    }
  }
}
/* }}} */

/* {{{ Function: remove_dir */
/**
 * Recursively removes a directory
 *
 * @param string $directory Directory to remove
 */
function remove_dir($directory)
{
  if (!eregi("^.*\/$",$directory)) {
    $directory .= "/";
  }

  clearstatcache();

  if (file_exists($directory)) {
    if ($dir = @opendir($directory)) {
      while (($item = readdir($dir)) !== false) {
        if ($item == "." or $item == "..") {
          continue;
        }

        if (is_file($directory.$item)) {
          unlink($directory.$item);
        } else {
          remove_dir($directory.$item);
        }
      }
    }
  }
}
/* }}} */
?>
