<?php

/* {{{ Function: module_includes */
/**
 * Create array of files to include from modules
 *
 * @param string $hook Name of file to match
 * @returns string
 */
function module_includes($hook)
{
  $includes = array();

  if ($dir = @opendir(_MODULES_)) {
    while (($item = readdir($dir)) !== false) {
      if ($item{0} == "." or $item == ".." or $item == "CVS") {
        continue;
      }
		
      if (is_dir(_MODULES_.$item)) {
        $filename = "$hook.php";

        if (file_exists(_MODULES_.$item."/hooks/$filename")) {
          $include = _MODULES_.$item."/hooks/$filename";
          array_push($includes,$include);
        }
      }
    }
    closedir($dir);
  }

  return $includes;
}
/* }}} */

/* {{{ Function: gen_cache */
/**
 * Generate cache files
 *
 * @param string $table Table to pull data from
 * @param string $id Unique id field of table
 * @param string $data Data field of table to cache
 */
function gen_cache($table,$id,$data)
{
  if (!isset($_ENV['cache']) || !isset($_ENV['cache'][$table]) || !$_ENV['cache'][$table]) {
/**
 * 12/22/2003 -Changed contine to return. Fixes error from caching modification.
 */
    return;
  }
  //print "CACHE: " . _CACHE_;

  if ($fp = fopen(_CACHE_."$table-$data-cache.php","w")) {
    fwrite($fp,"<?php\n");
    fwrite($fp,"\n// DO NOT EDIT THIS FILE\n\n");
    fwrite($fp,"\${$table}_{$data}_cache = array(\n");
    
    $sql  = "SELECT $id,$data "; // safe -- these are literal arguments elsewhere
    $sql .= "FROM $table ";
    $sql .= "ORDER BY $id ASC";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute();
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);

    // generate a PHP array entry; note that the trailing ',' is
    // acceptable to the parser.
    while (list($key,$val) = $result->fetchRow(MDB2_FETCHMODE_ORDERED)) {
      fwrite($fp,sprintf("\t%5.5d => '%s',\n", $key,addslashes($val)));
    }
    $result->free();
    $stmt->free();

    fwrite($fp,");\n");
    fwrite($fp,"?>");
    fclose($fp);
  }
}
/* }}} */

/* {{{ Function: aget */
/**
 * Get a key from an array without generating PHP notices if key doesn't exist.
 *
 * @param array  $array an array
 * @param mixed  $key key/idx
 */
function aget($array,$key) {
    return isset( $array[$key] ) ? $array[$key] : null;
}

/**
 * Deletes images in folder cache/graphs olders than today
 * 
 * @author edder_rojas
 * @return boolean if was succesfully
 */
function removeOldGraphs() {
	
	$today = strtotime(date("Y-m-d"));
	$path = _GRAPHS_;
	
	$files = glob($path . "*.png");
	if( !is_array( $files ) ) {
		$_ENV['error']->debug("Internal error, unable to glob files in $path.  Path or permissions problem?");
		return false;
	}
	foreach ($files as $file) {
		$fileDate = date("Y-m-d", filemtime($file));
		$fileDateStr = strtotime($fileDate);
		
		if ($fileDateStr < $today) {
			$rc = @unlink($file);
			if( !$rc  ) {
				$_ENV['error']->debug("Internal error, unable to remove file $file.  Permissions problem?");
				return false;
			}
		}
	}
	return true;
}
?>
