<?php

/* {{{ Function: admin_notify */
/**
 * Notify all admins by email with message
 * 
 * @param string $message Message to send to admins
 * @returns nothing
 */
function admin_notify($message)
{
  $stmt = $_ENV['dbi']->prepare("SELECT primary_email from users where admin='t'");
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result= $stmt->execute();
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
  $emails = $result->fetchCol(); 
  $result->free();
  $stmt->free();
  if (is_array($emails)) {
    // Make sure we have the mailer class and initialize it
    include_once(_CLASSES_."mail.class.php");
    if (!is_object($mailer)) {
      $mailer = new MAILER();
      $mailer->set("email_from",$_ENV['config']['email']['sender']);
    }
    $mailer->subject("Issue Tracker Admin Alert");
    $mailer->to($emails);
    $mailer->message($message);
    $mailer->send();
  }
}
/* }}} */

/* {{{ Function: getfield */
/**
 * Retrieve a field from a table by an id; used by the
 * below functions.
 *
 * @param string $table Table to pull field from
 * @param string $field Field to pull from table
 * @param integer $id ID to match
 * @returns string
 */
function getfield($table,$field,$idfield,$id)
{
  $rv = 'UNKNOWN';

  $sql  = "SELECT $field ";
  $sql .= "FROM $table ";
  $sql .= "WHERE $idfield=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($id));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  if ($data = $result->fetchOne()) {
    $rv = $data;
  }
  $result->free();
  $stmt->free();

  return $rv;
}
/* }}} */

/* {{{ Function: category */
/**
 * Wrapper function for getfield to retrieve Category name
 *
 * @param integer $cid ID of category to get
 * @returns string $category
 */
function category($cid)
{
  global $categories_category_cache;

  if (empty($cid)) {
    return "&nbsp;";
  }

  if (is_array($categories_category_cache)
  and $_ENV['cache']['categories']) {
    if (array_key_exists($cid,$categories_category_cache)) {
      return $categories_category_cache[$cid];
    }
  } else {
    gen_cache("categories","cid","category");
    return getfield("categories","category","cid",$cid);
  }
}
/* }}} */

/* {{{ Function: fieldtype */
/**
 * Wrapper function for getfield to retrieve Custom field type name
 *
 * @param integer $typeid ID of custom field type to get
 * @returns string $fieldtype
 */
function fieldtype($typeid)
{
  global $fields_field_cache;

  if (empty($typeid)) {
    return "&nbsp;";
  }

  if (is_array($fields_field_cache)
  and $_ENV['cache']['fields']) {
    if (array_key_exists($typeid,$fields_field_cache)) {
      return $fields_field_cache[$typeid];
    }
  } else {
    gen_cache("fields","typeid","field");
    return getfield("fields","field","typeid",$typeid);
  }
}
/* }}} */

/* {{{ Function: product */
/**
 * Wrapper function for getfield to retrieve Product name
 *
 * @param integer $pid ID of product
 * @returns string
 */
function product($pid)
{
  global $products_product_cache;

  if (empty($pid)) {
    return "&nbsp;";
  }

  if (is_array($products_product_cache)
  and $_ENV['cache']['products']) {
    if (array_key_exists($pid,$products_product_cache)) {
      return $products_product_cache[$pid];
    }
  } else {
    gen_cache("products","pid","product");
    return getfield("products","product","pid",$pid);
  }
}
/* }}} */
/**
 * Wrapper function for getfield to retrieve Product bgcolor
 *
 * @param integer $pid ID of product
 * @param integer $level number of hours
 * @returns string
 */
function product_bgcolor($pid,$opened)
{
  if (empty($pid) || empty($opened)) {
    return (false);
  }
  $level = ((time()-$opened)/60)/60;
  $sql  = "SELECT color ";
  $sql .= "FROM products_levels ";
  $sql .= "WHERE pid=? and level <= ? ";
  $sql .= "order by level desc";
  $stmt = $_ENV['dbi']->prepare($sql);
  $result = $stmt->execute(array($pid,$level));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
  $color = $result->fetchOne();
  $result->free();
  $stmt->free();
  if ($color)
     return ($color);
  else
     return (false);
}
/* }}} */

/* {{{ Function: status */
/**
 * Wrapper function for getfield to retrieve Status name
 *
 * @param integer $sid ID of status to get
 * @returns string $status
 */
function status($sid)
{
  global $statuses_status_cache;

  if (empty($sid)) {
    return "&nbsp;";
  }

  if (is_array($statuses_status_cache)
  and $_ENV['cache']['statuses']) {
    if (array_key_exists($sid,$statuses_status_cache)) {
      return $statuses_status_cache[$sid];
    }
  } else {
    gen_cache("statuses","sid","status");
    return getfield("statuses","status","sid",$sid);
  }
}
/* }}} */

/* {{{ Function: fetch_status */
/**
 * Retrieve an array of statuses matching given type
 *
 * @param integer $type Status Type (see conf/const.php for status type constants)
 * @return array
 */
function fetch_status($type)
{
  $statuses = array();

  if (!is_array($_ENV['stype'])) {
    $_ENV['stype'] = array();
  }

  if (is_array($type)) {
    foreach ($type as $val) {
      $s = fetch_status($val);
      foreach ($s as $sid) {
        if (!in_array($sid,$statuses)) {
          array_push($statuses,$sid);
        }
      }
    }
  } else {
    if (array_key_exists($type,$_ENV['stype'])) {
      return $_ENV['stype'][$type];
    } else {
      $sql  = "SELECT sid ";
      $sql .= "FROM statuses ";
      $sql .= "WHERE status_type=? ";
      $stmt = $_ENV['dbi']->prepare($sql); 
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute(array($type));
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
      while ($sid = $result->fetchOne()) {
        if (!in_array($sid,$statuses)) {
          array_push($statuses,$sid);
        }
      }
      $result->free();
      $stmt->free();
    }

    $_ENV['stype'][$type] = $statuses;
    return $statuses;
  }

  return $statuses;
}
/* }}} */

/* {{{ Function: fetch_subgroups */
/**
 * Retrieve an array of subgroups to this group
 *
 * @param $gid: group id
 * @param $prop_issue: boolean
 * @return array
 */
function fetch_subgroups($gid, $prop_issue) {
  $stmt = $_ENV['dbi']->prepare("SELECT child_gid from sub_groups where parent_gid=? and prop_issue=?");
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($gid, $prop_issue?'t':'f'));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $gids = $result->fetchCol(); 
  $result->free();
  $stmt->free();
  if (count($gids) > 0) {
    // Gotta put parent on fron of array for proper crumb listing of group.
    array_splice($gids, 0, 0, $gid);
  } else {
    $gids = array ($gid);
  }

  return $gids;
}
/* }}} */
?>
