<?php

/* {{{ Time Constants */
/**
 * These are used throughout EnterTrack as a shorthand for various
 * fixed lengths of time
 */
define("_MINUTE_",60);
define("_HOUR_",_MINUTE_ * 60);
define("_DAY_",_HOUR_ * 24);
define("_WEEK_",_DAY_ * 7);
define("_MONTH_",_WEEK_ * 4);
define("_YEAR_",_WEEK_ * 52);
/* }}} */

/**
 * Take a amount of time in seconds and make it humanly readable
 *
 * @param integer $time Amount of time in seconds
 * @return string $string
 */
function time_format($time, $type=NULL)
{
  // now here's how to reconstruct it
  $days = (int)($time / _DAY_);
  // find the remainder of that operation
  $remainder =  $time % _DAY_;
  // now find the hours in the remainder
  $hours  =  (int)($remainder / _HOUR_);
  // and the remainder of THAT operation
  $remainder = $remainder % _HOUR_;
  // now we're down to minutes
  $minutes = (int)($remainder / _MINUTE_);
  // and what remains is ...
  $seconds = $remainder % _MINUTE_;
  
  if ($type == NULL) {
    $string = sprintf("%3.3d %s %2.2d %s %2.2d %s %2.2d %s",
      $days,$days != 1 ? _("days") : _("day"),$hours,$hours != 1 ? _("hours") : _("hour"),
      $minutes,$minutes != 1 ? _("minutes") : _("minute"),$seconds,$seconds != 1 ? _("seconds") : _("second"));
  } elseif ($type == 'simple') {
    $string = sprintf("%3.3d%s %2.2d%s %2.2d%s",
      $days,_("d"),$hours, _("h"),$minutes, _("m"));
  }
  return $string;
}
/* }}} */

/* {{{ Function: time_diff */
/**
 * Take 2 amounts of time in seconds and make interval humanly readable
 *
 * @param integer $time1 Amount of time in seconds
 * @param integer $time2 Amount to subtract
 * @return string $string
 */
function time_diff($time1,$time2,$type=NULL)
{
  $interval = $time1 - $time2;
  $direction = ($interval < 0) ? _("earlier") : _("later");
  $adjusted  = time_format(abs($interval),$type);

  if ($type == NULL)
     return sprintf("%s %s", $adjusted, $direction);
  else
     return sprintf("%s", $adjusted);
}
/* }}} */
?>

