<?php
/**
 * Database Abstraction Layer
 *
 * A thin wrapper around PEAR's 'MDB2' module.
 */
  
require_once("MDB2.php");

/* {{{ Function dbi_init */
/**
 * Initialize the DB abstraction layer.  This layer is used in every
 * context, so the argument is ignored.
 *
 * @param string $context the context
 */
function dbi_init($context)
{
  // Initialize the database abstraction layer
  $_ENV['dbi'] = &MDB2::factory(array(
          'phptype' => $_ENV['config']['db']['type'],
          'hostspec' => $_ENV['config']['db']['host']
              . ($_ENV['config']['db']['port']?":":"")
              . $_ENV['config']['db']['port'],
          'username' => $_ENV['config']['db']['user'],
          'password' => $_ENV['config']['db']['pass'],
          'database' => $_ENV['config']['db']['name'],
      ),
      array(
          'persistent' => TRUE, // TODO: make this a config option
          'use_transactions' => FALSE
      ));
  if (PEAR::isError($_ENV['dbi'])) {
    die($_ENV['dbi']->getMessage());
  }

  $_ENV['dbi']->loadModule('Extended', null, false);
}
/* }}} */

/* {{{ Function dbi_last_insert_id_fix */
/**
 * A wrapper around $mdb2->lastInsertId for MySQL queries,
 * where http://pear.php.net/bugs/bug.php?id=8051 plagues
 * at least through MDB2 v. 1.1.0.
 *
 * @param string $table: the table
 * @param string $column: the column
 */
function dbi_last_insert_id_fix($table, $column)
{
  switch ($_ENV['config']['db']['type']) {
    case 'mysql':
      $id = $_ENV['dbi']->queryOne('SELECT LAST_INSERT_ID()', 'integer');
      break;

    default:
      $id = $_ENV['dbi']->lastInsertId($table, $column);
      break;
  }

  $_ENV['error']->check_pear_error($id, __FILE__, __LINE__);
  return $id;
}

/* }}} */
/* {{{ Function dbi_concat_fix */
/**
 * A wrapper around Concatenation functionality,
 *  In Postgres using the || operator
 *  In MySQL using the CONCAT funtion
 *
 * @param string $fields : the fields to concatanate
 * @param string $newName: the new name for the concatenated fields
 */
function dbi_concat_fix($fields, $newName)
{
  if ( ! is_array( $fields ) ) { 
     return $fields . " AS " . $newName;
  }

  switch ($_ENV['config']['db']['type']) {
    case 'mysql':
      $sql = "CONCAT( " . implode(",", $fields) . " )";
      break;

    default:
      $sql = implode( " || ", $fields );
      break;      
  }
  $sql .= " AS " . $newName . " ";

  return $sql;
}
/* }}} */
?>
