<?php

/*
 * base.php
 *
 * Base class for change types.
 */

/* {{{ Class Notif_Change_Base */
class Notif_Change_Base
{
  /* {{{ Accessors */
  function userid() { return null; }
  function issueid() { return null; }
  function gids() { return null; } /* a list of groups to which this change applies */
  function eid() { return null; }
  function fid() { return null; }
  /* }}} */

  /* {{{ Function get_recipients */
  /**
   * Calculate who needs to hear about this change, and why they need to
   * hear about it.  Returns an array of <userid, reason> tuples, where
   * reason is an arbitrary string that should be handled by code in
   * notif.php.
   *
   * @return array
   */
  function get_recipients() { die("Unimplemented"); }
  /* }}} */

  ////
  // Utility functions for subclasses

  /* {{{ Function _get_recipients_for_issue */
  /**
   * Calculate who needs to hear about a change to an issue.  Adds <userid,
   * reason> tuples to $rv, where reason is an arbitrary string that
   * should be handled by code in notif.php.
   *
   * @param list &$rv the list to fill with results
   * @param object &$issue the issue object to consult
   * @param boolean $private is this change private? (defaults to false)
   */
  function _get_recipients_for_issue(&$rv, &$issue, $private=FALSE)
  {
    if (!$issue) { return; } // be sane

    // issue techs
    foreach ($issue->techs() as $userid) {
      $rv[] = array($userid, REASON_ISSUE_TECH);
    }

    // issue clients, if this isn't private
    if (!$private && !$issue->is_private()) {
      foreach ($issue->clients() as $userid) {
        $rv[] = array($userid, REASON_ISSUE_CLIENT);
      }
    }

    // issue assignees
    foreach ($issue->assignees() as $userid) {
      $rv[] = array($userid, REASON_ISSUE_ASSIGNEE);
    }

    foreach ($issue->enumerate_groups() as $gid) {
      $group = &get_group($gid);
      if ($group) {
        // get subscribers
        $subscribers = $group->subscribers();

        // group techs
        foreach ($group->techs() as $userid) {
          $rv[] = array($userid, REASON_GROUP_TECH);
          if (in_array($userid, $subscribers)) {
            $rv[] = array($userid, REASON_GROUP_SUB);
          }
        }

        // group clients, if this isn't private
        if (!$private && !$issue->is_private()) {
          foreach ($group->clients() as $userid) {
            $rv[] = array($userid, REASON_GROUP_CLIENT);
            if (in_array($userid, $subscribers)) {
              $rv[] = array($userid, REASON_GROUP_SUB);
            }
          }
        }
      }
    }
  }
  /* }}} */
}
/* }}} */
