<?php

/*
 * new_event.php
 *
 * Represents the creation of a new event.
 */

require_once(_SUBSYSTEMS_."notif/changes/base.php");

/* {{{ Class Notif_Change_New_Event */
class Notif_Change_New_Event extends Notif_Change_Base
{
  /* {{{ Constructor */
  /**
   * constructor
   *
   * Record creation of a new event.
   *
   * @param integer $userid the user creating the event
   * @param integer $issueid the issue ID
   * @param integer $eid the event ID
   * @param string $action the new text of the event
   * @param boolean $private is this event private?
   */
  function Notif_Change_New_Event($userid, $issueid, $eid, $action, $private)
  {
    $this->_userid = $userid;
    $this->_issueid = $issueid;
    $this->_eid = $eid;
    $this->issue = &get_issue($issueid);
    $this->action = $action;
    $this->private = $private;
  }
  /* }}} */

  /* {{{ Accessors */
  function userid() { return $this->_userid; }
  function issueid() { return $this->_issueid; }
  function eid() { return $this->_eid; }
  function gids() { return $this->issue->enumerate_groups(); }
  /* }}} */

  /* {{{ Function get_recipients */
  /**
   * Calculate who needs to hear about this change, and why they need to
   * hear about it.  Returns an array of <userid, reason> tuples, where
   * reason is an arbitrary string that should be handled by code in
   * notif.php.
   *
   * @return array
   */
  function get_recipients()
  {
    $rv = array();
    $this->_get_recipients_for_issue($rv, $this->issue, $this->private);

    return $rv;
  }
  /* }}} */

  function summary()
  {
    return "Notif_Change_New_Event({$this->_userid}, {$this->_eid}, {$this->_issueid} (".$this->issue->summary()."), {$this->action}, {$this->private})";
  }
}
/* }}} */
