<?php

/*
 * new_issue.php
 *
 * Represents the creation of a new issue.
 */

require_once(_SUBSYSTEMS_."notif/changes/base.php");

/* {{{ Class Notif_Change_New_Issue */
class Notif_Change_New_Issue extends Notif_Change_Base
{
  /* {{{ Function constructor */
  /**
   * constructor
   *
   * Record creation of a new issue.
   *
   * @param integer $userid the user creating the issue
   * @param integer $gid the groupid for the new issue
   * @param integer $issueid the new issue's ID
   * @param string $summary the new issue's summary
   * @param string $description the new issue's description
   */
  function Notif_Change_New_issue($userid, $gid, $issueid, $summary, $description)
  {
    $this->_userid = $userid;
    $this->_gid = $gid;
    $this->_issueid = $issueid;
    $this->issue = &get_issue($issueid);
    $this->summary = $summary;
    $this->description = $description;
  }
  /* }}} */

  /* {{{ Accessors */
  function userid() { return $this->_userid; }
  function gids() { return array($this->_gid); }
  function issueid() { return $this->_issueid; }
  /* }}} */

  /* {{{ Function get_recipients */
  /**
   * Calculate who needs to hear about this change, and why they need to
   * hear about it.  Returns an array of <userid, reason> tuples, where
   * reason is an arbitrary string that should be handled by code in
   * notif.php.
   *
   * @return array
   */
  function get_recipients()
  {
    $rv = array();
    $this->_get_recipients_for_issue($rv, $this->issue);

    return $rv;
  }
  /* }}} */

  function summary()
  {
    return "Notif_Change_New_Issue({$this->_userid}, {$this->_gid}, {$this->_issueid} (".$this->issue->summary()."), {$this->summary}, {$this->description})";
  }
}
/* }}} */
