<?php

/*
 * settings_changes.php
 *
 * The settings_changes class represents a collection of settings changes to
 * the same issue.
 */

require_once(_SUBSYSTEMS_."notif/changes/base.php");

/* {{{ Class Notif_Change_Settings_Change */
class Notif_Change_Settings_Change extends Notif_Change_Base
{
  /* {{{ Function constructor */
  /**
   * Record a new settings changes.
   *
   * @param integer $userid the userid making the changes
   * @param integer $issueid the issue being changed
   * @param string $setting the setting being changed
   * @param string $old_value the old value for $setting
   * @param string $new_value the new value for $setting
   */
  function Notif_Change_Settings_Change($userid, $issueid, $setting, $old_value, $new_value)
  {
    $this->_userid = $userid;
    $this->_issueid = $issueid;
    $this->issue = &get_issue($issueid);
    $this->setting = $setting;
    $this->old_value = $old_value;
    $this->new_value = $new_value;
  }
  /* }}} */

  /* {{{ Accessors */
  function userid() { return $this->_userid; }
  function issueid() { return $this->_issueid; }
  function gids() { return $this->issue->enumerate_groups(); }
  /* }}} */

  /* {{{ Function get_recipients */
  /**
   * Calculate who needs to hear about this change, and why they need to
   * hear about it.  Returns an array of <userid, reason> tuples, where
   * reason is an arbitrary string that should be handled by code in
   * notif.php.
   *
   * @return array
   */
  function get_recipients()
  {
    $rv = array();
    $this->_get_recipients_for_issue($rv, $this->issue);

    return $rv;
  }
  /* }}} */

  function summary()
  {
    return "Notif_Change_Settings_Change({$this->_userid}, {$this->_issueid} (".$this->issue->summary()."), {$this->setting}, {$this->old_value}, {$this->new_value})";
  }
}
/* }}} */
