<?php

/*
 * session.php
 * 
 * Session-handling code.  This module has no API -- instead, based on the
 * ET config, it either sets up the default PHP session handler or installs
 * the custom ET session handler.
 */

/* {{{ Function session_init */
/**
 * Initialize session handling with the given context; sessions are only used
 * in the 'web' context.
 */
function session_init($context)
{
  /* return if we're not in a web context */
  if ($context == 'email') { return; }

  if (!$_ENV['config']['session']['et_handler']) {
    // For the PHP handler, we need write access to the sessions directory
    if (!is_writable(_SESSIONS_)) {
      echo "Sessions directory is not writable by the web server.  Please correct this, or
        disable the ET session handler in <tt>conf/config.php</tt>:
        <pre>\$_ENV['config']['session']['et_handler'] = false;</pre>";
      exit;
    }

    // Set the required parameters for the PHP handler
    session_save_path(_SESSIONS_);
     # Local Session Handling for this App. --marc@gridshield.net
     ini_set('session.gc_probability',5);
     ini_set("session.gc_maxlifetime",_MAX_SESSION_LIFETIME_);

  } else {
    // read the session handler functions and install them
    require_once(_SUBSYSTEMS_."session/et_handler.php");
    session_set_save_handler(
      "et_session_open",
      "et_session_close",
      "et_session_read",
      "et_session_write",
      "et_session_destroy",
      "et_session_gc"
    );
  }

  // Either way, start up the session
  session_name($_ENV['config']['session']['name']);

  // In CSS, we relax the caching rules
  if ($context == 'css') {
    session_cache_limiter('public');
  }

  session_start();
}
/* }}} */
?>
