<?php

/* session_handler.php
 *
 * This file is subordinate to session.php, and defines the functions that
 * implement the ET session handler.  It is only included if that handler
 * is configured.
 */

/* {{{ Function: et_session_open */
/**
 * Start a session - Do not access this function directly
 */
function et_session_open($save_path,$session_name)
{
  // perform GC on every session open
  et_session_gc();
  return TRUE;
}
/* }}} */

/* {{{ Function: et_session_close */
/**
 * Close a session - Do not access this function directly
 */
function et_session_close()
{
  return TRUE;
}
/* }}} */

/* {{{ Function: et_session_read */
/**
 * Read session data - Do not access this function directly
 */
function et_session_read($id)
{
  $expire = time() + ini_get("session.gc_maxlifetime");
  $data = '';

  $sql  = "SELECT session_data ";
  $sql .= "FROM sessions ";
  $sql .= "WHERE session_id=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($id));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $data = $result->fetchOne();
  $result->free();
  $stmt->free();
    
  if (is_null($data)) {
    $insert = array();
    $types = array();
    $insert['session_id'] = $id;
    $types[] = 'text';
    $insert['session_data'] = 'blank';
    $types[] = 'text';
    $insert['session_expires'] = $expire;
    $types[] = 'integer';
    $result = $_ENV['dbi']->extended->autoExecute("sessions", $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  }

  return $data;
}
/* }}} */

/* {{{ Function: et_session_write */
/**
 * Write session data - Do not access this function directly
 */
function et_session_write($id,$data)
{
  $expire = time() + ini_get("session.gc_maxlifetime");

  $result = $_ENV['dbi']->extended->autoExecute("sessions",
    array("session_data" => $data, "session_expires" => $expire),
    MDB2_AUTOQUERY_UPDATE,
    "session_id=" . $_ENV['dbi']->quote($id, 'text'),
    array('text', 'integer'));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  return TRUE;
}
/* }}} */

/* {{{ Function: et_session_destroy */
/**
 * Destroy session - Do not access this function directly
 */
function et_session_destroy($id)
{
  $result = $_ENV['dbi']->extended->autoExecute("sessions",
    null,
    MDB2_AUTOQUERY_DELETE,
    "session_id=" . $_ENV['dbi']->quote($id, 'integer')); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
  return TRUE;
}
/* }}} */

/* {{{ Function: et_session_gc */
/**
 * Session garbage cleanup - Do not access this function directly
 */
function et_session_gc()
{
  // TODO - emulate PHP's probabalistic GC algorithm
  $result = $_ENV['dbi']->extended->autoExecute("sessions",
    null,
    MDB2_AUTOQUERY_DELETE,
    "session_expires < " . time()); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  return TRUE;
}
/* }}} */
?>
