<?php

/*
 * tpl.php
 * 
 * This subsystem initializes the Smarty template engine, and stores
 * a reference to the Smarty object in $_ENV['tpl'];
 */

/* {{{ Function tpl_init */
/**
 * Initialize Smarty, the template engine.  Smarty is used only web page
 * generation as well as email formatting, so we pretty much need it in
 * every context.
 *
 * @param string $context the context
 */
function tpl_init($context)
{
  include_once((defined('SMARTY_DIR')? SMARTY_DIR:'')."Smarty.class.php");

  tpl_subclass_init();

  $_SESSION['theme'] = (!empty($_SESSION['theme']) and file_exists(_THEMES_.$_SESSION['theme'])) ? $_SESSION['theme'] : $_ENV['config']['display']['default_theme'];

  $_ENV['tpl'] = &new ETSmarty;

  $_ENV['tpl']->template_dir = _THEMES_.$_SESSION['theme']."/tpl/";
  $_ENV['tpl']->compile_dir  = _TPLCOMPILE_;
  $_ENV['tpl']->config_dir   = _TPLCONFIG_;
  $_ENV['tpl']->cache_dir    = _TPLCACHE_;
  $_ENV['tpl']->debugging    = FALSE;
  $_ENV['tpl']->caching      = FALSE;  // DO NOT CHANGE THIS!

  # set up some basic stuff that should be accessible everywhere
  $_ENV['tpl']->assign("config", $_ENV['config']);

  // Register template modifiers
  $_ENV['tpl']->register_modifier("number_format","tpl_number_format");
  $_ENV['tpl']->register_modifier("format","tpl_text_format");
  $_ENV['tpl']->register_modifier("format_datetime","tpl_format_datetime");

  // Register template functions
  $_ENV['tpl']->register_function("opennavtable","tpl_opennavtable");
  $_ENV['tpl']->register_function("closenavtable","tpl_closenavtable");
  $_ENV['tpl']->register_function("opentable","tpl_opentable");
  $_ENV['tpl']->register_function("closetable","tpl_closetable");
  $_ENV['tpl']->register_function("titlebar","tpl_titlebar");
  $_ENV['tpl']->register_function("username","tpl_username");
  $_ENV['tpl']->register_function("ident","tpl_ident");
  $_ENV['tpl']->register_function("groupname","tpl_group_name");
  $_ENV['tpl']->register_function("rowcolor","tpl_rowcolor");
  $_ENV['tpl']->register_function("bgcolor","tpl_bgcolor");
  $_ENV['tpl']->register_function("alphalist","tpl_alpha_list");
  $_ENV['tpl']->register_function("sevimg","tpl_severity_image");
  $_ENV['tpl']->register_function("sevtxt","tpl_severity_text");
  $_ENV['tpl']->register_function("category","tpl_category");
  $_ENV['tpl']->register_function("product","tpl_product");
  $_ENV['tpl']->register_function("status","tpl_status");
  $_ENV['tpl']->register_function("fsize","tpl_fsize");
  $_ENV['tpl']->register_function("subtitle","tpl_subtitle");
  $_ENV['tpl']->register_function("datetime_select","tpl_datetime_select");
  $_ENV['tpl']->register_function("gettext","tpl_gettext");
  $_ENV['tpl']->register_function("_","tpl_gettext");
  $_ENV['tpl']->register_function("breadcrumbs","tpl_breadcrumbs");
  $_ENV['tpl']->register_function("time_diff","tpl_time_diff");
}
/* }}} */

function tpl_subclass_init() {
    class ETSmarty extends Smarty {
        var $display = TRUE;
    
        /**
         * Toggle display. Allows us to globally disable calls to display(),
         * eg for text/plain pages.
         */
        function toggle_display($display)
        {
            $this->display = $display ? TRUE : FALSE;
        }
    
        function display( $tpl ) {
    
            if( $this->display ) {
                return parent::display( $tpl );
            }
            return TRUE;
        }
    }
}

/***
 * Remainder are Smarty callbacks as registered in tpl_init
 ***/
  
/**
 * Format number string
 */
function tpl_number_format($number,$decimal = 0)
{
  return number_format($number,$decimal);
}

/**
 * Open navigation table
 */
function tpl_opennavtable($params,&$smarty)
{
  // Only show errors through nav if not logged in yet
  if (!$_SESSION['userid'] && $_ENV['error']->errors_pending()) {
    $smarty->display("errors.tpl");
  }

  $smarty->display("opennavtable.tpl");
}

/**
 * Close navigation table
 */
function tpl_closenavtable($params,&$smarty)
{
  $smarty->display("closenavtable.tpl");
}

/**
 * Open content table
 */
function tpl_opentable($params,&$smarty)
{
  if ($_ENV['error']->errors_pending()) {
    $smarty->display("errors.tpl");
  }

  $smarty->display("opentable.tpl");
}

/**
 * Close content table
 */
function tpl_closetable($params,&$smarty)
{
  $smarty->display("closetable.tpl");
}

/**
 * Create titlebar row and possible linkbar row
 *
 * @param integer $colspan Number of columns titlebar should take
 * @param string $title Title to put in titlebar
 */
function tpl_titlebar($params,&$smarty)
{
  global $links;

  extract($params);

  $colspan = empty($colspan) ? 1 : $colspan;
  $title = empty($title) ? "&nbsp;" : $title;

  $buffer = "<tr><td class=\"titlebar\" colspan=\"$colspan\">"._($title)."</td></tr>\n";

  if (is_array($links) and count($links) > 0) {
    $buffer .= '<tr><td class="subtitle" colspan="'.$colspan.'">';
    foreach ($links as $link) {
      $buffer .= '[ ';
      $buffer .= '<a href="'.$link['url'].'">';
      if (!empty($link['img'])) {
        $buffer .= '<img src="'.$link['img'].'" width="16" height="16" border="0" title="'._($link['txt']).'" />&nbsp;';
      }
      $buffer .= _($link['txt']);
      $buffer .= '</a> ]';
    }
    $buffer .= '</td></tr>';
    $links = array();
  }

  return $buffer;
}

/**
 * Wrapper to username function for use in templates
 *
 * @param integer $id ID of user to return username for
 */
function tpl_username($params,&$smarty)
{
  extract($params);

  $username = username($id);
  return $username;
}

/**
 * Wrapper to ident function for use in templates
 *
 * @param integer $id ID of user
 */
function tpl_ident($params,&$smarty)
{
  extract($params);

  return ident($id);
}

/**
 * Wrapper to group_name function for use in templates
 *
 * @param integer $id ID of group to return name for
 */
function tpl_group_name($params,&$smarty)
{
  extract($params);

  $name = group_name($id);
  return $name;
}

/**
 * Wrapper to format_datetime function for use in templates
 *
 * @param integer $time Unix Timestamp
 */
function tpl_format_datetime($text)
{
  if ($text) {
    return $_ENV['i18n']->format_datetime($text);
  } else {
    return '';
  }
}

/**
 * Wrapper to rowcolor function for use in templates
 */
function tpl_rowcolor($params,&$smarty)
{
  static $rowclass;
  extract($params);

  $closed = fetch_status(array(TYPE_CLOSED,TYPE_AUTO_CLOSED));
  $bgcolor = product_bgcolor($pid,$opened);
  if (!$bgcolor || in_array($status,$closed) || $_SESSION['prefs']['show_bgcolor'] == 'f') {
     if (empty($rowclass)) {
       $rowclass = 1;
     }
     if ($rowclass == 2) {
       $rowclass = 1;
       return "row2";
     } else {
       $rowclass = 2;
       return "row1";
     }
  }
}

/**
 * Wrapper to bgcolor function for use in templates
 */
function tpl_bgcolor($params,&$smarty)
{
  extract($params);
  $closed = fetch_status(array(TYPE_CLOSED,TYPE_AUTO_CLOSED));
  if (!in_array($status,$closed) || $_SESSION['prefs']['show_bgcolor'] == 't') 
     return (product_bgcolor($pid,$opened));
}

/**
 * Builds list of links to each letter
 *
 * @param string $url
 */
function tpl_alpha_list($params,&$smarty)
{
  extract($params);
  if (!isset($curent)) $current='';

  $regular_fmt = "<a href=\"$url&start=%s\">%s</a>&nbsp;\n";
  $match_fmt = "<a href=\"$url&start=%s\"><b>%s</b></a>&nbsp;\n";
  
  // make sure the URL data is URL-safe, while the alphabet gets translated..
  $buffer = sprintf($current=="ALL"?$match_fmt:$regular_fmt, "ALL", _("ALL"));
  foreach (explode(" ", _("A B C D E F G H I J K L M N O P Q R S T U V W X Y Z")) as $ch) {
    $buffer .= sprintf($current==$ch?$match_fmt:$regular_fmt, urlencode($ch), $ch);
  }

  return $buffer;
}

/**
 * Wrapper to severity_image function for use in templates
 */
function tpl_severity_image($params,&$smarty)
{
  extract($params);
  switch($sev){
    case SEV_URGENT:    $img = $_ENV['imgs']['urgent']; break;
    case SEV_HIGH:      $img = $_ENV['imgs']['high'];   break;
    case SEV_NORMAL:    $img = $_ENV['imgs']['normal']; break;
    default:            $img = $_ENV['imgs']['low'];    break;
  }

  return $img;
}

/**
 * Wrapper to severity_text function for use in templates
 */
function tpl_severity_text($params,&$smarty)
{
  extract($params);
  return severity_text($sev);
}

/**
 * Wrapper to category function for use in templates
 */
function tpl_category($params,&$smarty)
{
  extract($params);
  return category($id);
}

/**
 * Wrapper to product function for use in templates
 */
function tpl_product($params,&$smarty)
{
  extract($params);
  return product($id);
}

/**
 * Wrapper to status function for use in templates
 */
function tpl_status($params,&$smarty)
{
  extract($params);
  return status($id);
}


/**
* Wrapper to time_diff function for use in templates
*/
function tpl_time_diff($params,&$smarty)
{
  extract($params);
  return time_diff($time1, $time2, $type);
}


/**
 * Format plain text for viewing in html
 *
 * @param string $text Text to format
 */
function tpl_text_format($text)
{
  // Get rid of carriage returns
  $text = str_replace("\r","",$text);
  // Strip out all html except simple text formatting
  $text = strip_tags($text,'<a><b><i><u>');
  // Convert urls to hyperlinks
  $text = eregi_replace("((http://)|(https://).[^\s]+)\ ","<a href=\"\\0\">\\0</a>",$text); 

  if ($_SESSION['prefs']['disable_wrap'] != "t") {
    $lines = explode("\n",$text);
    $text = "";

    $wrap = ereg("[0-9]+",$_SESSION['prefs']['word_wrap']) ? $_SESSION['prefs']['word_wrap'] : 80;
    foreach ($lines as $key => $val) {
      if (empty($val)) {
        $text .= "\n";
      } else {
        if (strlen($val) > $wrap) {
          $val = wordwrap($val,$wrap,"\n",TRUE);
        }

        $text .= $val."\n";
      }
    }
    
    $text = "<pre>$text</pre>";
  } else {
    // Replace newlines with <br />'s
    $text = str_replace("\n","<br />",$text);
  }
  return $text;
}

/**
 * Wrapper to the fsize function for use in templates
 *
 * @param integer $id Id of file to check size of
 */
function tpl_fsize($params,&$smarty)
{
  extract($params);
  return fsize($id);
}

/**
 * Create subtitle bar
 *
 * @param string $text Text to show in subtitle
 * @param integer $colspan Number of columns to take up
 */
function tpl_subtitle($params,&$smarty)
{
  extract($params);
  
  $buffer = "<tr><td class=\"subtitle\" colspan=\"$colspan\">"._($title)."</td></tr>";
  return $buffer;
}

/**
 * Date selection field, uses javascript calendar if available
 *
 * @param string $name Field name, also used for id
 * @param integer $value Default value
 */
function tpl_datetime_select($params,&$smarty)
{
  extract($params);
  
  if (empty($name)) {
    return;
  }
  
	$format = $_ENV['i18n']->get_date_format_spec();
  	
  if (!ereg("(4\.7)|(4\.8)",$_SERVER['HTTP_USER_AGENT']) and $_SESSION['javascript'] == "t") {
    $buffer  = '<input type="text" size="21" id="'.$name.'" name="'.$name.'" value="'.$value.'" />';
    $buffer .= '<a href="#" id="'.$name.'-trigger">';
    $buffer .= '<img src="' . _JS_CALENDAR_URI_ . '/img.gif" border="0" id="'.$name.'-img" style="cursor: pointer; vertical-align: top !important;" />';
    $buffer .= '</a>';
    $buffer .= '<script type="text/javascript">Calendar.setup({ showsTime : true, inputField : "' . $name . 
               '", button : "'.$name.'-trigger", ifFormat : "' . $format .'", daFormat : "' . $format . '" });</script>';
  } else {
    $buffer  = '<input type="text" size="20" name="'.$name.'" value="'.$value.'" />';
  }

  return $buffer;
}

/**
 * Wrapper function for gettext
 *
 * @param string $txt Text message
 * @param string $domain Text Domain to use
 */
function tpl_gettext($params,&$smarty)
{
  extract($params);

  if (empty($txt)) {
    return "";
  }

  if (!empty($domain)) {
    return dgettext($domain,$txt);
  } else {
    return _($txt);
  }
}

/**
 * Build breadcrumbs
 *
 * @param string $divider Character(s) to use as divider
 */
function tpl_breadcrumbs($args, &$smarty)
{
  $divider = " :: ";
  $crumbs = " <a href=\""._URL_."\">"._("Main")."</a>";

  if (!empty($_GET['module'])) {
    $crumbs .= "$divider<a href=\"?module=".$_GET['module']."\">";
    $crumbs .= _(ucwords($_GET['module']));
    $crumbs .= "</a>";
  }

  if (!empty($_GET['start'])) {
    $crumbs .= "<a href=\"?module=".$_GET['module'].
    	(!empty($_GET['action'])?"&action=".$_GET['action']:"")."&start=".$_GET['start']."\">";
    $crumbs .= "("._($_GET['start']).")";
    $crumbs .= "</a>";
  }

  if (!empty($_GET['type']) and empty($_GET['action'])) {
    $crumbs .= "$divider<a href=\"?module=".$_GET['module']."&type=".$_GET['type']."\">";
    $crumbs .= _(ucwords($_GET['type']));
    $crumbs .= "</a>";
  }

  if (!empty($_GET['gid'])) {
    if ($_GET['module'] == "issues") {
      $crumbs .= "$divider<a href=\"?module=issues&action=group&gid=".$_GET['gid']."\">";
    } else {
      $crumbs .= "$divider<a href=\"?module=groups&action=view&gid=".$_GET['gid']."\">";
    }
    $crumbs .= group_name($_GET['gid']);
    $crumbs .= "</a>";
  }

  if (!empty($_GET['uid'])) {
    $crumbs .= "$divider<a href=\"?module=users&action=view&uid=".$_GET['uid']."\">";
    $crumbs .= username($_GET['uid']);
    $crumbs .= "</a>";
  }

  if (!empty($_GET['issueid'])) {
    if (empty($_GET['gid'])) {
      $sql  = "SELECT primary_gid ";
      $sql .= "FROM issues ";
      $sql .= "WHERE issueid=? ";
      $stmt = $_ENV['dbi']->prepare($sql); 
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute(array($_GET['issueid']));
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
      $gid = $result->fetchOne();
      if ($gid) {
        $crumbs .= "$divider<a href=\"?module=issues&action=group&gid=$gid\">";
        $crumbs .= group_name($gid);
        $crumbs .= "</a>";
      }
      $result->free();
      $stmt->free();
    }

    $crumbs .= "$divider<a href=\"?module=issues&action=view&issueid=".$_GET['issueid']."\">";
    $crumbs .= _("Issue #").$_GET['issueid'];
    $crumbs .= "</a>";
  }

  if (!empty($_GET['action'])) {
    /**
     * Added this if/else logic for $_GET['action'] to
     * handle the new issue reports.
     * 
     * --Tim Boring (tim -at- timboring dot com) 2/28/2006
     */
    if (preg_match("/custom/i", $_GET['action'])) {
        list($dir,$report) = explode("/",$_GET['action']);
        $action = _(ucwords(str_replace("_"," ",$report)));
    } else {
        $uri = str_replace("/?module=".$_GET['module'],"",$_SERVER['REQUEST_URI']);
        $uri = str_replace("&action=".$_GET['action'],"",$uri);
        $action = _(ucwords(str_replace("_"," ",$_GET['action'])));
    }
    
    if (!empty($_GET['subaction'])) {
      $action .= " ("._(ucwords(str_replace("_"," ",$_GET['subaction']))).")";
    }

    if (!empty($_GET['type'])) {
      $action .= " ("._(ucwords(str_replace("_"," ",$_GET['type']))).")";
    }

    $url = "?module=".$_GET['module']."&action=".$_GET['action'];

    if (!empty($_GET['gid'])) {
      $url .= "&gid=".$_GET['gid'];
      $uri = str_replace("&gid=".$_GET['gid'],"",$uri);
    }

    if (!empty($_GET['uid'])) {
      $url .= "&uid=".$_GET['uid'];
      $uri = str_replace("&uid=".$_GET['uid'],"",$uri);
    }

    if (!empty($_GET['issueid'])) {
      $url .= "&issueid=".$_GET['issueid'];
      $uri = str_replace("&issueid=".$_GET['issueid'],"",$uri);
    }

    $crumbs .= "$divider<a href=\"$url\">$action</a>";
  }

  return $crumbs;
}
