<?php

if (eregi(basename(__FILE__),$_SERVER['PHP_SELF'])) {
  print "Direct module access forbidden.";
  exit;
}

/* {{{ Function: announcement_title */
/**
 * Gets title for announement
 *
 * @param integer $aid Id of announcement to pull title for
 * @return string $title
 */
function announcement_title($aid)
{
	$sql  = "SELECT title ";
	$sql .= "FROM announcements ";
	$sql .= "WHERE aid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
	$result = $stmt->execute(array($aid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
  $title = $result->fetchOne();
  $result->free();
  $stmt->free();
}
/* }}} */

/* {{{ Function: announcements */
/**
 * Pulls an array of announcements for the given group,
 * if gid is omitted then pull for global announcements
 *
 * @param integer $gid Id of group to pull announcements for
 * @return array
 */
function announcements($gid = null)
{
  // make sure to initialize $announcements as an array
  $announcements = array();

  if (empty($gid)) {
    $sql  = "SELECT aid,title ";
    $sql .= "FROM announcements ";
    $sql .= "WHERE is_global='t'";
    $sql .= "ORDER BY aid DESC";
    $params = array();
  } else {
    $sql  = "SELECT a.aid,a.title ";
    $sql .= "FROM announcements a,announce_permissions p ";
    $sql .= "WHERE a.aid=p.aid ";
    $sql .= "AND p.gid=? ";
    $sql .= "ORDER BY a.aid DESC";
    $params = array($gid);
  }

  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute($params);
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  while (list($aid,$title) = $result->fetchRow(MDB2_FETCHMODE_ORDERED)) {
    $announcements[$aid] = $title;
  }
  $result->free();
  $stmt->free();

  if (!empty($gid)) {
    $sql  = "SELECT s.parent_gid ";
    $sql .= "FROM sub_groups s, groups g ";
    $sql .= "WHERE s.parent_gid=g.gid ";
    $sql .= "AND s.child_gid=? ";
    $sql .= "AND g.prop_announce='t'";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $parent = $result->fetchOne();
    if ($parent) {
      $a = announcements($parent);

      if (count($a) > 0) {
        foreach ($a as $key => $val) {
          if (!array_key_exists($key,$announcements)) {
            $announcements[$key] = $val;
          }
        }
      }
    }
    $result->free();
    $stmt->free();

    $sql  = "SELECT child_gid ";
    $sql .= "FROM sub_groups ";
    $sql .= "WHERE parent_gid=? ";
    $sql .= "AND prop_announce='t'";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $child = $result->fetchOne();
    $result->free();
    $stmt->free();

    if ($child) {
      $a = announcements($child);

      if (count($a) > 0) {
        foreach ($a as $key => $val) {
          if (!array_key_exists($key,$announcements)) {
            $announcements[$key] = $val;
          }
        }
      }
    }
  }

  return $announcements;
}
/* }}} */

/* {{{ Function: can_view_announcement */
/**
 * Determine if a user can see an announcement
 *
 * @param integer $aid Id of announcement
 * @return boolean
 */
function can_view_announcement($aid)
{
  $permitted = TRUE;

	$sql  = "SELECT aid ";
	$sql .= "FROM announcements ";
	$sql .= "WHERE aid=? ";
	$sql .= "AND is_global='t'";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
	$result = $stmt->execute(array($aid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
	$permitted = ($result->numRows() > 0);
  $result->free();
  $stmt->free();

  if ($permitted) return TRUE;

  $sql  = "SELECT gid ";
  $sql .= "FROM announce_permissions ";
  $sql .= "WHERE aid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($aid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  while($gid = $result->fetchOne()) {
    if(in_array($gid,$_SESSION['groups'])){
      $permitted = TRUE;
      break;
    }
  }
  $result->free();
  $stmt->free();

  return $permitted;
}
/* }}} */
?>
