<?php

require_once(_INCLUDES_."/classes/dbobject.class.php");

/* {{{ Class Event */
/**
 * Event
 *
 * A class representing an event.  The class implements write-through caching,
 * such that any changes are reflected immediately into the database, and data
 * is loaded as needed.  Settings-access functions always return the existing
 * value, and set a new value if given an argument.
 */
class Event extends DBObject {
  /* {{{ Function constructor*/
  /**
   * Construct a new object with the given eid, assuming the
   * event exists in the database.
   *
   * @param $eid
   */
  function Event($issueid, $eid, $events_row)
  {
    $this->_eid = $eid;
    $this->_issueid = $issueid;
    $this->_data = $events_row;
  }
  /* }}} */

  /* {{{ Accessors */
  function eid() { return $this->_eid; }
  function issueid() { return $this->issueid; }

  function action($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("action", 'string');
    } else {
      return $this->_set("action", $new_val, 'string');
    }
  }

  function userid($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("userid", 'userid');
    } else {
      return $this->_set("userid", $new_val, 'userid', false);
    }
  }

  function is_private($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("private", 'boolean');
    } else {
      return $this->_set("private", $new_val, 'boolean', false);
    }
  }

  function status($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("status", 'status');
    } else {
      return $this->_set("status", $new_val, 'status', false);
    }
  }

  function fid($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("fid", 'number');
    } else {
      return $this->_set("fid", $new_val, 'number', false);
    }
  }

  function performed_on($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("performed_on", 'time');
    } else {
      return $this->_set("performed_on", $new_val, 'time', false);
    }
  }
  /* }}} */

  /* {{{ Function _get */
  /**
   * A general accessor function -- get the given setting.
   *
   * @param string $setting
   * @param string $type the type of the value (see includes/classes/dbobject.class.php)
   * @return mixed the old value of the setting
   */
  function _get($setting, $type)
  {
    $old_value = $this->_db_to_native($this->_data[$setting], $type);

    return $old_value;
  }
  /* }}} */

  /* {{{ Function _set */
  /**
   * A general mutator function -- set the given setting, and if NEW_VALUE is
   * specified, record that new setting in the database.  This both performs an
   * update query and records a notification.
   *
   * @param string $setting
   * @param mixed $new_value the new value for the setting, or null to simply
   *              retrieve the old value
   * @return mixed the old value of the setting
   */
  function _set($setting, $new_value, $type, $send_notif=true)
  {
    $old_value = $this->_db_to_native($this->_data[$setting], $type);

    if ($old_value != $new_value) {
      // change value
      $db_value = $this->_native_to_db($new_value, $type);
      $issue = &get_issue($this->_issueid);

      $result = $_ENV['dbi']->extended->autoExecute("events",
        array($setting => $db_value),
        MDB2_AUTOQUERY_UPDATE,
        sprintf("eid=%s", $_ENV['dbi']->quote($this->_eid, 'integer')),
        array($this->_db_type($type))); 
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
      
      if ($send_notif) {
        $_ENV['notif']->record_settings_change($_SESSION['userid'],
            $issue,
            "Event #{$this->_eid}/$setting",
            $this->_native_to_human($old_value, $type),
            $this->_native_to_human($new_value, $type));
      }
     }

    return $old_value;
  }
  /* }}} */
}
/* }}} */
?>
