<?php

require_once(_INCLUDES_."/classes/dbobject.class.php");

/* {{{ Class File */
/**
 * File
 *
 * A class representing a file.  The class implements write-through caching,
 * such that any changes are reflected immediately into the database, and data
 * is loaded as needed.  Settings-access functions always return the existing
 * value, and set a new value if given an argument.
 */
class File extends DBObject {
  /* {{{ Function constructor*/
  /**
   * Construct a new object with the given fid, assuming the
   * file exists in the database.
   *
   * @param $fid
   */
  function File($fid, $files_row)
  {
    $this->_fid = $fid;
    $this->_data = $files_row;
  }
  /* }}} */

  /* {{{ Accessors */
  function fid() { return $this->_fid; }

  function userid($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("userid", 'userid');
    } else {
      return $this->_set("userid", $new_val, 'userid');
    }
  }
  function typeid($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("typeid", 'number');
    } else {
      return $this->_set("typeid", $new_val, 'number');
    }
  }
  function uploaded_on($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("uploaded_on", 'time');
    } else {
      return $this->_set("uploaded_on", $new_val, 'time');
    }
  }
  function name($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("name", 'string');
    } else {
      return $this->_set("name", $new_val, 'string');
    }
  }
  function file_type($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("file_type", 'string');
    } else {
      return $this->_set("file_type", $new_val, 'string');
    }
  }
  function is_private($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("is_private", 'boolean');
    } else {
      return $this->_set("is_private", $new_val, 'boolean');
    }
  }
  /* }}} */

  /* {{{ Function _get */
  /**
   * A general accessor function -- get the given setting.
   *
   * @param string $setting
   * @param string $type the type of the value (see includes/classes/dbobject.class.php)
   * @return mixed the value of the setting
   */
  function _get($setting, $type)
  {
    $this->_load();
    $old_value = $this->_db_to_native($this->_data[$setting], $type);
    return $old_value;
  }
  /* }}} */

  /* {{{ Function _set */
  /**
   * A general mutator function -- Record a new setting in the database, and
   * return the old value.  This both performs an update query and records a
   * notification.
   *
   * @param string $setting
   * @param mixed $new_value the new value for the setting, or null to simply
   *              retrieve the old value
   * @param string $type the type of the value (see includes/classes/dbobject.class.php)
   * @return mixed the old value of the setting
   */
  function _set($setting, $new_value, $type)
  {
    $this->_load();
    $old_value = $this->_db_to_native($this->_data[$setting], $type);

    if ($old_value != $new_value) {
      // change the value
      $db_value = $this->_native_to_db($new_value, $type);
      $result = $_ENV['dbi']->extended->autoExecute("files",
        array($setting => $db_value),
        MDB2_AUTOQUERY_UPDATE,
        sprintf("fid=%s", $_ENV['dbi']->quote($this->_fid, 'integer')),
        array($this->_db_type($type))); 
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
      $this->_data[$setting] = $db_value;

      // TODO notifications for files
      // XXX doesn't have an associated gid -> get issue, use its primary
      // $_ENV['notif']->record_settings_change(userid, $this->_issueid, $setting,
      //         $this->_native_to_human($old_value, $type),
      //         $this->_native_to_human($new_value, $type));
    }

    return $old_value;
  }
  /* }}} */

}
/* }}} */
?>
