<?php
/* $Id: class.php 270 2007-05-20 18:18:54Z djmitche $ */

require_once(_INCLUDES_."/classes/dbobject.class.php");

/* {{{ Class Group */
/**
 * Group
 *
 * A class representing a group.  The class implements write-through caching,
 * such that any changes are reflected immediately into the database, and data
 * is loaded as needed.  Settings-access functions always return the existing
 * value, and set a new value if given an argument.
 */
class Group extends DBObject {
  /* {{{ Function constructor*/
  /**
   * Construct a new object with the given gid, assuming the
   * group exists in the database.
   *
   * @param integer $gid the groups id
   * @param array $groups_row the row from the groups table
   */
  function Group($gid, $groups_row)
  {
    $this->_gid = $gid;
    $this->_data = $groups_row;
  }
  /* }}} */

  ////
  // Accessors

  /* {{{ Accessors */
  function gid() { return $this->_gid; }

  function name($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("name", 'string');
    } else {
      return $this->_set("name", $new_val, 'string');
    }
  }

  function address($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("address", 'string');
    } else {
      return $this->_set("address", $new_val, 'string');
    }
  }

  function address2($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("address2", 'string');
    } else {
      return $this->_set("address2", $new_val, 'string');
    }
  }

  function contact($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("contact", 'string');
    } else {
      return $this->_set("contact", $new_val, 'string');
    }
  }

  function tech($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("tech", 'string');
    } else {
      return $this->_set("tech", $new_val, 'string');
    }
  }

  function tao($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("tao", 'string');
    } else {
      return $this->_set("tao", $new_val, 'string');
    }
  }

  function brm($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("brm", 'string');
    } else {
      return $this->_set("brm", $new_val, 'string');
    }
  }

  function sales($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("sales", 'string');
    } else {
      return $this->_set("sales", $new_val, 'string');
    }
  }

  function amount($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("amount", 'number');
    } else {
      return $this->_set("amount", $new_val, 'number');
    }
  }

  function bought($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("bought", 'number');
    } else {
      return $this->_set("bought", $new_val, 'number');
    }
  }

  function used($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("used", 'number');
    } else {
      return $this->_set("used", $new_val, 'number');
    }
  }

  function start_date($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("start_date", 'string');
    } else {
      return $this->_set("start_date", $new_val, 'string');
    }
  }

  function end_date($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("end_date", 'string');
    } else {
      return $this->_set("end_date", $new_val, 'string');
    }
  }

  function group_type($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("group_type", 'string');
    } else {
      return $this->_set("group_type", $new_val, 'string');
    }
  }

  function active($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("active", 'boolean');
    } else {
      return $this->_set("active", $new_val, 'boolean');
    }
  }

  function status_reports($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("status_reports", 'boolean');
    } else {
      return $this->_set("status_reports", $new_val, 'boolean');
    }
  }

  function notes($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("notes", 'string');
    } else {
      return $this->_set("notes", $new_val, 'string');
    }
  }

  function email($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("email", 'string');
    } else {
      return $this->_set("email", $new_val, 'string');
    }
  }

  // remaining attributes omitted for now
  /* }}} */

  ////
  // Higher-level functions

  /* {{{ Function: techs */
  /**
   * Return a list of techs for the group.
   *
   * TODO should eventually correspond to a "role" in the group
   */
  function techs()
  {
    // XXX this is temporarily how we determine this.. this will need to change to
    // something far more appropriate
    $sql  = "SELECT userid ";
    $sql .= "FROM group_users join permission_sets on group_users.perm_set = permission_sets.permsetid ";
    $sql .= "WHERE group_users.gid = ? ";
    $sql .= "AND permission_sets.permissions like '%view_private%'";
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($this->_gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $rv = $result->fetchCol(); 
    $result->free();
    $stmt->free();
    if (!$rv) { $rv = array(); }
    return $rv;
  }
  /* }}} */

  /* {{{ Function: clients */
  /**
   * Return a list of clients for the group.
   *
   * TODO should eventually correspond to a "role" in the group
   */
  function clients()
  {
    // XXX this is temporarily how we determine this.. this will need to change to
    // something far more appropriate
    $sql  = "SELECT userid ";
    $sql .= "FROM group_users join permission_sets on group_users.perm_set = permission_sets.permsetid ";
    $sql .= "WHERE group_users.gid = ? ";
    $sql .= "AND permission_sets.permissions not like '%view_private%'";
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($this->_gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $rv = $result->fetchCol(); 
    $result->free();
    $stmt->free();
    if (!$rv) { $rv = array(); }
    return $rv;
  }
  /* }}} */

  /* {{{ Function: is_tech */
  /**
   * Is the given user a tech for this group?
   *
   * TODO should eventually correspond to a "role" in the group
   *
   * @param integer $userid the userid of the user to investigate
   */
  function is_tech($userid)
  {
    // XXX this is temporarily how we determine this.. this will need to change to
    // something far more appropriate
    $sql  = "SELECT userid ";
    $sql .= "FROM group_users join permission_sets on group_users.perm_set = permission_sets.permsetid ";
    $sql .= "WHERE group_users.userid=? ";
    $sql .= "AND group_users.gid=? ";
    $sql .= "AND permission_sets.permissions like '%view_private%' ";
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($userid, $this->_gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $userid = $result->fetchOne(); 
    $result->free();
    $stmt->free();

    if ($userid) {
      return TRUE;
    }

    return FALSE;
  }
  /* }}} */

  /* {{{ Function: is_client */
  /**
   * Is the given user a client for this group?
   *
   * TODO should eventually correspond to a "role" in the group
   *
   * @param integer $userid the userid of the user to investigate
   */
  function is_client($userid)
  {
    // XXX this is temporarily how we determine this.. this will need to change to
    // something far more appropriate
    $sql  = "SELECT userid ";
    $sql .= "FROM group_users join permission_sets on group_users.perm_set = permission_sets.permsetid ";
    $sql .= "WHERE group_users.userid=? ";
    $sql .= "AND group_users.gid=? ";
    $sql .= "AND permission_sets.permissions not like '%view_private%' ";

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($userid, $this->_gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $userid = $result->fetchOne(); 
    $result->free();
    $stmt->free();

    if ($userid) {
      return TRUE;
    }

    return FALSE;
  }
  /* }}} */

  /* {{{ Function: subscribers */
  /**
   * Return a list of users who are subscribed to this group
   */
  function subscribers()
  {
    $stmt = $_ENV['dbi']->prepare("SELECT userid from group_users where subscribed='t' and gid = ?");
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($this->_gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $rv = $result->fetchCol(); 
    $result->free();
    $stmt->free();
    if (!$rv) { $rv = array(); }
    return $rv;
  }
  /* }}} */

  ////
  // Private functions

  /* {{{ Function _get */
  /**
   * A general accessor function -- get the given setting.
   *
   * @param string $setting
   * @param string $type the type of the value (see includes/classes/dbobject.class.php)
   * @return mixed the value of the setting
   */
  function _get($setting, $type)
  {
    $old_value = $this->_db_to_native($this->_data[$setting], $type);
    return $old_value;
  }
  /* }}} */

  /* {{{ Function _set */
  /**
   * A general mutator function -- Record a new setting in the database, and
   * return the old value.  This both performs an update query and records a
   * notification.
   *
   * @param string $setting
   * @param mixed $new_value the new value for the setting, or null to simply
   *              retrieve the old value
   * @param string $type the type of the value (see includes/classes/dbobject.class.php)
   * @return mixed the old value of the setting
   */
  function _set($setting, $new_value, $type)
  {
    $old_value = $this->_db_to_native($this->_data[$setting], $type);

    if ($old_value != $new_value) {
      // change the value
      $db_value = $this->_native_to_db($new_value, $type);
      $result = $_ENV['dbi']->extended->autoExecute("groups",
        array($setting => $db_value),
        MDB2_AUTOQUERY_UPDATE,
        sprintf("gid=%s", $_ENV['dbi']->quote($thi->_gid, 'integer')),
        array($this->_db_type($type))); 
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
      $this->_data[$setting] = $db_value;
    }

    return $old_value;
  }
  /* }}} */
}
/* }}} */
