<?php

require_once(_INCLUDES_."/classes/dbobject.class.php");

# TODO: add issue logging alongside the notifications (or maybe as part of the notifications??)

/* {{{ Class Issue */
/**
 * Issue
 *
 * A class representing an issue.  The class implements write-through caching,
 * such that any changes are reflected immediately into the database, and data
 * is loaded as needed.  Settings-access functions always return the existing
 * value, and set a new value if given an argument.
 */
class Issue extends DBObject {
  /* {{{ Function constructor*/
  /**
   * Construct a new object with the given issueid, assuming the
   * issue exists in the database.
   *
   * @param $issueid
   */
  function Issue($issueid, $data=null)
  {
    $this->_issueid = $issueid;
    $this->_data = $data;
    $this->_groups = null;
    $this->_events = null;
    $this->_techs = null;
    $this->_clients = null;
  }
  /* }}} */

  /* {{{ Accessors */
  function issueid() {
    return $this->_issueid;
  }

  function primary_gid($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("primary_gid", 'gid');
    } else {
      return $this->_set("primary_gid", $new_val, 'gid', true);
    }
  }
  function opened_by($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("opened_by", 'userid');
    } else {
      return $this->_set("opened_by", $new_val, 'userid', false);
    }
  }
  function opened($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("opened", 'time');
    } else {
      return $this->_set("opened", $new_val, 'time');
    }
  }
  function closed($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("closed", 'time');
    } else {
      return $this->_set("closed", $new_val, 'time');
    }
  }
  function modified($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("modified", 'time');
    } else {
      return $this->_set("modified", $new_val, 'time', false);
    }
  }
  function summary($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("summary", 'string');
    } else {
      return $this->_set("summary", $new_val, 'string', false);
    }
  }
  function problem($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("problem", 'string');
    } else {
      return $this->_set("problem", $new_val, 'string');
    }
  }
  function status($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("status", 'status');
    } else {
      return $this->_set("status", $new_val, 'status', false);
    }
  }
  function category($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("category", 'category');
    } else {
      return $this->_set("category", $new_val, 'category', false);
    }
  }
  function product($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("product", 'product');
    } else {
      return $this->_set("product", $new_val, 'product', false);
    }
  }
  function severity($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("severity", 'severity');
    } else {
      return $this->_set("severity", $new_val, 'severity', false);
    }
  }
  function is_private($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("private", 'boolean');
    } else {
      return $this->_set("private", $new_val, 'boolean', false);
    }
  }
  function istatus($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("istatus", 'status');
    } else {
      return $this->_set("istatus", $new_val, 'status', false);
    }
  }
  function iseverity($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("iseverity", 'severity');
    } else {
      return $this->_set("iseverity", $new_val, 'severity', false);
    }
  }
  function due_date($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("due_date", 'time');
    } else {
      return $this->_set("due_date", $new_val, 'time', false);
    }
  }
  function real_opened($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("real_opened", 'time');
    } else {
      return $this->_set("real_opened", $new_val, 'time', false);
    }
  }
  function real_closed($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("real_closed", 'time');
    } else {
      return $this->_set("real_closed", $new_val, 'time', false);
    }
  }
  function modification_note($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("modification_note", 'time');
    } else {
      return $this->_set("modification_note", $new_val, 'time', false);
    }
  }
  function adjusted_by($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("adjusted_by", 'userid');
    } else {
      return $this->_set("adjusted_by", $new_val, 'userid', false);
    }
  }
  /* }}} */

  ////
  // High level (non-trivial) modifications to issues
  //
  // These call through to the lower-level functions

  /* {{{ Function: escalate */
  /**
   * Escalate an issue
   *
   * @param integer $gid Group to escalate to
   */
  function escalate($gid)
  {
    $ig = $this->group($gid);

    if ($ig) {
      // if this issue already has a record for that group,
      // change its show_issue setting
      $ig->show_issue(TRUE);
    } else {
      // Otherwise, add a new issue group
      $ig = $this->add_group($gid);
      $ig->show_issue(TRUE);
      $ig->opened(time());
      $this->modified(time());
    }
  }
  /* }}} */

  /* {{{ Function: deescalate */
  /**
   * De-escalate an issue
   *
   * @param integer $gid Group to de-escalate to
   */
  function deescalate($gid)
  {
    $ig = $this->group($gid);
    if (!$ig) { return; } // this shouldn't happen

    // don't show this group for this issue anymore
    $ig->show_issue(FALSE);
    $ig->deescalated(TRUE);

    // add an event (XXX is this necessary?)
    $this->add_event(group_name($gid)." has de-escalated the issue.",
      $_SESSION['userid'], null, 0.0, TRUE, null, null, $gid);

    // update the issue itself
    $this->modified(time());
    
    issue_log($this->_issueid,sprintf(_("De-escalated from %s"),group_name($gid)),TRUE);
  }
  /* }}} */

  /* {{{ Function: reopen */
  /**
   * Reopen an issue
   */
  function reopen()
  {
    list($registered) = fetch_status(TYPE_REGISTERED);
    $this->status($registered);
  	issue_log($this->_issueid,_("Issue Reopened"));
  }
  /* }}} */

  /* {{{ Function: add_file */
  /**
   * Add a file to an issue
   *
   * @author Dustin J. Mitchell <dustin@cs.uchicago.edu>
   * @param integer $userid the userid responsible for the addition
   * @param integer $file the body of the file
   * @param integer $filename the body of the file
   * @return boolean, true on success
   */
  function add_file($issueid, $userid, $file, $filename)
  {
    $issueid = $this->_issueid;

    ////
    // First, make sure the proper directory structure is in place.

    // Make sure the files directory is defined
    if (!defined("_FILES_")) {
      $_ENV['error']->logger("Files directory not defined.","downloads");
      return false;
    }

    // Make sure the files directory exists
    if (file_exists(_FILES_)) {
      // Make sure files directory is writeable
      if (!is_writeable(_FILES_)) {
        $_ENV['error']->logger("Files directory '"._FILES_."' not writeable.","downloads");
        return false;
      }
    } else {
      // Create the files directory
      if (!mkdir(_FILES_,0700)) {
        $_ENV['error']->logger("Files directory could not be created.","downloads");
        return false;
      }
      chown(_FILES_,_WEBUSR_);
      chgrp(_FILES_,_WEBGRP_);
    }

    // Make sure the 'issues' directory exists
    if (!file_exists(_FILES_."issues")) {
      if (!mkdir(_FILES_."issues",0700)) {
        $_ENV['error']->logger("Issues file directory could not be created.","downloads");
        return false;
      }
      chown(_FILES_."issues",_WEBUSR_);
      chgrp(_FILES_."issues",_WEBGRP_);
    } else {
      // Make sure the directory is writeable
      if (!is_writeable(_FILES_."issues")) {
        $_ENV['error']->logger("Issues file directory not writeable.","downloads");
        return false;
      }
    }

    // Make sure the issue specific directory exists
    if (!file_exists(_FILES_."issues/$issueid")) {
      if (!mkdir(_FILES_."issues/$issueid",0700)) {
        $_ENV['error']->logger("Could not create files directory for issue $issueid.","downloads");
        return false;
      }
      chown(_FILES_."issues/$issueid",_WEBUSR_);
      chgrp(_FILES_."issues/$issueid",_WEBGRP_);
    } else {
      // Make sure the issue specific file directory is writeable
      if (!is_writeable(_FILES_."issues/$issueid")) {
        $_ENV['error']->logger("Files directory for issue $issueid is not writeable.","downloads");
        return false;
      }
    }

    ////
    // Now actually post the uploaded file

    // Get current timestamp
    $timestamp = time();

    // prepend a timestamp to the file
    if (!empty($filename)) {
      $full_filename = "$timestamp-$filename";
    } else {
      $full_filename = "$timestamp-anonymous";
    }

    $pathname = _FILES_."issues/$issueid/$full_filename";

    // TODO: special-case filename, fopen mode based on content-type?
    // TODO: store content-type and return on download
    // TODO: this should happen under modules/files, and most of this stuff should
    // be abstracted there too, perhaps as an interface to one of several backends?
    if (!($fp = fopen($pathname,"w"))) {
      $_ENV['error']->logger("Could not open '$pathname' for writing.", "downloads");
      return false;
    }

    fwrite($fp, $file);
    $_ENV['error']->logger("Wrote '$pathname'.", "downloads");
    fclose($fp);
    chown($pathname,_WEBUSR_);
    chgrp($pathname,_WEBGRP_);

    $_ENV['error']->logger($filename." uploaded to issue $issueid.","downloads");
    issue_log($issueid,sprintf(_("File uploaded: %s"),$filename), FALSE, $userid);

    $insert = array();
    $types = array();
    $insert['file_type'] = "issues";
    $types[] = 'text';
    $insert['typeid'] = $issueid;
    $types[] = 'integer';
    $insert['userid'] = $userid;
    $types[] = 'integer';
    $insert['uploaded_on'] = $timestamp;
    $types[] = 'integer';
    $insert['name'] = $filename;
    $types[] = 'text';
    $result = $_ENV['dbi']->extended->autoExecute("files",
      $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);

    return dbi_last_insert_id_fix("files", "fid");
  }
  /* }}} */

  ////
  // Subscribers (clients and techs) functions

  /* {{{ Function: clients */
  /**
   * Return the list of clients for this issue
   *
   * @return array of clients' userids
   */
  function clients()
  {
    $this->_load_subscribers();
    return $this->_clients;
  }
  /* }}} */

  /* {{{ Function: techs */
  /**
   * Return the list of technicians for this issue
   *
   * @return array of techs' userids
   */
  function techs()
  {
    $this->_load_subscribers();
    return $this->_techs;
  }
  /* }}} */

  /* {{{ Function: is_client */
  /**
   * Is the given user a client of this issue?
   *
   * @param integer $userid the userid of the user to investigate
   */
  function is_client($userid)
  {
    $this->_load_subscribers();
    return in_array($userid, $this->_clients);
  }
  /* }}} */

  /* {{{ Function: is_tech */
  /**
   * Is the given user a tech of this issue?
   *
   * @param integer $userid the userid of the user to investigate
   */
  function is_tech($userid)
  {
    $this->_load_subscribers();
    return in_array($userid, $this->_techs);
  }
  /* }}} */

  /* {{{ Function: add_client */
  /**
   * Add a new client to the list of clients for this issue.  Note that
   * if this user is already a client or a tech, no change takes place.
   *
   * @param integer $userid the userid of the client to add
   */
  function add_client($userid)
  {
    $this->_load_subscribers();

    // if they're already in there, we're done
    if (in_array($userid, $this->_clients)) { return; }
    if (in_array($userid, $this->_techs)) { return; }

    // add to our local cache
    $this->_clients[] = $userid;

    // insert into db
    $insert = array();
    $types = array();
    $insert['issueid'] = $this->_issueid;
    $types[] = 'integer';
    $insert['userid'] = $userid;
    $types[] = 'integer';
    $insert['is_tech'] = 'f';
    $types[] = 'text';
    $result = $_ENV['dbi']->extended->autoExecute("issue_subscribers",
      $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  }
  /* }}} */

  /* {{{ Function: add_tech */
  /**
   * Add a new tech to the list of techs for this issue.  Note that
   * if this user is already a client, then when this function returns
   * they will no longer be a client.
   *
   * @param integer $userid the userid of the tech to add
   */
  function add_tech($userid)
  {
    $this->_load_subscribers();

    // if they're already in there, we're done
    if (in_array($userid, $this->_techs)) { return; }

    // add to our local cache
    $this->_techs[] = $userid;

    // if they're not a client, we'll do an INSERT; otherwise an UPDATE
    $clients_idx = array_search($userid, $this->_clients);
    if ($clients_idx === FALSE) {
      $insert = array();
      $types = array();
      $insert['issueid'] = $this->_issueid;
      $types[] = 'integer';
      $insert['userid'] = $userid;
      $types[] = 'integer';
      $insert['is_tech'] = 't';
      $types[] = 'text';

      $result = $_ENV['dbi']->extended->autoExecute("issue_subscribers",
        $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    } else {
      unset($this->_clients[$clients_idx]);

      $result = $_ENV['dbi']->extended->autoExecute("issue_subscribers",
        array("is_tech" => "t"),
        MDB2_AUTOQUERY_UPDATE,
        sprintf("userid=%s and issueid=%s",
                $_ENV['dbi']->quote($userid),
                $_ENV['dbi']->quote($this->_issueid)),
        array("text")); 
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    }
  }
  /* }}} */

  /* {{{ Function: remove_client */
  /**
   * Remove a client from the list of clients
   *
   * @param integer $userid the userid of the client to remove
   */
  function remove_client($userid)
  {
    // first delete it
    $result = $_ENV['dbi']->extended->autoExecute("issue_subscribers",
      null,
      MDB2_AUTOQUERY_DELETE,
      sprintf("is_tech = 'f' and userid=%s and issueid=%s", 
            $_ENV['dbi']->quote($userid, 'integer'), 
            $_ENV['dbi']->quote($this->_issueid, 'integer'))); 
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

    // and then invalidate our cache
    $this->_clients = null;
  }
  /* }}} */

  /* {{{ Function: remove_tech */
  /**
   * Remove a tech from the list of techs
   *
   * @param integer $userid the userid of the techs to remove
   */
  function remove_tech($userid)
  {
    // first delete it
    $result = $_ENV['dbi']->extended->autoExecute("issue_subscribers",
      null,
      MDB2_AUTOQUERY_DELETE,
      sprintf("is_tech = 't' and userid=%s and issueid=%s", 
            $_ENV['dbi']->quote($userid, 'integer'), 
            $_ENV['dbi']->quote($this->_issueid, 'integer'))); 
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

    // and then invalidate our cache
    $this->_techs = null;
  }
  /* }}} */

  ////
  // Event functions

  /* {{{ Function: enumerate_events */
  /**
   * List the events for this issue, sorted by performed_on.
   *
   * @param integer $eid
   * @return object the Event object (see modules/events/hooks/class.php)
   */
  function enumerate_events($eid)
  {
    $events = $this->_load_events();
    return array_keys($events);
  }
  /* }}} */

 /* {{{ Function: count_events */
 /**
  * Count all the events added for this issue.
  *
  * @return number of events for the issue
  */
  function count_events()
  {
     $events = $this->_load_events();
     return count($events);
  }
  /* }}} */

  /* {{{ Function: event */
  /**
   * Get the Event object for the given eid.
   *
   * @param integer $eid
   * @return object the Event object (see modules/events/hooks/class.php)
   */
  function event($eid)
  {
    $events = $this->_load_events();
    if (array_key_exists($eid, $events)) {
      return $events[$eid];
    } else {
      return null;
    }
  }
  /* }}} */

  /* {{{ Function: add_event */
  /**
   * Add a new event to this issue
   *
   * @param integer $action the text of the event
   * @param integer $userid the user creating the event
   * @param integer $status the new status of the issue
   * @param float $duration the length of time spent on this event
   * @param boolean $private is this event private?
   * @param integer $fid ID of an attached file, if any
   * @param integer $performed_on the time the event was created (default now)
   * @return object the Event object (see modules/events/hooks/class.php), or FALSE if an error occurred
   */
  function add_event($action, $userid, $status=null, $duration=0, $private=FALSE, $fid=null, $performed_on=null, $gid)
  {
    if (is_null($performed_on)) { $performed_on = time(); }

    $insert = array();
    $types = array();
    $insert['issueid'] = $this->_issueid;
    $types[] = 'integer';
    $insert['action'] = $action;
    $types[] = 'text';
    $insert['userid'] = $userid;
    $types[] = 'integer';
    $insert['status'] = $status;
    $types[] = 'integer';
    $insert['duration'] = $duration;
    $types[] = 'decimal';
    $insert['private'] = $private? 't' : 'f';
    $types[] = 'text';
    $insert['gid'] = $gid;
    $types[] = 'integer';
    
    if ($fid) {
        $insert['fid'] = $fid;
        $types[] = 'integer';
    }
    $insert['performed_on'] = $performed_on;
    $types[] = 'integer';

    $result = $_ENV['dbi']->extended->autoExecute("events",
      $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $eid = dbi_last_insert_id_fix("events", "eid");

    if (empty($eid)) {
      $_ENV['error']->logger("Could not add new event", "phperrors");
      return false;
    }

    issue_log($this->_issueid,_("Event posted"),FALSE,$userid);

    // add this user as a client, if they're not already a client or tech
    $this->add_client($userid);

    // record this for notification
    $_ENV['notif']->record_new_event($userid, $this->_issueid, $eid, $action, $private);

    // load the Event object into the cache and then return it
    $this->_load_event($eid);
    return $this->event($eid);
  }
  /* }}} */

  ////
  // Group functions

  /* {{{ Function enumerate_groups */
  /**
   * Enumerate the groups in which this issue appears
   *
   * @return array of gids
   */
  function enumerate_groups()
  {
    $groups = $this->_load_groups();
    return array_keys($groups);
  }
  /* }}} */

  /* {{{ Function emenumerate_groups_user */
  /**
   * Enumerate the groups in which this issue appears and the user has permission
   *
   * @param integer $userid
   * @return array of gids
   */
  function enumerate_groups_user($userid)
  {
    $groups = $this->_load_groups();
    foreach ($groups as $g) {
    	if (user_in_group($g->gid(),$userid))
	   $group[] = $g->gid();
    }
    return $group;
  }
  /* }}} */



  /* {{{ Function group */
  /**
   * Get a reference to issue information pertaining to the given group.
   *
   * @param integer $gid
   * @return object an IssueGroup object
   */
  function group($gid)
  {
    $groups = $this->_load_groups();
    if (array_key_exists($gid, $groups)) {
      return $groups[$gid];
    } else {
      return null;
    }
  }
  /* }}} */

  /* {{{ Function add_group */
  /**
   * Add a new group to this issue
   *
   * @param integer $gid
   * @return object an IssueGroup object
   */
  function add_group($gid)
  {
    $insert = array();
    $types = array();
    $insert['issueid'] = $this->_issueid;
    $types[] = 'integer';
    $insert['gid'] = $gid;
    $types[] = 'integer';

    $result = $_ENV['dbi']->extended->autoExecute("issue_groups",
      $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

    return $this->_load_group($gid);
  }
  /* }}} */

  /* {{{ Function remove_group */
  /**
   * Delete information about the indicated group.
   *
   * @param integer $gid
   */
  function remove_group($gid)
  {
    $result = $_ENV['dbi']->extended->autoExecute("issue_groups",
      null,
      MDB2_AUTOQUERY_DELETE,
      sprintf("issueid=%s and gid=%s", 
            $_ENV['dbi']->quote($this->_issueid, 'integer'), 
            $_ENV['dbi']->quote($gid, 'integer'))); 
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

    return $this->_del_group($gid);
  }
  /* }}} */

  /* {{{ Function assignees */
  /**
   * Retrive an array (gid => userid) of users assigned to this issue.
   *
   * @return array (gid => userid)
   */
  function assignees()
  {
    $group_data = $this->_load_groups();

    $rv = array();
    foreach ($group_data as $gid => $info) {
      $assigned = $info->assigned_to();
      if ($assigned && $info->show_issue()) { $rv[$gid] = $assigned; }
    }

    return $rv;
  }
  /* }}} */

  ////
  // Private functions

  /* {{{ Function _load */
  /**
   * Load the issue from the database
   */
  function _load()
  {
    if (!is_null($this->_data)) { return; }

  }
  /* }}} */

  /* {{{ Function _get */
  /**
   * A general accessor function -- get the given setting.
   *
   * @param string $setting
   * @param string $type the type of the value (see includes/classes/dbobject.class.php)
   * @return mixed the value of the setting
   */
  function _get($setting, $type)
  {
    $this->_load();
    $old_value = $this->_db_to_native($this->_data[$setting], $type);
    return $old_value;
  }
  /* }}} */

  /* {{{ Function _set */
  /**
   * A general mutator function -- Record a new setting in the database, and
   * return the old value.  This both performs an update query and records a
   * notification.
   *
   * @param string $setting
   * @param mixed $new_value the new value for the setting, or null to simply
   *              retrieve the old value
   * @param string $type the type of the value (see includes/classes/dbobject.class.php)
   * @param boolean $send_notif should a notification be sent? (defaults to True)
   * @return mixed the old value of the setting
   */
  function _set($setting, $new_value, $type, $send_notif=TRUE)
  {
    $this->_load();
    $old_value = $this->_db_to_native($this->_data[$setting], $type);

    if ($old_value != $new_value) {
      // change the value
      $db_value = $this->_native_to_db($new_value, $type);
      $result = $_ENV['dbi']->extended->autoExecute("issues",
        array($setting => $db_value),
        MDB2_AUTOQUERY_UPDATE,
        sprintf("issueid=%s", $_ENV['dbi']->quote($this->_issueid, 'integer')),
        array($this->_db_type($type))); 
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

      $this->_data[$setting] = $db_value;

      if ($send_notif) {
        $_ENV['notif']->record_settings_change($_SESSION['userid'],
                $this->_issueid, $setting,
                $this->_native_to_human($old_value, $type),
                $this->_native_to_human($new_value, $type));
      }
    }

    return $old_value;
  }
  /* }}} */

  /* {{{ Function: _load_events */
  /**
   * Load and return the events for this issue, sorted by performed_on.
   *
   * @return array with eids as keys and Event objects as values
   */
  function _load_events()
  {
    if (!is_null($this->_events)) { return $this->_events; }

    $sql  = "SELECT eid, issueid, status, userid, performed_on, duration, fid, private, action ";
    $sql .= "FROM events ";
    $sql .= "WHERE issueid=? ";
    $sql .= "ORDER BY performed_on ";
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($this->_issueid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $event_rows = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    $this->_events = array();
    foreach ($event_rows as $event_row) {
      $eid = $event_row['eid'];
      $this->_events[$eid] =& new Event($this->_issueid, $eid, $event_row);
    }
    return $this->_events;
  }
  /* }}} */

  /* {{{ Function: _load_event */
  /**
   * Load and return an Event object for the given eid; used when a new
   * row is inserted.  This function assumes that the newly added event is
   * later than the events already in the issue.
   *
   * @param integer $eid
   * @return object an Event object
   */
  function _load_event($eid)
  {
    $this->_load_events();

    $sql  = "SELECT eid, status, userid, performed_on, duration, fid, private, action ";
    $sql .= "FROM events ";
    $sql .= "WHERE issueid=? AND eid=? ";
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($this->_issueid, $eid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $event_rows = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    $event_row = $event_rows[0];
    $this->_events[$eid] =& new IssueGroup($this->_issueid, $eid, $event_row);
    return $this->_events[$eid];
  }
  /* }}} */

  /* {{{ Function: _load_groups */
  /**
   * Load and return the issue_groups data for this issue.
   *
   * @return array with gids as keys and IssueGroup objects as values
   */
  function _load_groups()
  {
    if (!is_null($this->_groups)) { return $this->_groups; }

    $sql  = "SELECT gid, assigned_to, opened, first_response, show_issue, deescalated ";
    $sql .= "FROM issue_groups ";
    $sql .= "WHERE issueid=? ";
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($this->_issueid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $group_rows = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    $this->_groups = array();
    foreach ($group_rows as $group_row) {
      $gid = $group_row['gid'];
      $this->_groups[$gid] =& new IssueGroup($this->_issueid, $gid, $group_row);
    }
    return $this->_groups;
  }
  /* }}} */

  /* {{{ Function: _load_group */
  /**
   * Load and return an IssueGroup object for the given gid; used when a new
   * row is inserted.
   *
   * @param integer $gid
   * @return object an IssueGroup object
   */
  function _load_group($gid)
  {
    $this->_load_groups();

    $sql  = "SELECT assigned_to, opened, first_response, show_issue, deescalated ";
    $sql .= "FROM issue_groups ";
    $sql .= "WHERE issueid=? AND gid=?";
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($this->_issueid, $gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $group_rows = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    $group_row = $group_rows[0];
    $this->_groups[$gid] =& new IssueGroup($this->_issueid, $gid, $group_row);
    return $this->_groups[$gid];
  }
  /* }}} */

  /* {{{ Function: _load_subscribers */
  /**
   * Load the list of subscribers for this issue
   */
  function _load_subscribers()
  {
    if (is_null($this->_clients) || is_null($this->_techs)) {
      $sql  = "SELECT userid, is_tech ";
      $sql .= "FROM issue_subscribers ";
      $sql .= "WHERE issueid=? ";
      $stmt = $_ENV['dbi']->prepare($sql);
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute(array($this->_issueid));
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
      $subscribers = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
      $result->free();
      $stmt->free();

      $this->_clients = array();
      $this->_techs = array();
      if ($subscribers) {
        foreach ($subscribers as $subscriber) {
          if ($subscriber['is_tech'] == 't') {
            $this->_techs[] = $subscriber['userid'];
          } else {
            $this->_clients[] = $subscriber['userid'];
          }
        }
      }
    }
  }
  /* }}} */

  /* {{{ Function: _del_group */
  /**
   * Remove the indicated group from the cached list of IssueGroup objects; used
   * when a row is deleted from the table.
   *
   * @param integer $gid
   */
  function _del_group($gid)
  {
    if (!is_null($this->_groups)) {
      unset($this->_groups[$gid]);
    }
  }
  /* }}} */
}
/* }}} */

/* {{{ Class IssueGroup */
/**
 * IssueGroup
 *
 * Information about an issue that is specific to a group.
 */
class IssueGroup extends DBObject
{
  /* {{{ Function constructor */
  /**
   * @param integer $issueid
   * @param integer $gid
   * @param array $issue_groups_row
   */
  function IssueGroup($issueid, $gid, $issue_groups_row)
  {
    $this->_issueid = $issueid;
    $this->gid = $gid;
    $this->group_name = group_name($gid);

    $this->_data = $issue_groups_row;
  }
  /* }}} */

  /* {{{ Accessors */
  function issueid() { return $this->_issueid; }
  function gid() { return $this->gid; }
  function group_name() { return $this->group_name; }

  function assigned_to($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("assigned_to", 'userid');
    } else {
      return $this->_set("assigned_to", $new_val, 'userid', true);
    }
  }

  function opened($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("opened", 'time');
    } else {
      return $this->_set("opened", $new_val, 'time', false);
    }
  }

  function first_response($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("first_response", 'time');
    } else {
      return $this->_set("first_response", $new_val, 'time', false);
    }
  }

  function show_issue($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("show_issue", 'boolean');
    } else {
      return $this->_set("show_issue", $new_val, 'boolean', false);
    }
  }

  function deescalated($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("deescalated", 'boolean');
    } else {
      return $this->_set("deescalated", $new_val, 'boolean', false);
    }
  }
  /* }}} */

  /* {{{ Function _get */
  /**
   * A general accessor function -- get the given setting.
   *
   * @param integer $issueid
   * @param string $type the type of the value (see includes/classes/dbobject.class.php)
   * @return mixed the old value of the setting
   */
  function _get($setting, $type)
  {
    $old_value = $this->_db_to_native($this->_data[$setting], $type);

    return $old_value;
  }
  /* }}} */

  /* {{{ Function _set */
  /**
   * A general mutator function -- set the given setting, and if NEW_VALUE is
   * specified, record that new setting in the database.  This both performs an
   * update query and records a notification.
   *
   * @param integer $issueid
   * @param mixed $new_value the new value for the setting, or null to simply
   *              retrieve the old value
   * @param boolean $send_notif should a notification be sent? (defaults to True)
   * @return mixed the old value of the setting
   */
  function _set($setting, $new_value, $type, $send_notif=True)
  {
    $old_value = $this->_db_to_native($this->_data[$setting], $type);

    if ($old_value != $new_value) {
      // change value
      $db_value = $this->_native_to_db($new_value, $type);
      $result = $_ENV['dbi']->extended->autoExecute("issue_groups",
        array($setting => $db_value),
        MDB2_AUTOQUERY_UPDATE,
        sprintf("issueid=%s and gid=%s", $_ENV['dbi']->quote($this->_issueid, 'integer'), $_ENV['dbi']->quote($this->gid(), 'integer')),
        array($this->_db_type($type))); 
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

      if ($send_notif) {
        $_ENV['notif']->record_settings_change($_SESSION['userid'],
              $this->_issueid,
              "{$this->group_name}/$setting",
              $this->_native_to_human($old_value, $type),
              $this->_native_to_human($new_value, $type));
      }
    }

    return $old_value;
  }
  /* }}} */

}
/* }}} */
?>
