<?php

if (eregi(basename(__FILE__),$_SERVER['PHP_SELF'])) {
  print "Direct module access forbidden.";
  exit;
}


////
// Object access

/* {{{ Function: get_issue */
/**
 * Get the issue object for the given issueid.
 *
 * @param integer $issueid
 * @return object the Issue object (see modules/issues/hooks/class.php)
 */
function get_issue($issueid, $cached_ok = true)
{
  static $issue_cache = array();

  if ( !$cached_ok || !array_key_exists($issueid, $issue_cache)) {
    $sql  = "SELECT primary_gid, opened_by, opened, closed, modified, summary, problem, status, category, ";
    $sql .= "       product, severity, private, istatus, iseverity, due_date, real_opened, real_closed, ";
    $sql .= "       modification_note, adjusted_by ";
    $sql .= "FROM issues ";
    $sql .= "WHERE issueid=? ";

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($issueid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $data = $result->fetchRow(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();

    if (!$data) { return null; }
    $issue_cache[$issueid] =& new Issue($issueid, $data);
  }

  return $issue_cache[$issueid];
}
/* }}} */

/* {{{ Function: add_new_issue */
/**
 * Add a new issue to the system
 *
 * @param integer $userid the userid responsible for the addition (null for anonymous)
 * @param integer $gid the group id for this new issue
 * @param integer $summary the title of the issue
 * @param integer $problem the "problem" of the issue
 * @return object the Issue object (see modules/issues/hooks/class.php), or FALSE if an error occurred
 */
function add_new_issue($userid, $gid, $summary, $problem, $severity)
{
  // Get the 'Registered' status
  list($registered) = fetch_status(TYPE_REGISTERED);

  $insert = array();
  $types = array();
  $insert['primary_gid'] = $gid;
  $types[] = 'integer';
  $insert['status'] = $registered;
  $types[] = 'integer';
  $insert['opened_by'] = $userid;
  $types[] = 'integer';
  $insert['opened'] = time();
  $types[] = 'integer';
  $insert['modified'] = time();
  $types[] = 'integer';
  $insert['summary'] = $summary;
  $types[] = 'text';
  $insert['problem'] = $problem;
  $types[] = 'text';
  $insert['severity'] = $severity;
  $types[] = 'integer';

  $result = $_ENV['dbi']->extended->autoExecute("issues",
    $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  
  $issueid = dbi_last_insert_id_fix("issues", "issueid");

  if (empty($issueid)) {
    $_ENV['error']->logger("Could not add new issue '$summary'", "phperrors");
    return false;
  }

  $insert = array();
  $types = array();
  $insert['issueid'] = $issueid;
  $types[] = 'integer';
  $insert['gid'] = $gid;
  $types[] = 'integer';
  $insert['opened'] = time();
  $types[] = 'integer';
  $result = $_ENV['dbi']->extended->autoExecute("issue_groups",
    $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

  issue_log($issueid,_("Issue Registered"),FALSE,$userid);

  /* record this for notification */
  $_ENV['notif']->record_new_issue($userid, $gid, $issueid, $summary, $problem);

  /* Create a new Issue object; it may reload the issue data from the database, but
   * that's OK -- it means it will pick up any default values specified in the schema */
  $issue = &get_issue($issueid);
  if (!$issue) {
    $_ENV['error']->fatal_error("Could not create new issue");
  }

  if ($userid) {
    $issue->add_client($userid);
  }

  return $issue;
}
/* }}} */

////
// Legacy glue (functions that utilize the new class interface)

/* {{{ Function: show_issue */
/**
 * Determine if a issue should be shown in a group
 *
 * @param integer $issueid ID of issue
 * @param integer $gid ID of group
 * @return boolean
 */
function show_issue($issueid,$gid)
{
  if (!$issueid || !$gid)
    return FALSE;

  $issue = &get_issue($issueid);
  $issuegroup = $issue->group($gid);
  if ($issuegroup) {
    return $issuegroup->show_issue();
  }

  return FALSE;
}
/* }}} */

////
// Legacy functions (need to be deciphered and ported to the object system)

/* {{{ Function: issue_groups */
/**
 * Retrieve the groups that are allowed to see an issue
 *
 * @param integer $issueid ID of issue
 * @return array a list of array('gid' => gid, 'name' => name)
 */
//TODO: rename this function
function issue_groups($issueid)
{
  $groups = array();

  $sql  = "SELECT i.gid,g.name ";
  $sql .= "FROM issue_groups i, groups g ";
  $sql .= "WHERE i.issueid=? ";
  $sql .= "AND i.gid=g.gid ";
  $sql .= "AND i.show_issue='t' ";
  $sql .= "ORDER BY g.name";
  $stmt = $_ENV['dbi']->prepare($sql);
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($issueid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $groups = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
  $result->free();
  $stmt->free();
  if (!$groups) { $groups = array(); }
  
  // Used by new issue visibility function.
  // Dave Joyce - 7/27/2004
  $gids = array();
  foreach($groups as $group){
  	$gids[] = $group['gid'];
  }
  if ($gids) {
    $gid = implode(",", $gids);
    
    $sql  = "SELECT v.viewer_gid AS gid,g.name ";
    $sql .= "FROM group_issue_visibility AS v ";
    $sql .= "JOIN groups AS g ON g.gid=v.viewer_gid ";
    $sql .= "WHERE v.owner_gid IN ($gid)"; // safe -- generated above
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute();
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
	  while ($row = $result->fetchRow(MDB2_FETCHMODE_ASSOC)) {
	    $groups[] = array('gid' => $row['gid'], 'name' => $row['name']);
    }
    $result->free();
    $stmt->free();
  }

  return $groups;
}
/* }}} */

/* {{{ Function: issue_group_field_value */
/**
 * Retrieve the current value of group-defined field for issue
 *
 * @param integer $gfid ID of group-defined field
 * @param integer $issueid ID of issue
 * @return mixed
 */
function issue_group_field_value($gfid,$issueid)
{
	//Check for value.
  $stmt = $_ENV['dbi']->prepare("SELECT COUNT(*) FROM issue_group_fields WHERE issueid=? AND gfid=?");
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($issueid, $gfid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
	$count = $result->fetchOne(); 
  $result->free();
  $stmt->free();

	if(is_null($count) || $count == 0){
		return null;
	}
	
	//Check type.
  $stmt = $_ENV['dbi']->prepare("SELECT case when value_idx is null then 0 else 1 end as has_idx
    from issue_group_fields where issueid=? AND gfid=?");
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($issueid, $gfid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
	$has_idx = $result->fetchOne(); 
  $result->free();
  $stmt->free();
	$field = ($has_idx == 1) ? "value_idx" : "value";
	
  $stmt = $_ENV['dbi']->prepare("SELECT $field FROM issue_group_fields WHERE issueid=? AND gfid=?");
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($issueid, $gfid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
	$value = $result->fetchOne(); 
  $result->free();
  $stmt->free();

	return $value;
}
/* }}} */

/* {{{ Function: can_view_issue */
/**
 * Determine if a user can see an issue
 *
 * @param integer $issueid ID of issue
 * @param integer $userid ID of user
 * @return boolean
 */
function can_view_issue($issueid,$userid = null)
{

  if (is_manager()) {
  	return TRUE;
  }

  if (is_null($userid)) {
    $user_groups = $_SESSION['groups'];
  } else {
    $user = &get_user($userid);
    $user_groups = $user->groups();
  }

  $groups = issue_groups($issueid);

  foreach ($groups as $group) {
    if (in_array($group['gid'],$user_groups)) {
      return TRUE;
	}
  }
}
/* }}} */

/* {{{ Function: issue_priv */
/**
 * Determine if a user has a privilege on a issue
 *
 * @param integer $issueid ID of issue
 * @param string $perm Privilege to check against
 * @param integer $userid ID of user to check *BASTARDS*
 * @return boolean
 */
function issue_priv($issueid,$perm,$userid = null)
{

	if (empty($userid)) {
		$userid = $_SESSION['userid'];
	}

	if(is_manager($userid)){
		return TRUE;
	}

	$groups = issue_groups($issueid);

	foreach ($groups as $group) {
		if (permission_check($perm,$group['gid'],$userid)) {
			return TRUE;	
		}
	}

	return FALSE;
}
/* }}} */

/* {{{ Function: num_new_issues */
/**
 * Pull number of new issues for a group
 *
 * @param integer $gid ID of group to check
 * @return integer
 */
function num_new_issues($gid)
{

  list($registered) = fetch_status(TYPE_REGISTERED);

	$sql  = "SELECT COUNT(i.issueid) ";
	$sql .= "FROM issues i, issue_groups g ";
	$sql .= "WHERE i.issueid=g.issueid ";
	$sql .= "AND g.gid=? ";
	$sql .= !permission_check("view_private",$gid) ? " AND i.private !='t'" : "";
	$sql .= "AND i.status=?";
  $stmt = $_ENV['dbi']->prepare($sql);
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($gid, $registered));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $new = $result->fetchOne(); 
  $result->free();
  $stmt->free();
  return $new;
}
/* }}} */

/* {{{ Function: num_open_issues */
/**
 * Pull number of open issues for this group
 *
 * @param integer $gid ID of group to check
 * @return integer
 */
function num_open_issues($gid)
{

  $closed = implode(",",fetch_status(array(TYPE_CLOSED,TYPE_AUTO_CLOSED)));

	$sql  = "SELECT COUNT(i.issueid) ";
	$sql .= "FROM issues i, issue_groups g ";
	$sql .= "WHERE i.issueid=g.issueid ";
	$sql .= "AND g.gid=? ";
	$sql .= !permission_check("view_private",$gid) ? "AND i.private !='t' " : "";
  if ($closed) $sql .= "AND i.status NOT IN ($closed) ";
  $sql .= "AND g.show_issue='t' ";

  $stmt = $_ENV['dbi']->prepare($sql);
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $open = $result->fetchOne(); 
  $result->free();
  $stmt->free();

  return $open;
}
/* }}} */

/* {{{ Funciton: last_activity */
/**
 * Find out when last action was for this group
 *
 * @param integer $gid ID of group
 * @return array
 */
function last_activity($gid)
{

  $sql  = "SELECT e.performed_on,e.userid ";
  $sql .= "FROM issues i, issue_groups g, events e ";
  $sql .= "WHERE i.issueid=g.issueid ";
  $sql .= "AND i.issueid=e.issueid ";
  $sql .= "AND g.gid=? ";
  $sql .= "ORDER BY e.performed_on DESC ";
  $sql .= "LIMIT 1";
  $stmt = $_ENV['dbi']->prepare($sql);
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $data = $result->fetchRow(MDB2_FETCHMODE_ASSOC); 
  $result->free();
  $stmt->free();
  if (!is_null($data)) {
    $last = array(
      "date" => $_ENV['i18n']->format_datetime($data['performed_on']),
      "user" => username($data['userid'])
    );

    return $last;
  }

  return null;
}
/* }}} */

/* {{{ Function: closed */
/**
 * Determines if a issue it closed or not
 *
 * @param integer $issueid ID of issue to check
 * @return boolean
 */
function closed($issueid)
{

  $stmt = $_ENV['dbi']->prepare("SELECT status from issues where issueid=?");
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($issueid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $status = $result->fetchOne(); 
  $result->free();
  $stmt->free();

  $closed = fetch_status(array(TYPE_CLOSED,TYPE_AUTO_CLOSED));
  if (in_array($status,$closed)) {
    return TRUE;
  }

  return FALSE;
}
/* }}} */

/* {{{ Function: issue_members */
/**
 * Retrieve array of all users in issue groups
 *
 * @param integer $issueid ID of issue
 * @return array
 */ 
function issue_members($issueid)
{

	$groups = issue_groups($issueid);
	$issue_members = array();

	foreach ($groups as $group) {
	  $members = group_members($group['gid']);

		foreach ($members as $key => $val) {
      if (array_key_exists($key,$issue_members)) {
        continue;
      }
      $issue_members[$key] = $val;
	  }
	}

  return $issue_members;
}
/* }}} */

/* {{{ Function: event_modify_time */
/**
 * Get the modification time of an event
 *
 * @param integer $eid ID of event
 * @return array
 */
function event_modify_time($eid)
{

  $sql  = "SELECT modified,userid ";
  $sql .= "FROM event_modifications ";
  $sql .= "WHERE eid=? ";
  $sql .= "ORDER BY modified DESC ";
  $sql .= "LIMIT 1";
  $stmt = $_ENV['dbi']->prepare($sql);
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($eid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
  $data = $result->fetchRow(MDB2_FETCHMODE_ASSOC); 
  $result->free();
  $stmt->free();
  if (!is_null($data)) {
    $modify = array(
      "time" => $_ENV['i18n']->format_datetime($data['modified']),
      "user" => username($data['userid'])
    );

    return $modify;
  }

  return null;
}
/* }}} */

/* {{{ Function: issue_escalation_groups */
/**
 * Retrieve all escalation groups for a issue
 *
 * @param integer $issueid ID of issue
 * @return array
 */
function issue_escalation_groups($issueid)
{

	$egroups = array();
	$groups = issue_groups($issueid);

	foreach ($groups as $group) {
		$e = escalation_groups($group['gid']);

		foreach ($e as $gid => $name) {
      if (!in_array($gid,$egroups)) {
        $egroups[$gid] = $name;
			}
		}
	}

	return $egroups;
}
/* }}} */

/* {{{ Function: issue_statuses */
/**
 * Retrieve array of all statuses for a issue
 * 
 * @param integer $issueid ID of issue
 * @return array
 */
function issue_statuses($issueid)
{

	$statuses = array();
	$groups = issue_groups($issueid);

	foreach ($groups as $group) {
		$gstatuses = group_statuses($group['gid']);

		foreach ($gstatuses as $key => $val) {
      if (!array_key_exists($key,$statuses)) {
        $statuses[$key] = $val;
      }
		}
	}

	return $statuses;
}
/* }}} */

/* {{{ Function: issue_istatuses */
/**
 * Retrieve array of all internal statuses for a issue
 * 
 * @param integer $issueid ID of issue
 * @return array
 */
function issue_istatuses($issueid)
{

	$statuses = array();
	$groups = issue_groups($issueid);

	foreach ($groups as $group) {
		$gstatuses = group_istatuses($group['gid']);

		foreach ($gstatuses as $key => $val) {
      if (!array_key_exists($key,$statuses)) {
        $statuses[$key] = $val;
      }
		}
	}

	return $statuses;
}
/* }}} */

/* {{{ Function: issue_categories */
/**
 * Retrieve array of all categories for a issue
 * 
 * @param integer $issueid ID of issue
 * @return array
 */
function issue_categories($issueid)
{

	$categories = array();
	$groups = issue_groups($issueid);

	foreach ($groups as $group) {
		$gcategories = group_categories($group['gid']);

	foreach ($gcategories as $key => $val) {
      if (!array_key_exists($key,$categories)) {
				$categories[$key] = $val;
			}
		}
	}

	return $categories;
}
/* }}} */

/* {{{ Function: issue_products */
/**
 * Retrieve array of all products for a issue
 * 
 * @author David Joyce <david.joyce@timco.aero>
 * @param integer $issueid ID of issue
 * @return array
 */
function issue_products($issueid)
{

	$products = array();
	$groups = issue_groups($issueid);

	foreach ($groups as $group) {
		$gproducts = group_products($group['gid']);

		foreach ($gproducts as $key => $val) {
      			if (!array_key_exists($key,$products)) {
				$products[$key] = $val;
			}
		}
	}

	return $products;
}
/* }}} */

/* {{{ Function: issue_log */
/**
 * Adds an entry to the logs for an issue
 *
 * @param integer $issueid ID of issue
 * @param string $msg Message to log
 * @param integer $userid ID of user
 */
function issue_log($issueid,$msg,$private = null,$userid = null)
{

  if (empty($msg) or empty($issueid)) {
    $_ENV['error']->logger("issue_log() called with missing msg or issueid", 'system_errors' );
  	return;
  }

  $insert = array();
  $types = array();
  $insert['issueid'] = $issueid;
  $types[] = 'integer';
  $insert['logged'] = time();
  $types[] = 'integer';
  $insert['userid'] = !is_null($userid) ? $userid : $_SESSION['userid'];
  $types[] = 'integer';
  $insert['message'] = $msg;
  $types[] = 'text';
  $insert['private'] = $private === TRUE ? "t" : "f";
  $types[] = 'text';

  // userid is required.  
  if (empty($insert['userid'])) {
    $_ENV['error']->logger( "issue_log() called with missing userid, and no userid found in session.", 'system_errors' );
    return;
  }

  $result = $_ENV['dbi']->extended->autoExecute("issue_log",
    $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
}
/* }}} */

/* {{{ Function: update_view_tracking */
/**
 * Update the viewed time on an issue for the current user
 *
 * @param integer $issueid ID of issue
 */
function update_view_tracking($issueid)
{

  $stmt = $_ENV['dbi']->prepare("SELECT viewed from view_tracking where issueid=? and userid=?");
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($issueid, $_SESSION['userid']));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $viewed = $result->fetchOne(); 
  $result->free();
  $stmt->free();

  if (!is_null($viewed)) {
    $result = $_ENV['dbi']->extended->autoExecute("view_tracking",
      array("viewed" => time()),
      MDB2_AUTOQUERY_UPDATE,
      sprintf("issueid=%s and userid=%s", 
            $_ENV['dbi']->quote($issueid, 'integer'), 
            $_ENV['dbi']->quote($_SESSION['userid'], 'integer')),
      array('integer')); 
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  } else {
    $insert = array();
    $types = array();
    $insert['issueid']= $issueid;
    $types[] = 'integer';
    $insert['userid'] = $_SESSION['userid'];
    $types[] = 'integer';
    $insert['viewed'] = time();
    $types[] = 'integer';
    $result = $_ENV['dbi']->extended->autoExecute("view_tracking",
      $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  }
}
/* }}} */

/* {{{ Function: unread_events */
/**
 * Retrieve the number of events in an issue that given users has not seen
 *
 * @param integer $issueid ID of issue
 * @param integer $userid ID of user
 * @param boolean $private Count private or not
 * @return integer
 */
function unread_events($issueid,$userid,$private = FALSE)
{

  $stmt = $_ENV['dbi']->prepare("SELECT viewed from view_tracking where issueid=? and userid=?");
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($issueid, $userid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $viewed = $result->fetchOne(); 
  $result->free();
  $stmt->free();

  if (is_null($viewed)) {
    $viewed = 0;
  }

  $stmt = $_ENV['dbi']->prepare("SELECT count(eid) from events where issueid=? and performed_on > ?"
      . (!$private ? " AND private != 't' " : ""));
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($issueid, $viewed));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $count = $result->fetchOne(); 
  $result->free();
  $stmt->free();

  return $count;
}
/* }}} */

/* {{{ Function: issue_exists */
/**
 * Checks to see if an issue exists
 *
 * @param integer $issueid Id of issue
 * @return boolean
 */
function issue_exists($issueid)
{

  $stmt = $_ENV['dbi']->prepare("SELECT issueid from issues where issueid=?");
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($issueid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $id = $result->fetchOne(); 
  $result->free();
  $stmt->free();

  if (!is_null($id)) {
    return TRUE;
  }

  return FALSE;
}
/* }}} */

/* {{{ Function: issue_escalated_to */
/** 
 * Determines if given issue was escalated to this group
 *
 * @param integer $issueid ID of issue
 * @param integer $gid ID of group
 * @return boolean
 */
function issue_escalated_to($issueid,$gid)
{

  $stmt = $_ENV['dbi']->prepare("SELECT primary_gid from issues where issueid=?");
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($issueid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $igid = $result->fetchOne(); 
  $result->free();
  $stmt->free();

  if (!empty($igid)) {
    if ($gid != $igid) {
      return TRUE;
    }
  }

  return FALSE;
}
/* }}} */

/* {{{ Function: issue_escalated_from */
/** 
 * Determines if given issue was escalated from this group
 *
 * @param integer $issueid ID of issue
 * @param integer $gid ID of group
 * @return boolean
 */
function issue_escalated_from($issueid,$gid)
{

  $stmt = $_ENV['dbi']->prepare("SELECT primary_gid from issues where issueid=?");
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($issueid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $igid = $result->fetchOne(); 
  $result->free();
  $stmt->free();

  if (!empty($igid)) {
    if ($gid == $igid) {
      $stmt = $_ENV['dbi']->prepare("SELECT count(gid) from issue_groups where issueid=?");
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute(array($issueid));
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
      $count = $result->fetchOne(); 
      $result->free();
      $stmt->free();
      if ($count > 1) {
        return TRUE;
      }
    }
  }

  return FALSE;
}
/* }}} */

/* {{{ Function: search_issues */
/**
 * Retrieve array of issues by search parameters based on user prefs.
 *
 * @author Dave Joyce <david.joyce@timco.aero>
 * @return array
 */
function search_issues()
{

	if (empty($_GET['sort'])) {
		if ($_SESSION['prefs']['sort_by'] == 'g_name'){
			$_GET['sort'] = 'primary_gid';
		}else{
			$_GET['sort'] = $_SESSION['prefs']['sort_by'];
		}
	}

	$issues = array();

  $params = array();
	$sql  = "SELECT DISTINCT i.issueid,i.primary_gid,i.summary ";

	foreach ($_SESSION['prefs']['show_fields'] as $val) {
		switch ($val) {
      case "flags":
        continue;
        break;
			case "assigned_to":
				$sql .= ",g.assigned_to";
				break;
			case "opened_by":
				$sql .= ",u.username";
				break;
			case "time_open":
				$sql .= ",i.opened";
				break;
			default:
        $val = preg_replace('/[^a-zA-Z0-9]/', '_', $val);
				$sql .= ",i.$val"; // sanitized
				break;
		}
	}

	$sql .= " FROM issues i 
			LEFT JOIN issue_groups g USING(issueid) 
			LEFT JOIN events e USING (issueid) ";
	$sql .= in_array("opened_by",$_SESSION['prefs']['show_fields']) ? "LEFT JOIN users AS u ON u.userid=i.opened_by " : "";

	//Added 7/13/04: Add group-defined fields as search criteria.
	$sql .= isset($_SESSION['custom_fields']) ? "LEFT JOIN issue_group_fields AS ig ON ig.issueid=i.issueid " : "";

  $where_clause = array();
  if (isset($_POST['criteria']) && $_POST['criteria'] != '') {
    $wp  = "(LOWER(i.problem) LIKE LOWER(?) ";
    $params[] = '%' . $_POST['criteria'] . '%';
    $wp .= "OR LOWER(i.summary) LIKE LOWER(?) ";
    $params[] = '%' . $_POST['criteria'] . '%';
    $wp .= "OR LOWER(e.action) LIKE LOWER(?))";
    $params[] = '%' . $_POST['criteria'] . '%';
    $where_clause[] = $wp;
  }

  // Get a list of the options that are negated
  if (array_key_exists('negate', $_POST)) {
    $negate_list = $_POST['negate'];
  } else {
    $negate_list = array();
  }
	
	//Added 7/13/04: Search parameter negation.
  if (is_array($_POST['groups'])) {
    $list = array();
    foreach ($_POST['groups'] as $id) { if ($id) $list[] = $id+0; } // sanitize
    if (sizeof($list) > 0) {
      $list = implode(",", $list);
      $wp = in_array("groups", $negate_list)
          ? "g.gid NOT IN ($list) "
          : "g.gid IN ($list) ";
      $where_clause[] = $wp;
    }
  }
	
  if (is_array($_POST['opened'])) {
    $list = array();
    foreach ($_POST['opened'] as $id) { $list[] = $id+0; } // sanitize
    $list = implode(",", $list);
	  $wp = in_array("opened", $negate_list)
			  ? "i.opened_by NOT IN ($list) "
			  : "i.opened_by IN ($list) ";
    $where_clause[] = $wp;
  }
	
  if (is_array($_POST['status'])) {
    $list = array();
    foreach ($_POST['status'] as $id) { $list[] = $id+0; } // sanitize
    $list = implode(",", $list);
	  $wp = in_array("status", $negate_list)
			  ? "i.status NOT IN ($list) "
			  : "i.status IN ($list) ";
    $where_clause[] = $wp;
  }
	
  if (is_array($_POST['category'])) {
    $list = array();
    foreach ($_POST['category'] as $id) { $list[] = $id+0; } // sanitize
    $list = implode(",", $list);
	  $wp = in_array("category", $negate_list)
			  ? "i.category NOT IN ($list) "
			  : "i.category IN ($list) ";
    $where_clause[] = $wp;
  }
	
  if (is_array($_POST['product'])) {
    $list = array();
    foreach ($_POST['product'] as $id) { $list[] = $id+0; } // sanitize
    $list = implode(",", $list);
	  $wp = in_array("product", $negate_list)
			  ? "i.product NOT IN ($list) "
			  : "i.product IN ($list) ";
    $where_clause[] = $wp;
  }
	
	if (!empty($_POST['opened_sdate']) && !empty($_POST['opened_edate'])) {
		$unixsdate = strtotime($_POST['opened_sdate']);
		$unixedate = strtotime($_POST['opened_edate']);
		
		$wp = " i.opened ";
		$wp .= in_array("opened_date", $negate_list) ? "NOT " : "";
		$wp .= "BETWEEN ? AND ? "; 
    $params[] = $unixsdate;
    $params[] = $unixedate;
    $where_clause[] = $wp;
	}
	
	if (!empty($_POST['closed_sdate']) && !empty($_POST['closed_edate'])) {
		$unixsdate = strtotime($_POST['closed_sdate']);
		$unixedate = strtotime($_POST['closed_edate']);
		
		$wp = " i.closed ";
		$wp .= in_array("closed_date", $negate_list) ? "NOT " : "";
		$wp .= "BETWEEN ? AND ? ";
    $params[] = $unixsdate;
    $params[] = $unixedate;
    $where_clause[] = $wp;
	}
	
	//Added 7/13/04: Add group-defined fields as search criteria.
	if (is_array($_SESSION['custom_fields']) && count($_POST) > 0) {
		$fields = array();
		foreach ($_SESSION['custom_fields'] as $gfid => $info) {
			if (array_key_exists($info['name'],$_POST) && !empty($_POST[$info['name']])) {
				$fields[] = $gfid;
				switch ($info['type']) {
				  case "select":
				  case "select multiple":
          if (is_array($_POST[$info['name']])) {
            $list = array();
            foreach ($_POST[$info['name']] as $id) { $list[] = $id+0; } // sanitize
            $list = implode(",",$list);
					  $q .= "AND ig.value_idx IN ($list) ";
          } else {
					  $q .= "AND ig.value_idx=". ($_POST[$info['name']]+0) . " "; // sanitize
          }
					break;
				  case "checkbox":
				    $q .= "AND ig.value='on' ";
				    break;
				  default:
				    $q .= "AND LOWER(ig.value) LIKE LOWER(?) ";
            $params[] = '%' . $_POST[$info['name']] . '%';
				    break;
				}
			}
		}
		if (count($fields) > 0) {
      $list = array();
      foreach ($fields as $id) { $list[] = $id+0; } // sanitize
      $list = implode(",", $list);
		  $wp = "ig.gfid IN ($list) ";
		  $wp .= $q;
		}
	}
      
  if (!is_admin()) {
      $permitted_groups = implode(",",$_SESSION['groups']);	  
      $wp = "g.gid in ($permitted_groups)";
      $where_clause[] = $wp;
  }


	
  // finish off the where clause
  if ($where_clause)
    $sql .= "WHERE " . implode(" AND ", $where_clause);
	
	if ($_GET['sort'] == "assigned_to") {
		$sql .= "ORDER BY g.assigned_to ";
	} else if ($_GET['sort'] == "opened_by") {
		$sql .= "ORDER BY u.username ";
	} else {
    $sort = preg_replace('/[^a-zA-Z0-9]/', '_', $_GET['sort']);
		$sql .= "ORDER BY i.$sort ";
	}

	if ($_GET['sort'] == "severity") {
		$sql .= $_GET['reverse'] == "true" ? "DESC" : "ASC";
	} else {
		$sql .= $_GET['reverse'] == "true" ? "ASC" : "DESC";
	}
    // for debugging
     //echo "<br />$sql<pre>"; var_dump($params); echo "</pre>";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
	$result = $stmt->execute($params);
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  while ($row = $result->fetchRow(MDB2_FETCHMODE_ASSOC)) {
    $issues[$row['issueid']] = $row;
  }
  $result->free();
  $stmt->free();
	
	return $issues;
}
/* }}} */ 

/* {{{ Function: severity_text */
/**
 * Text for Severities
 *
 * @param integer $severity Severity to display text for
 * @returns string
 */
function severity_text($severity)
{
  static $severities;
  
  // initialize on first call (after gettext is set up)
  if (!$severities) {
    $severities = array(
      SEV_URGENT => _("Urgent"),
      SEV_HIGH => _("High"),
      SEV_NORMAL => _("Normal"),
      SEV_LOW => _("Low")
    );
  }

  if (array_key_exists($severity, $severities)) {
    return $severities[$severity];
  } else {
    return $severities[SEV_LOW];
  }
}
/* }}} */

/* {{{ Function: status_type_text */
/**
 * Text for Statuses, used in notification
 *
 * @param integer $status Status to display text for
 * @returns string
 */
function status_type_text ($status) {
   global $status_types;

   $sql = "select status_type from statuses where sid = ?";
   $stmt = $_ENV['dbi']->prepare($sql);
   $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
   $result = $stmt->execute(array($status));
   $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
   $type = $result->fetchOne();
   $result->free();
   $stmt->free();

   switch ($type) {
	case TYPE_REGISTERED:
		$text = "NEW";
		break;
	case TYPE_WAITING:
		$text = "UPDATED";
		break;
	case TYPE_LONG_TERM:
		$text = "UPDATED";
		break;
	case TYPE_STALE:
		$text = "UPDATED";
		break;
	case TYPE_CLOSED:
		$text = "CLOSED";
		break;
	case TYPE_AUTO_CLOSED:
		$text = "CLOSED";
		break;
	default:
		$text = "UNKNOWN";
		break;
   }

   return $text;
}
/* }}} */
?>
