<?php
/**
 * @package EnterTrack
 * @subpackage Reports_Module
 */
if (strpos(basename(__FILE__),$_SERVER['PHP_SELF'])) {
  print "Direct module access forbidden.";
  exit;
}

ignore_user_abort(TRUE);
set_time_limit(_MINUTE_ * 10);

if (!is_group_admin($_POST['gid'])) {
  $_ENV['error']->push_error(_("You must be a group administrator to run this report"));
  redirect();
}

function getMemberDetail($gmembers, $startDate, $endDate, $gid)
{
    $member_detail = array();
    foreach ($gmembers as $uid => $username) {
        $params = array();
        $sql = "SELECT i.issueid,i.summary,i.opened,i.closed,i.real_opened,i.real_closed,i.modification_note
  	  		  		FROM issues AS i
  	  		  		JOIN issue_groups AS ig ON ig.issueid=i.issueid
  	  		  		JOIN users AS u ON u.userid=ig.assigned_to
  	  		  		WHERE i.closed > 0 
  	  		  		AND (i.real_opened IS NOT NULL OR i.real_closed IS NOT NULL) ";
        if (isset($startDate)) {
          $sql .= "AND (i.closed BETWEEN ? AND ?) ";
          $params[] = $startDate;
          $params[] = $endDate;
        }
	$sql .= "AND u.userid=? AND ig.gid = ? "; 
	$sql .= "ORDER BY i.issueid";
	$params[] = $uid;
	$params[] = $gid;
        $stmt = $_ENV['dbi']->prepare($sql);
        $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
        $res = $stmt->execute($params);
        $result = $res->fetchAll(MDB2_FETCHMODE_ASSOC); 
        $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
        $res->free();
        $stmt->free();
        if (count($result) > 0) {
            $member_detail[$uid] = array();
            foreach ($result as $key => $row) {
                $member_detail[$uid][$key] = array();
                $member_detail[$uid][$key]['issueid'] = $row['issueid'];
                $member_detail[$uid][$key]['summary'] = $row['summary'];
                $member_detail[$uid][$key]['sys_opened'] = date("m-d-Y H:i:s", $row['opened']);
                if (!is_null($row['real_opened'])) {
                    $member_detail[$uid][$key]['adj_opened'] = date("m-d-Y H:i:s", $row['real_opened']);
                    $member_detail[$uid][$key]['opened_delta'] = time_diff($row['real_opened'],
                    $row['opened']);
                }
                $member_detail[$uid][$key]['sys_closed'] = date("m-d-Y h:i:s", $row['closed']);
                if (!is_null($row['real_closed'])) {
                    $member_detail[$uid][$key]['adj_closed'] = date("m-d-Y h:i:s", $row['real_closed']);
                    $member_detail[$uid][$key]['closed_delta'] = time_diff($row['real_closed'],
                    $row['closed']);
                }
                $member_detail[$uid][$key]['formatted'] =
                    sprintf("<tr><td>%d</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td></tr>\n" .
		    "<tr><td colspan=\"6\"><b>"._("Justification").":</b> %s\n",
                    $row['issueid'],
                    substr($row['summary'],0,32),
                    date("m-d-Y H:i:s", $row['opened']),
                    !is_null($row['real_opened']) ? date("m-d-Y H:i:s", $row['real_opened']) : " ",
                    date("m-d-Y H:i:s", $row['closed']),
                    !is_null($row['real_closed']) ? date("m-d-Y H:i:s", $row['real_closed']) : " ",
                    substr($row['modification_note'],0,75));
                if ($row['real_opened']) {
                    $member_detail[$uid][$key]['formatted'] .=
                    "<b>"._("Opened Delta").":</b>{$member_detail[$uid][$key]['opened_delta']}\n";
		}
                if ($row['real_closed']) {
                    $member_detail[$uid][$key]['formatted'] .=
                    "<b>"._("Closed Delta").":</b>{$member_detail[$uid][$key]['closed_delta']}\n";
		}
		$member_detail[$uid][$key]['formatted'] .= "</td></tr>";


            }
        }
    }

    return $member_detail;
}

function getMemberSummary($gmembers, $startDate, $endDate, $gid)
{
    $member_summary = array();
    foreach ($gmembers as $uid => $username) {
        $params = array();
        $sql = "SELECT COUNT(i.issueid) AS total_modified
	    			FROM issues AS i
	    			JOIN issue_groups AS ig ON ig.issueid=i.issueid 
	    			WHERE i.closed > 0
	    	  		AND (i.real_opened IS NOT NULL OR i.real_closed IS NOT NULL) ";
        if (isset($startDate)) {
          $sql .= "AND (i.closed BETWEEN ? AND ?) ";
          $params[] = $startDate;
          $params[] = $endData;
        }
        $sql .="AND ig.assigned_to=? AND ig.gid =?";
	$params[] = $uid;
	$params[] = $gid;

	$stmt = $_ENV['dbi']->prepare($sql);
	$_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
        $result = $stmt->execute($params);
        $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
        $modified = $result->fetchOne(); 
        $result->free();
        $stmt->free();

	unset($params);
	
	$sql = "SELECT COUNT(i.issueid) AS total_modified
	    			FROM issues AS i
	    			JOIN issue_groups AS ig ON ig.issueid=i.issueid 
	    			WHERE i.closed > 0 ";
        if (isset($startDate)) {
          $sql .= "AND (i.closed BETWEEN ? AND ?) ";
          $params[] = $startDate;
          $params[] = $endData;
        }
        $sql .="AND ig.assigned_to=? AND ig.gid=?";
	$params[] = $uid;
	$params[] = $gid;

	// for debugging
	//echo "<br />$sql<pre>"; var_dump($params); echo "</pre>";
	
	$stmt = $_ENV['dbi']->prepare($sql);
        $result = $stmt->execute($params);
        $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
        $closed = $result->fetchOne(); 
        $result->free();
        $stmt->free();

        if ($closed > 0) {
            $member_summary[$uid] = array();
            $member_summary[$uid]['closed']   = $closed;
            $member_summary[$uid]['modified'] = $modified;
            $member_summary[$uid]['percent']  = ($modified/$closed) * 100;
            $member_summary[$uid]['formatted'] = sprintf("<tr><td>%s</td><td>%d</td><td>%d</td><td>%.1f%%</td></tr>",
            $username,
            $closed,
            $modified,
            ($modified/$closed) * 100);
        }
    }

    return $member_summary;
}

$startDate = null;
if (!empty($_POST['sdate'])) {
  $startDate = $_ENV['i18n']->parse_datetime($_POST['sdate']);
  if (!$startDate) {
    $_ENV['error']->push_error(_("Invalid start date."));
  }
}

$endDate = null;
if (!empty($_POST['edate'])) {
  $endDate = $_ENV['i18n']->parse_datetime($_POST['edate']);
  if (!$endDate) {
    $_ENV['error']->push_error(_("Invalid end date."));
  }
}

// default end to current time
if ($startDate && !$endDate) {
  $endDate = time();
}

if (!$_ENV['error']->errors_pending()) {
	$gid		= $_POST['gid'];
	$gmembers	= group_members($gid);
	$tplData	= array();

	switch ($_POST['report_type']) {
	  case "detail":
		$member_detail = getMemberDetail($gmembers, $startDate, $endDate, $gid);	       
		$tplData['title'] =  group_name($gid);
		$tplData['memberDetail'] = $member_detail;
		$tplData['count'] = count($member_detail);
		$tpl = 'reports/group_modified_time_report_detail.tpl';
		break;
	  case "full":
		$member_detail = getMemberDetail($gmembers, $startDate, $endDate, $gid);		
	    $member_summary = getMemberSummary($gmembers, $startDate, $endDate, $gid);
		$tplData['title'] =  group_name($gid);
		$tplData['memberSummary'] = $member_summary;
		$tplData['memberDetail'] = $member_detail;
		$tplData['count'] = count($member_detail);
		$tpl = 'reports/group_modified_time_report.tpl';
		break;
	  case "summary":
	    $member_summary = getMemberSummary($gmembers, $startDate, $endDate, $gid);
		$tplData['title'] =  group_name($gid);
		$tplData['memberSummary'] = $member_summary;
		$tplData['count'] = count($member_summary);
		$tpl = 'reports/group_modified_time_report_summary.tpl';
		break;
	}
	if (isset($startDate)) {
	  $txtStartDate = $_ENV['i18n']->format_datetime($startDate);
	  $txtEndDate = $_ENV['i18n']->format_datetime($endDate);
	  $tplData['title'] .= " ($txtStartDate "._("to")." $txtEndDate)";
	}
	$tplData['time'] = time();
	$tplData['user'] = $_SESSION['userid'];
	$_ENV['tpl']->assign("report", $tplData);
	$_ENV['tpl']->display($tpl);
} else {
	redirect();
}
?>
