<?php
/* $Id: tickets_by_category.reports.php 401 2008-05-17 18:51:47Z danda $ */
/**
 * @package EnterTrack
 * @subpackage Reports_Module
 */

if (strpos(basename(__FILE__),$_SERVER['PHP_SELF'])) {
  print "Direct module access forbidden.";
  exit;
}

$issue_fields = array(
    "issueid"   => "Issue Id",
    
    "summary"   => "Summary",
    "category"  => "Category",
    "product"   => "Product",
    "status"    => "Status",
    "severity"  => "Severity",
    "opened"    => "Date Opened"
);

$sort_options = array(
    "ascending"                 => "Ascending",
    "descending"                => "Descending",
);

$export_formats = array(
    "csv"   => "Comma Separated Value (.csv)",
    "xls"   => "Excel Spreadsheet (.xls)"
);

// Get the groups accessible to the user
if (is_manager()) {
  $sql  = "SELECT gid ";
  $sql .= "FROM groups ";
  $sql .= "where active='t' ";
  $sql .= "ORDER BY name";
  $stmt = $_ENV['dbi']->prepare("SELECt gid from groups where active='t' order by name");
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute();
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
  $groups = $result->fetchCol(); 
  $result->free();
  $stmt->free();
} else {
  $groups = $_SESSION['groups'];  
}

// add visible groups to possible reports
foreach ($groups as $key=>$gid) 
{
    $stmt = $_ENV['dbi']->prepare("SELECT owner_gid from group_issue_visibility where viewer_gid=?");
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $visible_groups = $result->fetchCol(); 
    $result->free();
    $stmt->free();

    if ($visible_groups) {
      $groups = array_merge($groups, $visible_groups);   
    }
}

$groups = array_unique($groups);

// set the start and end dates
$startDate = null;
if (!empty($_POST['sdate'])) {
  $startDate = $_ENV['i18n']->parse_datetime($_POST['sdate']);
  if (!$startDate) {
    $_ENV['error']->push_error(_("Invalid start date."));
  }
}

$endDate = null;
if (!empty($_POST['edate'])) {
  $endDate = $_ENV['i18n']->parse_datetime($_POST['edate']);
  if (!$endDate) {
    $_ENV['error']->push_error(_("Invalid end date."));
  }
}

// default end to current time
if ($startDate && !$endDate) {
  $endDate = time();
}

if (isset($startDate)) {
    $report->set_date_range($startDate,$endDate);
}

/**
 * Get specific categories.
 * 
 * If the user is a manager, allow them to select from all
 * the categories; otherwise, we'll just display the categories for the user's
 * groups.
 */
if (is_manager()) {
    $stmt = $_ENV['dbi']->prepare("SELECT category from categories order by category");
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute();
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $categories = $result->fetchCol(); 
    $result->free();
    $stmt->free();
} else {
    $categories = array();
    
    // get the group categories
    foreach ($_SESSION['groups'] as $key => $gid) {
        $stmt = $_ENV['dbi']->prepare("SELECT cid from group_categories where gid=?");
        $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
        $result = $stmt->execute(array($gid));
        $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
        $c = $result->fetchCol(); 
        $result->free();
        $stmt->free();
    
        unset($sql);
    
        // now lookup the name for each category
        foreach ($c as $cid) {
            $sql    = "SELECT category ";
            $sql   .= "FROM categories ";
            $sql   .= "WHERE cid = $cid";
            $stmt = $_ENV['dbi']->prepare("SELECT category from categories where cid=?");
            $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
            $result = $stmt->execute(array($cid));
            $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
            $category = $result->fetchCol(); 
            $result->free();
            $stmt->free();
            if (!in_array($category[0], $categories)) {
              array_push($categories, $category[0]);
            }
        }
    }
}

if (is_manager()) {
  $sql  = "SELECT gid ";
  $sql .= "FROM groups ";
  $sql .= "where active='t' ";
  $sql .= "ORDER BY name";
  $stmt = $_ENV['dbi']->prepare("SELECT gid from groups where active='t' order by name");
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute();
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
  $groups = $result->fetchCol(); 
  $result->free();
  $stmt->free();
} else {
  $groups = $_SESSION['groups'];  
}

// add visible groups to possible reports
foreach ($groups as $key=>$gid) 
{
    $stmt = $_ENV['dbi']->prepare("SELECt owner_gid from group_issue_visibility where viewer_gid=?");
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $visible_groups = $result->fetchCol(); 
    $result->free();
    $stmt->free();

    if ($visible_groups) {
      $groups = array_merge($groups, $visible_groups);   
    }
}

$groups = array_unique($groups);

$_ENV['tpl']->assign('report', 'Issues by Category');
$_ENV['tpl']->assign('kind', 'issue');
$_ENV['tpl']->assign('type', 'category');
$_ENV['tpl']->assign('groups', $groups);
$_ENV['tpl']->assign('issue_fields', $issue_fields);
$_ENV['tpl']->assign('sort_options', $sort_options);
$_ENV['tpl']->assign('export_formats', $export_formats);
$_ENV['tpl']->assign('categories', $categories);
$_ENV['tpl']->display("reports/custom/group_issues_form.tpl");
?>
