<?php
/* $Id:$ */
/**
 * @package EnterTrack
 * @subpackage Reports_Module
 */
 
us_main();
exit;

/**
 * main().  Top level, performs all processing for user statistics report.
 * both form display and report generation.
 * 
 * The user stats report basically consists of all users that worked
 * (posted events) to issues in the selected groups during a given
 * time period.  The report shows (for each user) things like total 
 * hours worked,  average hours worked, and total events posted.
 * 
 */
function us_main() {

    if (strpos(basename(__FILE__),$_SERVER['PHP_SELF'])) {
      print "Direct module access forbidden.";
      exit;
    }

    $groups = get_groups_for_user();

    if (empty($groups)) {
        $_ENV['error']->push_error( _("You don't have permission to run 'User Statistics' report.") );
        redirect();
        exit;  
    }

    if ( count($_POST)) {
	    $data = run_report();
	    $totals = calc_totals( $data );
	    display_report( aget($_POST, 'format'), $data, $totals );
    } else {
    	display_form( $groups );
    }
}

/**
 * Verifies that all values in an array are numeric.
 * 
 */
function verify_numeric_array($array, $empty_allowed, $error_msg ) {
    if( !$empty_allowed && (!is_array( $array ) || !count( $array )) ) {
        print $error_msg;
        exit;
    }
    foreach( $array as $val ) {
        if( !is_numeric( $val ) ) {
            print $error_msg;
            exit;
        }
    }
}

/**
 * Finds groups that current user can generate reports for.
 * Must be system admin, group admin or group technician.
 * 
 */
function get_groups_for_user() {

    // Get user permissions, if not admin of any group redirect
    $groups = array();
    if( is_admin() ) {
        $stmt = $_ENV['dbi']->prepare("SELECT gid from groups where active='t' order by name");
        $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
        $result = $stmt->execute();
        $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
        $groups = $result->fetchCol(); 
        $result->free();
        $stmt->free();
    }
    else {
        foreach($_SESSION['groups'] as $group) {      
            if ( is_group_admin($group) || permission_check("technician",$group) ) {
                $groups[] = $group;  
            }
        }
    }

    return $groups;
}

/**
 * Generates a report.
 * 
 */
function run_report( ) {
    $gid_list = aget( $_POST, 'gid');
    verify_numeric_array( $gid_list, false, _('Invalid group selected.') );

    $groups_data = get_report_data( $gid_list, 
                                    aget( $_POST, 'sdate'), 
                                    aget( $_POST, 'edate') );
    return $groups_data;
}

/**
 * Report generation worker function.  Contains SQL logic.
 * 
 */
function get_report_data( $group_id_list, $start_date, $end_date ) {
    $params = array();

    $startDate = null;
    if ( $start_date ) {
      $startDate = $_ENV['i18n']->parse_datetime($start_date);
      if (!$startDate) {
        $_ENV['error']->push_error(_("Invalid start date."));
      }
    }
    
    $endDate = null;
    if ( $end_date ) {
      $endDate = $_ENV['i18n']->parse_datetime($end_date);
      if (!$endDate) {
        $_ENV['error']->push_error(_("Invalid end date."));
      }
    }

    $gid_list = implode( ',', $group_id_list );

    $time_now = time();

	// Verifies if set the start date and generate clauses 
    if ($startDate) {
    	$opened_start_clause = " and i.opened >= '$startDate'";
    	$events_start_clause = " and e.performed_on >= '$startDate'";
    	$closed_start_clause = " and i.closed >= '$startDate'";
    	 
    } else {
    	$opened_start_clause = '';
    	$events_start_clause = '';
    	$closed_start_clause = '';
    }
    // Verifies if set the end date and generate clauses
    if ($endDate) {
    	$opened_end_clause = " and i.opened <= '$endDate'";
    	$events_end_clause = " and e.performed_on <= '$endDate'";
    	$closed_end_clause = " and i.closed <= '$endDate'";
    } else {
    	$opened_end_clause = '';
    	$events_end_clause = '';
    	$closed_end_clause = '';
    }
	
    $sql = <<< END
select
 g.gid,
 g.name,
 (select count(*) from issue_groups ig, issues i where ig.gid = g.gid and ig.issueid = i.issueid $opened_start_clause $opened_end_clause ) as num_issues_opened,
 (select count(*) from issue_groups ig, issues i, events e where e.issueid = ig.issueid and ig.gid = g.gid and ig.issueid = i.issueid $events_start_clause $events_end_clause ) as num_events,
 (select coalesce(sum(e.duration),0) from issue_groups ig, issues i, events e where e.issueid = ig.issueid and ig.gid = g.gid and ig.issueid = i.issueid $events_start_clause $events_end_clause ) as sum_events,
 (select count(*) from issue_groups ig, issues i inner join statuses s on i.status = s.sid where ig.gid = g.gid and ig.issueid = i.issueid and s.status_type in (2) ) as num_waiting,
 (select count(*) from issue_groups ig, issues i inner join statuses s on i.status = s.sid where ig.gid = g.gid and ig.issueid = i.issueid and s.status_type in (3) ) as num_longterm,
 (select count(*) from issue_groups ig, issues i inner join statuses s on i.status = s.sid where ig.gid = g.gid and ig.issueid = i.issueid $closed_start_clause $closed_end_clause and s.status_type in (5,6) ) as num_closed,
 (select count(*) from issue_groups ig, issues i inner join statuses s on i.status = s.sid where ig.gid = g.gid and ig.issueid = i.issueid and s.status_type not in (5,6) ) as num_active,
 (select count(*) from issue_groups ig, issues i inner join statuses s on i.status = s.sid where ig.gid = g.gid and ig.issueid = i.issueid and i.due_date < $time_now and s.status_type not in (5,6)) as num_past_due_date
from 
 groups g
where
 g.gid in ($gid_list)
END;

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $data = $result->fetchAll(MDB2_FETCHMODE_ASSOC);
    $result->free();
    $stmt->free();

    return $data;
}

/**
 * Calculates totals for each column in an input array.
 * 
 */
function calc_totals( $data ) {
    $totals = array();
    foreach( $data as $group ) {
        foreach( $group as $key => $amt ) {
            $totals[$key] =(float)(isset( $totals[$key] ) ? $totals[$key] + $amt : $amt);
        }
    }
    return $totals;
}

/**
 * Displays the report form where user can choose report options.
 * 
 */
function display_form( $groups ) {
	
	$issue_fields = array(
		"issueid"   => "Issue Id",
		"category"  => "Category",
		"product"   => "Product",
		"status"    => "Status",
		"severity"  => "Severity",
		"opened"    => "Date Opened",
		"modified"  => "Date Modified",
		"closed"    => "Date Closed",
		"opened_by" => "Opened By",
		"assigned_to" => "Assigned to",
		"problem"   => "Subject",
	);
	
	$event_fields = array(
		"action"   => "Event Posted",
		"duration" => "Time Worked"
	);    
	
	$sort_options = array(
		"ascending"  => "Ascending",
		"descending" => "Descending",
	);
	
	$_ENV['tpl']->assign('report', 'Group Summary');
	$_ENV['tpl']->assign('groups', $groups);
	$_ENV['tpl']->assign('issue_fields', $issue_fields);
	$_ENV['tpl']->assign('event_fields', $event_fields);
	$_ENV['tpl']->assign('sort_options', $sort_options);
	$_ENV['tpl']->display("reports/group_summary.tpl");
}

/**
 * Displays report in various formats
 * 
 */
function display_report( $format, $data, $totals ) {

    switch( trim($format) ) {
    case 'html': display_report_html( $data, $totals ); break;
    case 'csv': display_report_csv( $data, $totals ); break;
    case 'word': display_report_word( $data, $totals ); break;
    case 'excel': display_report_excel( $data, $totals ); break;
    case 'pdf': display_report_pdf( $data, $totals ); break;
    default: 
        $_ENV['error']->push_error(_("Unknown report format."));
        break;
    }
}


/**
 * Displays report in HTML via smarty.
 * 
 */
function display_report_html( $data, $totals ) {
    $_ENV['tpl']->assign('report', 'Groups Summary');
    $_ENV['tpl']->assign('data', $data);
    $_ENV['tpl']->assign('totals', $totals);
    $_ENV['tpl']->assign('time', time());
    $_ENV['tpl']->assign('user', $_SESSION['userid']);
    $_ENV['tpl']->assign('startDate', $_ENV['i18n']->parse_datetime(aget($_POST, 'sdate')));
    $_ENV['tpl']->assign('endDate', $_ENV['i18n']->parse_datetime(aget($_POST, 'edate')));
    $_ENV['tpl']->display("reports/group_summary_result.tpl");	
}

/**
 * Displays report in CSV
 * 
 */
function display_report_csv( $data, $totals ) {

    header( "Content-Type: text/plain" );

    $fh = tmpfile();
    if( !$fh ) {
        $_ENV['error']->push_error(_("Unable to generate CSV output."));
        return false;
    }

    // print a header using array keys.
    if ( count( $data ) ) {
        fputcsv( $fh, array_keys( $data[0] ) );
    }

    foreach( $data as $row_data ) {
        fputcsv( $fh, $row_data );
    }
    fseek($fh, 0);

    $buf = stream_get_contents( $fh );

    fclose( $fh );

    echo $buf;
}


/**
 * Displays report in PDF
 * 
 */
function display_report_pdf( $data, $totals ) {
    echo "PDF Report Format Unimplemented";
}

/**
 * Displays report in MS Word Doc format
 * 
 */
function display_report_word( $data, $totals ) {
    echo "Word Report Format Unimplemented";
}

/**
 * Displays report in MS Excel format.
 * 
 */
function display_report_excel( $data, $totals ) {
    echo "Excel Report Format Unimplemented";
}

/*
 * OLD IMPLEMENTATION
 */
// Get user permissions, if not admin of any group redirect
$groups = array();
if( is_admin() ) {
    $stmt = $_ENV['dbi']->prepare("SELECT gid from groups where active='t' order by name");
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute();
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $groups = $result->fetchCol(); 
    $result->free();
    $stmt->free();
}
else {
    foreach($_SESSION['groups'] as $group) {      
        if ( is_group_admin($group) ) {
            $groups[] = $group;  
        }
    }
}
if (empty($groups)) {
    $_ENV['error']->push_error( _("You don't have permission to run 'Group Summary' report.") );
    redirect();
    exit;  
}
?>
