<?php
if (strpos(basename(__FILE__),$_SERVER['PHP_SELF'])) {
  print "Direct module access forbidden.";
  exit;
}

$data = array();
$params = array();

foreach( $_POST['gid'] as $gid ) {
    if( !is_numeric( $gid ) ) {
        print "Invalid group selected.";
        exit;
    }
}

function verify_numeric_array($array, $empty_allowed, $error_msg ) {
    if( !$empty_allowed && (!is_array( $array ) || !count( $array )) ) {
        print $error_msg;
        exit;
    }
    foreach( $array as $val ) {
        if( !is_numeric( $val ) ) {
            print $error_msg;
            exit;
        }
    }
}

function get_issues( ) {
    $gid_list = aget( $_POST, 'gid');
    verify_numeric_array( $gid_list, false, _('Invalid group selected.') );
//    verify_numeric_array( aget( $_POST, 'issue_fields'), true, _('Invalid issue fields.') );
//    verify_numeric_array( aget( $_POST, 'event_fields'), true, _('Invalid event fields.') );

    $groups_data = array();

    foreach( $gid_list as $gid ) {
        $issues_data = get_issues_by_group( $gid, 
                                            aget( $_POST, 'issue_fields'), 
                                            aget( $_POST, 'hide_private_issues'),
                                            aget( $_POST, 'sdate'),
                                            aget( $_POST, 'edate') );

        $issues_list = array_keys( $issues_data );
        $duration = 0;
        if( count( $issues_list ) ) {
            $events_list = get_events_by_issues( $issues_list,
                                                 aget( $_POST, 'event_fields'), 
                                                 aget( $_POST, 'hide_private_events') );

            foreach( $events_list as $event ) {
                $issues_data[$event['issueid']]['events'][] = $event;
            }

	    $duration = get_total_duration_by_issues( $issues_list, false,
	   					      aget( $_POST, 'sdate'),
						      aget( $_POST, 'edate') );

            $duration_events_hidden = $duration;
            if( aget( $_POST, 'hide_private_events') ) {
		$duration_events_hidden = get_total_duration_by_issues( $issues_list, true,
		    							aget( $_POST, 'sdate'),
									aget( $_POST, 'edate') );
            }
        }

        $groups_data[] = array( 'gid' => $gid,
                                'hours' => $duration,
                                'hours_events_hidden' => $duration_events_hidden,
                                'issues' => $issues_data,
                             );
    }
    return $groups_data;
}

function get_issues_by_group( $group_id, $issue_fields, $hide_private_issues, $start_date, $end_date ) {

    $startDate = null;
    if ( $start_date ) {
      $startDate = $_ENV['i18n']->parse_datetime($start_date);
      if (!$startDate) {
        $_ENV['error']->push_error(_("Invalid start date."));
      }
    }
    
    $endDate = null;
    if ( $end_date ) {
      $endDate = $_ENV['i18n']->parse_datetime($end_date);
      if (!$endDate) {
        $_ENV['error']->push_error(_("Invalid end date."));
      }
    }

    $gid_list = implode( ',', $group_list );
    
    $sql  = "SELECT i.issueid, i.summary ";
    if( count( $issue_fields ) ) {
        $sql .= ', ';
        foreach ($issue_fields as $field) {
            if ($field =='gid' || $field == 'assigned_to') {
                $sql .= "ig.$field, ";       
            }
            else {
                $sql .= "i.$field, ";
            }
        }
    }
    $sql .= "null as fake FROM issue_groups ig, issues i ";
    $sql .= "WHERE ig.issueid = i.issueid and ig.gid = ? ";
    $params[] = $group_id;

    if( $startDate && $endDate ) {
        $sql .= "AND (select count(*) from events e where e.issueid = i.issueid and e.performed_on >= ? and e.performed_on <= ?) > 0 ";
        $params[] = $startDate;
        $params[] = $endDate;
    }
    else if( $startDate ) {
        $sql .= "AND (select count(*) from events e where e.issueid = i.issueid and e.performed_on >= ? ) > 0 ";
        $params[] = $startDate;
    }
    else if( $endDate ) {
        $sql .= "AND (select count(*) from events e where e.issueid = i.issueid and e.performed_on <= ? ) > 0 ";
        $params[] = $endDate;
    }

    // Filter out private issues if requested.
    if( $hide_private_issues ) {
       $sql .= "AND i.private != 't' ";
    }

    $sql .= ($sort == "descending") ? "ORDER BY i.issueid DESC " : "ORDER BY i.issueid ";

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $data = $result->fetchAll(MDB2_FETCHMODE_ASSOC);
    $result->free();
    $stmt->free();

    $new_data = array();
    foreach( $data as $row ) {
        $new_data[$row['issueid']] = $row;
    }

    return $new_data;
}

function get_total_duration_by_issues( $issue_list, $hide_private_events, $start_date, $end_date ) {
    $startDate = null;
    if ( $start_date ) {
         $startDate = $_ENV['i18n']->parse_datetime($start_date);
         if (!$startDate) {
            $_ENV['error']->push_error(_("Invalid start date."));
	 }
    }

    $endDate = null;
    if ( $end_date ) {
         $endDate = $_ENV['i18n']->parse_datetime($end_date);
         if (!$endDate) {
            $_ENV['error']->push_error(_("Invalid end date."));
         }
     }

    $issueid_list = implode( ',', $issue_list );

    $sql  = "SELECT sum(e.duration) from events e where e.issueid in ($issueid_list) ";
    // Filter out private events if requested.
    if( $hide_private_events ) {
        $sql .= "AND e.private != 't' ";
    }
    if ($startDate != "" || $endDate != "") {
       $sql .= "AND e.performed_on between ? and ?";
       $params[] = $startDate;
       $params[] = $endDate;
    }
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $hours = $result->fetchOne();
    $result->free();
    $stmt->free();

    return $hours;
}

function get_events_by_issues( $issue_list, $event_fields, $hide_private_events ) {

    $issueid_list = implode( ',', $issue_list );

    $params = array();
    $sql  = "SELECT e.issueid, ";
    foreach ($event_fields as $field) {
         $sql .= "e.$field, ";
    }
    $sql .= "e.action, userid "; 
    $sql .= "FROM events e ";
    $sql .= "WHERE e.issueid in ($issueid_list) ";
    // Filter out private events if requested.
    if( $hide_private_events ) {
        $sql .= "AND e.private != 't' ";
    }

    $sql .= "ORDER BY e.performed_on ";

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $data = $result->fetchAll(MDB2_FETCHMODE_ASSOC);
    $result->free();
    $stmt->free();

    return $data;
}

function get_issues_by_groups_old( $group_list ) {

    foreach ($group_list as $group) {
       $tmp['gid'] = $group;
    
       $sql  = "SELECT sum(e.duration) FROM events e, issue_groups ig, issues i ";
       $sql .= "WHERE ig.issueid = e.issueid AND ig.issueid = i.issueid and ig.gid = ? ";
       // Filter out private issues if requested.
       if( isset( $_POST['hide_private_issues'] ) && $_POST['hide_private_issues'] ) {
           $sql .= "AND i.private != 't' ";
       }
       $params[] = $group;
       $stmt = $_ENV['dbi']->prepare($sql);
       $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
       $result = $stmt->execute($params);
       $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
       $tmp['hours'] = $result->fetchOne();
       $result->free();
       $stmt->free();
    
       unset($params);
    
       $sql  = "SELECT ";
       foreach ($_POST['issue_fields'] as $field) {
          if ($field =='gid' || $field == 'assigned_to')
    	$sql .= "ig.$field, ";       
          else
            $sql .= "i.$field, ";
       }
       
       $sql .= "i.summary, i.issueid "; // by default the issue summary is always display it
       $sql .= "FROM issues i, issue_groups ig ";
       $sql .= "WHERE ig.issueid = i.issueid ";
       $sql .= "AND  ig.gid = ? ";
       // Filter out private issues if requested.
       if( isset( $_POST['hide_private_issues'] ) && $_POST['hide_private_issues'] ) {
           $sql .= "AND i.private != 't' ";
       }
       $params[] = $group;
     
       $sql .= ($sort == "descending") ? "ORDER BY i.issueid DESC " : "ORDER BY i.issueid ";
    
       $stmt = $_ENV['dbi']->prepare($sql);
       $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
       $result = $stmt->execute($params);
       $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
       $tmp['issues'] = $result->fetchAll(MDB2_FETCHMODE_ASSOC);  
       $result->free();
       $stmt->free();
    
    
       if ($_POST['show_events']) {
           foreach ($tmp['issues'] as $i=>$issue) {
              $params = array();
              $sql  = "SELECT ";
              foreach ($_POST['event_fields'] as $field) {
                   $sql .= "e.$field, ";
              }
              $sql .= "e.action, userid "; 
              $sql .= "FROM events e ";
              $sql .= "WHERE e.issueid = ? ";
              // Filter out private events if requested.
              if( isset( $_POST['hide_private_events'] ) && $_POST['hide_private_events'] ) {
                  $sql .= "AND e.private != 't' ";
              }
              $params[] = $issue['issueid'];
        
              $sql .= "ORDER BY e.performed_on ";
       
              $stmt = $_ENV['dbi']->prepare($sql);
              $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
              $result = $stmt->execute($params);
              $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
              $tmp['issues'][$i]['events'] = $result->fetchAll(MDB2_FETCHMODE_ASSOC);  
              $result->free();
              $stmt->free();
           }
       }
       unset($params);
       $data[] = $tmp;
    }
    return $data;
}

//$data = get_issues_by_groups_old( $_POST['gid'] );
$data = get_issues();

$_ENV['tpl']->assign('report', 'Issues');
$_ENV['tpl']->assign('data', $data);
$_ENV['tpl']->assign('issue_fields', $_POST['issue_fields']);
$_ENV['tpl']->assign('event_fields', $_POST['event_fields']);
$_ENV['tpl']->assign('show_events',$_POST['show_events']);
$_ENV['tpl']->assign('time', time());
$_ENV['tpl']->assign('user', $_SESSION['userid']);
$_ENV['tpl']->assign('startDate', $_ENV['i18n']->parse_datetime(aget($_POST, 'sdate')));
$_ENV['tpl']->assign('endDate', $_ENV['i18n']->parse_datetime(aget($_POST, 'edate')));
$_ENV['tpl']->display("reports/issues_result.tpl");

?>
