<?php

/* $Id$ */

/**
 * This file implements the group summary report generation logic.
 * 
 * @package EnterTrack
 * @subpackage Reports_Module
 */

// We only execute main().
main();


/**
 * Processing entry point.  Top-level procedure.
 * 
 */
function main() {
//    ini_set( 'display_errors', 'on' );
//    ini_set( 'error_reporting', E_ALL );

    if (strpos(basename(__FILE__),$_SERVER['PHP_SELF'])) {
        print "Direct module access forbidden.";
        exit;
    }
    
    $data = get_report();
    display_report_csv( $data );
}


/**
 * Verifies that all values in an array are numeric.
 * 
 */
function verify_numeric_array($array, $empty_allowed, $error_msg ) {
    if( !$empty_allowed && (!is_array( $array ) || !count( $array )) ) {
        print $error_msg;
        exit;
    }
    foreach( $array as $val ) {
        if( !is_numeric( $val ) ) {
            print $error_msg;
            exit;
        }
    }
}


/**
 * Generates a report.
 * 
 */
function get_report( ) {
    $gid_list = aget( $_POST, 'gid');
    verify_numeric_array( $gid_list, false, _('Invalid group selected.'|t) );

    $groups_data = get_report_data( $gid_list, 
                                    aget( $_POST, 'sdate'), 
                                    aget( $_POST, 'edate') );
    return $groups_data;
}

/**
 * Report generation worker function.  Contains SQL logic.
 * 
 */
function get_report_data( $group_id_list, $start_date, $end_date ) {
    $params = array();

    $startDate = null;
    if ( $start_date ) {
      $startDate = $_ENV['i18n']->parse_datetime($start_date);
      if (!$startDate) {
        $_ENV['error']->push_error(_("Invalid start date."));
      }
    }
    
    $endDate = null;
    if ( $end_date ) {
      $endDate = $_ENV['i18n']->parse_datetime($end_date);
      if (!$endDate) {
        $_ENV['error']->push_error(_("Invalid end date."));
      }
    }

    $gid_list = implode( ',', $group_id_list );

    $time_now = time();

    $start_clause = $startDate ? " and (i.opened >= '$startDate' or ( e.performed_on is not null and e.performed_on >= '$startDate' ))" : '';
    $end_clause = $endDate ? " and (i.opened <= '$endDate' or ( e.performed_on is not null and e.performed_on <= '$endDate' ))" : '';

    $sql = <<< END
select
 g.name,
 i.issueid,
 i.opened,
 i.modified,
 i.closed,
 i.due_date,
 i.summary,
 REPLACE(REPLACE(i.problem, '\r', ''), '\n', ' ') as problem,
 p.product,
 c.category,
 s.status,
 ig.assigned_to,
 REPLACE(REPLACE(e.action, '\r', ''), '\n', ' ') as action,
 e.performed_on,
 e.userid,
 e.duration
from 
 issue_groups ig inner join issues i on (ig.issueid = i.issueid)
                 left outer join groups g on (ig.gid = g.gid)
                 left outer join products p on (i.product = p.pid)
                 left outer join categories c on (i.category = c.cid)
                 left outer join statuses s on (i.status = s.sid)
                 left outer join events e on (e.issueid = i.issueid )
where
 ig.gid in ($gid_list)
 $start_clause
 $end_clause
order by
 ig.gid,
 i.issueid,
 e.eid
END;

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $data = $result->fetchAll(MDB2_FETCHMODE_ASSOC);
    $result->free();
    $stmt->free();

    return $data;
}

/**
 * Displays report in CSV
 * 
 */
function display_report_csv( $data ) {

    // use text/csv with content-disposition so browser should popup save/open dialog.
    header( 'Content-Type: text/csv' );
    header( sprintf( 'Content-Disposition: attachment; filename="report-rawdata-%s.csv"', date('c') ) );
    
    // use text/plain for browser to display in browser window.
    //header( "Content-Type: text/plain" );

    $column_names = array(
        _('Group'),
        _('Issue Id'),
        _('Opened'),
        _('Last Modification'),
        _('Closed'),
        _('Due Date'),
        _('Summary'),
        _('Issue Description'),
        _('Product'),
        _('Category'),
        _('Status'),
        _('Assigned to'),
        _('Event Description'),
        _('Event posted'),
        _('Event posted by'),
        _('Time Worked')
    );

    $fh = tmpfile();
    if( !$fh ) {
        $_ENV['error']->push_error(_("Unable to generate CSV output."));
        return false;
    }

    fputcsv( $fh, $column_names );

    foreach( $data as $row_data ) {
        $row_data = format_row( $row_data );
        fputcsv( $fh, $row_data );
    }
    fseek($fh, 0);

    $buf = stream_get_contents( $fh );

    fclose( $fh );

    echo $buf;
}

/**
 * Modifies certain fields for display purposes.
 * 
 */
function format_row( $row_data ) {
    $row_data['opened'] = $_ENV['i18n']->format_datetime( $row_data['opened'] );
    $row_data['modified'] = $_ENV['i18n']->format_datetime( $row_data['modified'] );
    $row_data['closed'] = $_ENV['i18n']->format_datetime( $row_data['closed'] );
    $row_data['due_date'] = $_ENV['i18n']->format_datetime( $row_data['due_date'] );
    $row_data['performed_on'] = $_ENV['i18n']->format_datetime( $row_data['performed_on'] );
    $row_data['userid'] = ident( $row_data['userid'] );
    $row_data['assigned_to'] = ident( $row_data['assigned_to'] );

    return $row_data;
}

?>
