<?php
/* $Id:$ */
/**
 * @package EnterTrack
 * @subpackage Reports_Module
 */
// using jpgraph library
include_once (_JPGRAPH_ . 'jpgraph.php');
include_once (_JPGRAPH_ . 'jpgraph_bar.php');

us_main();
exit;

/**
 * main().  Top level, performs all processing for user statistics report.
 * both form display and report generation.
 * 
 * The user stats report basically consists of all users that worked
 * (posted events) to issues in the selected groups during a given
 * time period.  The report shows (for each user) things like total 
 * hours worked,  average hours worked, and total events posted.
 * 
 */
function us_main() {

    if (strpos(basename(__FILE__),$_SERVER['PHP_SELF'])) {
      print "Direct module access forbidden.";
      exit;
    }

    $groups = get_groups_for_user();

    if (empty($groups)) {
        $_ENV['error']->push_error( _("You don't have permission to run 'User Statistics' report.") );
        redirect();
        exit;  
    }

    if ( count($_POST) && !((aget($_POST,'events') == 0) && (aget($_POST,'hours') == 0))) {
	    $data = run_report();
	    display_report( aget($_POST, 'format'), $data );
    }
    else {
	    if (count($_POST) > 0)
	    	$_ENV['error']->push_error( _("You must select a least one option (Events, Hours)") );
	    	
	    display_form( $groups );
    }

}

/**
 * Finds groups that current user can generate reports for.
 * Must be system admin, group admin or group technician.
 * 
 */
function get_groups_for_user() {

    // Get user permissions, if not admin of any group redirect
    $groups = array();
    if( is_admin() ) {
        $stmt = $_ENV['dbi']->prepare("SELECT gid from groups where active='t' order by name");
        $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
        $result = $stmt->execute();
        $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
        $groups = $result->fetchCol(); 
        $result->free();
        $stmt->free();
    }
    else {
        foreach($_SESSION['groups'] as $group) {      
            if ( is_group_admin($group) || permission_check("technician",$group) ) {
                $groups[] = $group;  
            }
        }
    }

    return $groups;
}

/**
 * Displays the report form where user can choose report options.
 * 
 */
function display_form( $groups ) {
    $_ENV['tpl']->assign('report', 'User Statistics');
    $_ENV['tpl']->assign('groups', $groups);
    $_ENV['tpl']->display("reports/user_statistics.tpl");
}

/**
 * Get post options events or hours
 * @author edder
 * @return array options to generate the report
 */
function get_options() {
	$options['events'] = aget($_POST, 'events');
	$options['hours'] = aget($_POST, 'hours');
	$options['sdate'] = aget( $_POST, 'sdate');
	$options['edate'] = aget( $_POST, 'edate');
	
	return $options;
}

/**
 * Generates a report.
 * 
 */
function run_report( ) {
    $gid_list = aget( $_POST, 'gid');
	$options = get_options();
	 
    $report_data = get_report_data( $gid_list, $options );
    return $report_data;
}

/**
 * Report generation worker function.  Contains SQL logic.
 * Last row of returned data contains totals.
 * 
 */
function get_report_data( $group_id_list, $options ) {
    $params = array();

    $startDate = null;
    if ( $options['sdate'] ) {
      $startDate = $_ENV['i18n']->parse_datetime($options['sdate']);
      if (!$startDate) {
        $_ENV['error']->push_error(_("Invalid start date."));
      }
    }
    
    $endDate = null;
    if ( $options['edate'] ) {
      $endDate = $_ENV['i18n']->parse_datetime($options['edate']);
      if (!$endDate) {
        $_ENV['error']->push_error(_("Invalid end date."));
      }
    }

    $gid_list = implode( ',', $group_id_list );

    $time_now = time();

    $start_clause = $startDate ? " and e.performed_on >= '$startDate'" : '';
    $end_clause = $endDate ? " and e.performed_on <= '$endDate'" : '';
    $cnt_events = $options['events'] == 1 ? "count( e.eid ) as cnt_events," : '';
    $sum_duration = $options['hours'] == 1 ? "round( sum( e.duration ), 2) as sum_duration," : '';
    $exclusion = $options['hours'] == 1 && $options['events'] != 1 ? " and e.duration <> 0.00 " : '';
    
    $order_by = 'ORDER BY ';
    if ( $options['events'] == 1 ) {
    	if ($options['hours'] == 1) {
    		$order_by .= ' sum_duration DESC, ';
    	}
    		$order_by .= 'cnt_events DESC ';
    } else {
    	$order_by .= ' sum_duration DESC ';
    	if ($options['events'] == 1) {
    		$order_by .= ' cnt_events DESC, ';
    	}
    }
    
    $sqlData = <<< END
select
 u.userid,
 u.username,
 u.first_name,
 u.last_name,
 $cnt_events
 $sum_duration
 round( avg( e.duration ), 2) as avg_duration
from 
 events e inner join users u on ( e.userid = u.userid )
          inner join issue_groups ig on (e.issueid = ig.issueid)
 where
 	ig.gid in ($gid_list)
 	$start_clause
 	$end_clause
 	$exclusion
group by 
	u.userid, 
	u.username, 
	u.first_name, 
	u.last_name
$order_by
END;

$sqlTotals = <<< END
select null, null, null, null,
 $cnt_events
 $sum_duration
 COALESCE(round( avg( e.duration ), 2)) as avg_duration
from 
 events e inner join users u on ( e.userid = u.userid )
          inner join issue_groups ig on (e.issueid = ig.issueid)
  where
 	ig.gid in ($gid_list)
 	$start_clause
 	$end_clause
END;
    // Note:  We use the 'with rollup' clause which means that we get an
    // extra row in the output that represents totals.
    // See: http://dev.mysql.com/doc/refman/5.0/en/group-by-modifiers.html
    
    // Extra-Note: with rollup is not compatible with postgreSql so it was
    // change it by union with summary select

	// Query for the data ordered
    $stmt = $_ENV['dbi']->prepare($sqlData);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $data = $result->fetchAll(MDB2_FETCHMODE_ASSOC);
    $result->free();
    $stmt->free();
    
    // Query for the totals
    $stmt = $_ENV['dbi']->prepare($sqlTotals);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $totals = $result->fetchAll(MDB2_FETCHMODE_ASSOC);
    $result->free();
    $stmt->free();
    
    // Insert totals row at finish row in data allready ordered
    array_push($data, $totals[0]);
		
    return $data;
}

/**
 * Displays report in various formats
 * 
 */
function display_report( $format, $data ) {

    switch( trim($format) ) {
    case 'html': display_report_html( $data ); break;
    case 'csv': display_report_csv( $data ); break;
    default: 
        $_ENV['error']->push_error(_("Unknown report format."));
        break;
    }
}


/**
 * Displays report in HTML via smarty.
 * 
 */
function display_report_html( $data ) {

    // remove the last row, which is totals.
    $totals = array_pop( $data );
    // Generate a graph of this data.
    $graph_url = generate_graph( $data );
    
    $options = get_options();

    $_ENV['tpl']->assign('report', _('User Statistics') );
    $_ENV['tpl']->assign('graphUrl', $graph_url);
    $_ENV['tpl']->assign('data', $data);
    $_ENV['tpl']->assign('options', $options);
    $_ENV['tpl']->assign('totals', $totals);
    $_ENV['tpl']->assign('time', time() );
    $_ENV['tpl']->assign('user', $_SESSION['userid']);
    $_ENV['tpl']->assign('startDate', $_ENV['i18n']->parse_datetime($options['sdate']));
    $_ENV['tpl']->assign('endDate', $_ENV['i18n']->parse_datetime($options['edate']));
    $_ENV['tpl']->display("reports/user_statistics_result.tpl");
}


/**
 * Displays report in HTML via smarty.
 * 
 * @param array $data report data generated from get_report_data()
 * @author edder_rojas
 * 
 * @return string image_url URL path to chart image 
 */

function generate_graph($data) {

	$hoursData = array();
	$eventsData = array();
	$users = array();
	$index = 0;
	$options = get_options();
	
	foreach ($data as $user) {
		$users[$index] = $user['first_name'] . ' ' . $user['last_name'];
		$hoursData[$index] = $options['hours'] == 1 ? $user['sum_duration'] : null;
		$eventsData[$index] = $options['events'] == 1 ? $user['cnt_events'] : null;
		$index++;
	}
	
	$title = '';
	$bars = array();
	
	if ($options['hours'] == 1 ) {
		$title = 'Hours';
			
		// Create the Hours bar plot
		$hoursPlot  = new BarPlot($hoursData);
		$hoursPlot->SetFillGradient("lightsteelblue","navy",GRAD_HOR);
		$hoursPlot->SetLegend('Hours');
		$hoursPlot->value->Show();
		$bars[] = $hoursPlot;
	} 
	
	if ($options['events'] == 1 ) {
		$title = $options['hours'] == 1 ? 'Hours & Events' : 'Events';
			
		// Create the Events bar plot
		$eventsPlot  = new BarPlot($eventsData);
		$eventsPlot->SetFillGradient("salmon3","brown3",GRAD_HOR);
		$eventsPlot->SetLegend('Events');
		$eventsPlot->value->Show();
		$bars[] = $eventsPlot;
	}
	// Set the graph width as 1000px max Width
	$graphWidth = (count($data) * 200);
	$graphWidth = $graphWidth >= 1000 ? 900 : $graphWidth;
	
	// Set the graph height as 600px max Height
	$graphHeight += $graphWidth == 900 ? count($data) * 35 : 250;
	
		
	// Create the graph. These two call always are required
	$graph = new Graph($graphWidth, $graphHeight, "auto");
	$graph->SetScale("textint");
	
	if ($graphWidth == 900) {
		$graph->Set90AndMargin(140,140,40,30);
		$graph->xaxis->SetLabelMargin(5);
		$graph->xaxis->SetPos('min');
		$graph->xaxis->SetLabelAlign('right','center');
		$graph->yaxis->scale->SetGrace(20);
		$graph->yaxis->SetTickSide(SIDE_LEFT);
		$graph->yaxis->SetLabelSide(SIDE_RIGHT);
		$graph->yaxis->SetLabelAlign('center','top');
		$graph->yaxis->title->SetAngle(0);
		$graph->yaxis->SetPos('max');
		$graph->subtitle->Set(_($title));
	} else {
		$graph->img->SetMargin(40,100,30,30);
		$graph->yaxis->title->Set(_($title));
		$graph->yaxis->scale->SetGrace(10);
	}
	
	// Configuration and design of the graphic
	$graph->SetFrame(true,'gray');
	$graph->SetMarginColor('white');
	$graph->title->Set(_("User Statistics"));
	$graph->title->SetFont(FF_FONT1, FS_BOLD,15);
	$graph->yaxis->title->SetFont(FF_FONT1, FS_BOLD);
	$graph->xaxis->SetTickLabels($users);
	$graph->xaxis->title->SetFont(FF_FONT1, FS_BOLD);
	$graph->legend->Pos( 0.03,0.5,"right" ,"center");
	
	// Grouped plots
	$gbplot  = new GroupBarPlot ($bars); 	
	$graph->Add( $gbplot);


	// Generate random name for image file to avoid conflicts with other processes.
	$imgUrl = 'img' . uniqid() . '.png';
	$imgUrl = 'cache/graphs/'. $imgUrl;

	// Display the graph and save it to disk
	$graph->Stroke($imgUrl);
	
	/*
	 * Global module function that removes old graphs images to make things clean
	 */ 
	removeOldGraphs();	
	
	return $imgUrl;
}

/**
 * Displays report in CSV
 * 
 */
function display_report_csv( $data ) {

    header( "Content-Type: text/plain" );

    // remove the last row, which is totals.
    $totals_row = array_pop( $data );

    $fh = tmpfile();
    if( !$fh ) {
        $_ENV['error']->push_error(_("Unable to generate CSV output."));
        return false;
    }

    // print a header using array keys.
    if ( count( $data ) ) {
        fputcsv( $fh, array_keys( $data[0] ) );
    }

    foreach( $data as $row_data ) {
        fputcsv( $fh, $row_data );
    }
    fseek($fh, 0);

    $buf = stream_get_contents( $fh );

    fclose( $fh );

    echo $buf;
}



?>
