<?php
/* $Id: class.php 391 2008-05-08 16:50:37Z danda $ */

require_once(_INCLUDES_."/classes/dbobject.class.php");

/* {{{ Class User */
/**
 * User
 *
 * A class representing a user.  The class implements write-through caching,
 * such that any changes are reflected immediately into the database, and data
 * is loaded as needed.  Settings-access functions always return the existing
 * value, and set a new value if given an argument.
 */
class User extends DBObject {
  /* {{{ Function constructor*/
  /**
   * Construct a new object with the given userid, assuming the
   * user exists in the database.
   *
   * @param integer $userid the user's userid
   * @param array $users_row a row from the users table
   */
  function User($userid, $users_row)
  {
    $this->_userid = $userid;
    $this->_data = $users_row;
  }
  /* }}} */

  /* {{{ Accessors */
  function userid() { return $this->_userid; }

  function username($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("username", 'string');
    } else {
      // XXX warning: caller must check for username collision
      return $this->_set("username", $new_val, 'username');
    }
  }

  function password($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("password", 'string');
    } else {
      return $this->_set("password", $new_val, 'string');
    }
  }

  function first_name($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("first_name", 'string');
    } else {
      return $this->_set("first_name", $new_val, 'string');
    }
  }

  function last_name($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("last_name", 'string');
    } else {
      return $this->_set("last_name", $new_val, 'string');
    }
  }

  function address($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("address", 'string');
    } else {
      return $this->_set("address", $new_val, 'string');
    }
  }

  function address2($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("address2", 'string');
    } else {
      return $this->_set("address2", $new_val, 'string');
    }
  }

  function telephone($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("telephone", 'string');
    } else {
      return $this->_set("telephone", $new_val, 'string');
    }
  }

  function primary_email($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("primary_email", 'string');
    } else {
      // XXX warning: caller must check for email collision
      return $this->_set("primary_email", $new_val, 'string');
    }
  }

  function sms($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("sms", 'string');
    } else {
      return $this->_set("sms", $new_val, 'string');
    }
  }

  function admin($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("admin", 'boolean');
    } else {
      return $this->_set("admin", $new_val, 'boolean');
    }
  }

  function active($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("active", 'boolean');
    } else {
      return $this->_set("active", $new_val, 'boolean');
    }
  }

  function theme($new_val=null) {
    if (func_num_args() == 0) {
      return $this->_get("theme", 'string');
    } else {
      return $this->_set("theme", $new_val, 'string');
    }
  }
  /* }}} */

  ////
  // Misc.

  /* {{{ Function is_email_only */
  /**
   * Is this user an email-only user?
   *
   * @return boolean
   */
  function is_email_only()
  {
    return !$this->username();
  }
  /* }}} */

  /* {{{ Function groups */
  /**
   * Retrive list of active groups the user belongs to
   *
   * @return mixed
   */
  function groups()
  {
    $first = TRUE;

    $groups = array();

    $sql  = "select u.gid ";
    $sql .= "from group_users u, groups g ";
    $sql .= "where u.userid=? ";
    $sql .= "and g.gid = u.gid ";
    $sql .= "and g.active = 't' ";
    $sql .= "order by g.name";

    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($this->_userid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $groups = $result->fetchCol();
    $result->free();
    $stmt->free();

    return $groups;
  }
  /* }}} */

  ////
  // Email management

  /* {{{ Function alternate_emails */
  /**
   * Retrive list of alternate emails the user has (including the
   * primary email)
   *
   * @return mixed
   */
  function alternate_emails()
  {
    $alts = array();

    $sql  = "select email ";
    $sql .= "from alternate_emails ";
    $sql .= "where userid=? ";
    $sql .= "order by email ";

    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($this->_userid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $alts = $result->fetchCol();
    $result->free();
    $stmt->free();

    return $alts;
  }
  /* }}} */

  /* {{{ Function add_new_email */
  /**
   * Add a new email address to this user's list of alternate addresses.
   * This function assumes that the caller has already checked for potential
   * email-address collsisions.
   *
   * @param string $email the address
   */
  function add_new_email($email)
  {
    $insert = array();
    $types = array();
    $insert['email'] = $email;
    $types[] = 'text';
    $insert['userid'] = $this->_userid;
    $types[] = 'integer';
    $result = $_ENV['dbi']->extended->autoExecute("alternate_emails",
      $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  }
  /* }}} */

  /* {{{ Function delete_email */
  /**
   * Remove an existing email address from the user's list of alternate addresses
   *
   * @param string $email the address
   */
  function delete_email($email)
  {
    // delete the email
    $result = $_ENV['dbi']->extended->autoExecute("alternate_emails",
      null,
      MDB2_AUTOQUERY_DELETE,
      sprintf("email=%s", $_ENV['dbi']->quote($email, 'text'))); 
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

    // if that email was the primary email, select a new one (at "random")
    if ($email == $this->primary_email()) {
      $emails = $this->alternate_emails();
      if ($emails) {
        $this->primary_email($emails[0]);
      }
    }
  }
  /* }}} */

  ////
  // Notifications

  /* {{{ Function send_notifications */
  /**
   * Send out the given notifications for this user.
   *
   * @param list $notices of notices for this user.
   * @param assoc &$sent_addresses_by_channel_and_issue returns lists of sent_addresses, per channel, per issue
   */
  function send_notifications($notices, &$sent_addresses_by_channel_and_issue)
  {
    // We're going to apportion changes into channels in accordance with
    // this user's prefs
    $changes_for_channels = array();

    foreach ($notices as $notice) {
      $channels_for_change = array(); // use an array to eliminate duplicates with multiple reasons
      $prefs = $_ENV['notif']->get_user_prefs($this->_userid);
      foreach ($notice->reasons as $reason) {
        if (isset($prefs[$reason])) {
          $channels = $prefs[$reason];
          foreach ($channels as $channelid => $interested) {
            if ($interested) {
              $channels_for_change[$channelid] = 1;
            }
          }
        }
      }

      // now put one of each of those changes in the channel
      foreach ($channels_for_change as $channelid => $ignored) {
        if (!isset($changes_for_channels[$channelid])) {
          $changes_for_channels[$channelid] = array();
        }
        $changes_for_channels[$channelid][] = $notice->change;
      }
    }

    /* now actually send those notifications */
    $sent_addresses_by_channel = array();
    foreach ($changes_for_channels as $channelid => $changes) {
      $channel = $_ENV['notif']->get_channel($channelid);
      if ($channel) {
        // $sent_addresses is passed by reference.
        $sent_addresses = array();
        $channel->send_notifications($this->_userid, $changes, $sent_addresses);
        $sent_addresses_by_channel_and_issue[$channelid] = $sent_addresses;
      }
    }
  }
  /* }}} */

  ////
  // Private functions

  /* {{{ Function _get */
  /**
   * A general accessor function -- get the given setting.
   *
   * @param string $setting
   * @param string $type the type of the value (see includes/classes/dbobject.class.php)
   * @return mixed the value of the setting
   */
  function _get($setting, $type)
  {
    $old_value = $this->_db_to_native($this->_data[$setting], $type);
    return $old_value;
  }
  /* }}} */

  /* {{{ Function _set */
  /**
   * A general mutator function -- Record a new setting in the database, and
   * return the old value.  This both performs an update query and records a
   * notification.
   *
   * @param string $setting
   * @param mixed $new_value the new value for the setting, or null to simply
   *              retrieve the old value
   * @param string $type the type of the value (see includes/classes/dbobject.class.php)
   * @return mixed the old value of the setting
   */
  function _set($setting, $new_value, $type)
  {
    $old_value = $this->_db_to_native($this->_data[$setting], $type);

    if ($old_value != $new_value) {
      // change the value
      $db_value = $this->_native_to_db($new_value, $type);
      $result = $_ENV['dbi']->extended->autoExecute("users",
        array($setting => $db_value),
        MDB2_AUTOQUERY_UPDATE,
        sprintf("userid=%s", $_ENV['dbi']->quote($this->_userid, 'integer')),
        array($this->_db_type($type))); 
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

      $this->_data[$setting] = $db_value;
    }

    return $old_value;
  }
  /* }}} */

}
/* }}} */
