<?php
if (eregi(basename(__FILE__),$_SERVER['PHP_SELF'])) {
  print "Direct module access forbidden.";
  exit;
}

include_once(_CLASSES_."mail.class.php");

/* {{{ Function send_email_verification */
/**
 * Send a verification email to the given email address.  The email
 * will contain a link which will invoke modules/verify/$action.verify.php.
 *
 * @param $email string destination email address
 * @param $message string the message to include in the email; it should
 *        refer to "the link below".
 * @param $action string the action to invoke when the link is clicked
 * @param $arguments array any additional data required for the action
 *        (see get_verification_arguments)
 */
function send_email_verification($email, $message, $action, $arguments)
{
  // invent a token for the link
  $verification_key = substr(md5(rand(1,1000000)), 0, 32);

  // insert it into the table
  $insert = array();
  $types = array();
  $insert["action"] = $action;
  $types[] = 'text';
  $insert["arguments"] = serialize($arguments);
  $types[] = 'text';
  $insert["verification_key"] = $verification_key;
  $types[] = 'text';
  $result = $_ENV['dbi']->extended->autoExecute("email_verification",
    $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

  // fix up a nice URL
  $url = _URL_."?module=verify&key=".$verification_key;

  // generate a message
  $msg =& new Mailer();
  $msg->to($email);
  $msg->subject(sprintf("%s %s", $_ENV['config']['display']['company'], _("email verification")));

  $m  = sprintf(_("This email was sent to verify the accuracy of the email address '%s'.\n"), $email);
  $m .= _("If you did not request this email, you may safely ignore it.\n\n");
  $m .= $message;
  $m .= "\n\n";
  $m .= _("Please click on or otherwise access the following link to continue:\n\n");
  $m .= "  $url\n\n";
  $m .= sprintf(_("If you have any questions or difficulties with this email, please contact the site\nadministrator at %s."), $_ENV['config']['email']['admin']);
  $msg->message($m);
  $msg->send();
}
/* }}} */

/* {{{ Function get_verification_arguments */
/**
 * Fetch the arguments for the verification fetched by the current link;
 * see the $arguments parameter of send_email_verification
 *
 * @return array the arguments
 */
function get_verification_arguments()
{
  // get the key
  if (!isset($_GET['key'])) { return null; }
  $verification_key = $_GET['key'];

  // sanity-check it, even though it gets db-quoted below
  if (!eregi("^[a-z0-9]+$", $verification_key)) { return null; }

  $sql  = "SELECT arguments ";
  $sql .= "FROM email_verification ";
  $sql .= "WHERE verification_key=? ";
  $stmt = $_ENV['dbi']->prepare($sql);
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($verification_key));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $user_row = $result->fetchRow(MDB2_FETCHMODE_ASSOC); 
  $result->free();
  $stmt->free();
  if (!is_null($user_row)) {
    // delete the row
    $stmt = $_ENV['dbi']->prepare("DELETE from email_verification where verification_key=?"); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $stmt->execute(array($verification_key));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $result->free();
    $stmt->free();

    // and return the arguments
    return unserialize($user_row['arguments']);
  } else {
    return null;
  }
}
/* }}} */
