<?php
// Run a series of tests to be sure EnterTrack will function.

define("_PATH_",dirname(dirname(__FILE__)));
include(_PATH_."/setup/util/common.php");
include(_PATH_."/setup/util/config.php");
include(_PATH_."/conf/paths.php");

// make sure this user has authenticated for setup
if (!isset($_SESSION['authenticated']) || !$_SESSION['authenticated']) {
  redirect("index.php");
}

// make sure we have the info we need from the database section
if (!isset($_SESSION['database'])) {
  redirect("database.php");
}

$c = new Configuration();

// determine whether we have config changes to make
$need_new_config = false;
if ($c->needs_upgrade()) {
  $need_new_config = true;
}

// and see whether we need to change the db info
$db_info_changed = false;
foreach (array('type', 'host', 'port', 'name', 'user', 'pass') as $k) {
  if ($c->data['db'][$k] != $_SESSION['database'][$k]) {
    $need_new_config = true;
    $db_info_changed = true;
  }
}

/* {{{ Upgrade explanation page */
function explanation_page($c, $save_error)
{
  global $db_info_changed;
  ?>
  <h2>Configuration Changes</h2>
  <p>The following changes are being made to your configuration:
  <div class="setup-steps">
  <?php
    // upgrade to the latest version first,
    $c->upgrade();

    // then update database information
    if ($db_info_changed) {
      echo "<div class=\"step\">Updating database information</div>";
      foreach (array('type', 'host', 'port', 'name', 'user', 'pass') as $k) {
        $c->data['db'][$k] = $_SESSION['database'][$k];
      }
    }
  ?>
  </div>
  <p>resulting in the following configuration file: (<a href="config.php?download=1">download</a>)</p>
  <?php

  // store the new config text in a session variable so we don't have
  // to recreate it for download later.
  $_SESSION['config_file'] = $c->generate_new_text();
  echo "<pre class=\"config-file\">\n";
  echo htmlentities("<?" . "php\n" . $_SESSION['config_file']);
  echo "</pre>";

  // determine whether we can write the file ourselves
  ?>
  <table width="100%">
  <tr><td width="29%" align="right">Configuration Options:</td>
  <?php if (is_writeable(dirname($c->default_filename())) && !$save_error): ?>
      <td width="80%">
      <input type="radio" name="save" value="save" checked="checked">
      The wizard should save this configuration to the appropriate location
      </td>
  <?php else: ?>
      <td width="80%">
      <input type="radio" name="save" value="save" disabled="disabled">
      The wizard should save this configuration to the appropriate location<br />
      <span class="error">(The wizard cannot save the file to the configuration directory due to insufficient permissions)</span>
      </td>
  <?php endif; ?>
  </tr>
  <tr><td width="29%" align="right">&nbsp;</td>
      <td width="80%">
      <input type="radio" name="save" value="files" >
      Save the configuration file to <tt><?php echo _FILES_; ?>config.php</tt>; I will move it to the appropriate location.
      </td>
  <tr><td width="29%" align="right">&nbsp;</td>
      <td width="80%">
      <input type="radio" name="save" value="manual" >
      I have downloaded the file from this page and manually installed it at
      <tt><?php echo $c->config_filename; ?></tt> on my web server.</td>
      </td>
  </tr>
  </table>
  <input type="hidden" name="action" value="finish">
<?php
}
/* }}} */

/* {{{ No-changes page */
function no_changes_page($c)
{ ?>
  <h2>No Changes Required</h2>
  <p>Your configuration is ready to go; click "Next" to finish your setup.</p>
  <input type="hidden" name="action" value="finish">
  <?php
}
/* }}} */

// handle downloads
if ($_GET['download']) {
  header("Content-Type: text/plain; filename=\"config.php\"");
  echo "<?" . "php\n";
  echo $_SESSION['config_file'];
  exit;
}

$save_error = false;
if ($_SERVER['REQUEST_METHOD'] == 'POST') {
  if (isset($_POST['_start_over'])) {
    redirect("index.php");
  }

  if (isset($_POST['save'])) {
    switch ($_POST['save']) {
      case 'save':
        if ($c->save($_SESSION['config_file'], $c->default_filename())) {
          redirect("finish.php");
        } else {
          $save_error = true;
        }
        break;

      case 'files':
        if ($c->save($_SESSION['config_file'], _FILES_."config.php")) {
          redirect("finish.php");
        } else {
          $save_error = true;
        }
        break;

      case 'manual':
        // assume they did the right thing
        redirect("finish.php");
        break;
    }
  }

  if (isset($_POST['action'])) {
    switch ($_POST['action']) {
      case 'finish':
        redirect("finish.php");
        break;
    }
  }
}

page_header("Setup :: Configuration", "Use this page to automatically update your EnterTrack configuration.");
step_js();

// display the configuration-update page
if ($need_new_config) {
  explanation_page($c, $save_error);

// otherwise shoo the user on to the final page.
} else {
  no_changes_page($c);
}

navigation_box(true);
page_footer();
