<?php
// Run a series of tests to be sure EnterTrack will function.

define("_PATH_",dirname(dirname(__FILE__)));
include(_PATH_."/setup/util/common.php");
include(_PATH_."/setup/util/db.php");
include(_PATH_."/setup/util/config.php");
include(_PATH_."/setup/util/schema.php");
include(_PATH_."/setup/util/schema_upgrades.php");

// make sure this user has authenticated for setup
if (!isset($_SESSION['authenticated']) || !$_SESSION['authenticated']) {
  redirect("index.php");
}

// make sure we have the info from the tests that we need.
if (!isset($_SESSION['tests']['db'])) {
  redirect("tests.php");
}

////
// page-rendering functions

/* {{{ setup_type page */
function database_setup_type_page()
{
  // Some shortcuts to make the below more readable
  $S = $_SESSION['database'];

  // The form on the page
  ?>
<h2>Setup Type</h2>
<p>Are you installing EnterTrack from scratch, or upgrading an existing EnterTrack or IssueTracker installation?</p>
<table width="100%">
<tr><td align="right" width="20%">Installing New:</td><td><input type="radio" name="setup_type" value="install"<?php if ($S['setup_type'] == 'install') { echo ' CHECKED'; } ?>><?php err("setup_type"); ?></td></tr>
<tr><td align="right" width="20%">Upgrading Existing:</td><td><input type="radio" name="setup_type" value="upgrade"<?php if ($S['setup_type'] == 'upgrade') { echo ' CHECKED'; } ?>></td></tr>
</table>
<input type="hidden" name="stage" value="setup_type">
<?php
  navigation_box(TRUE);
}

function database_setup_type_submit()
{
  // reset all error flags
  $_SESSION['database']['error'] = array();
  $proceed = true;

  if (isset($_POST['setup_type'])) {
    $_SESSION['database']['setup_type'] = $_POST['setup_type'];
    return 'config';
  }

  $_SESSION['database']['error']['setup_type'] = "Please specify a setup type.";
  return 'setup_type';
}
/* }}} */

/* {{{ config page */
function database_config_page()
{
  // Some shortcuts to make the below more readable
  $S = $_SESSION['database'];
  $T = $S['setup_type'];

  // The forms on the page
  ?>
<h2>Database Administrator Credentials</h2>
<?php if ($S['setup_type'] == 'install'): ?>
<p>In many cases, the operations involved in setting up a new EnterTrack installation
(creating a user, a database, and tables) require permissions beyond those granted to
an ordinary database username.
<?php else: ?>
<p>In many cases, the operations involved in upgrading an EnterTrack installation (modifying
the database schema) require permissions beyond those granted to an ordinary database
username.
<?php endif; ?>
If you have an administrative login for the database server, enter it here.  If you do not
have such a login, you may continue without one.  Consult your database administrator for
help with any permissions errors that may occur.</p>
<table width="100%">
<tr><td align="right" width="20%">Admin Username:</td>
    <td width="80%"><input type="text" name="admin_user" size="30" value="<?php echo $S['admin_user']; ?>">
  <?php err("admin_user"); ?></td></tr>

<tr><td align="right" width="20%">Admin Password:</td>
    <td width="80%"><input type="password" name="admin_pass" size="30" value="<?php echo $S['admin_pass']; ?>">
  <?php err("admin_pass"); ?></td></tr>
</table>
<h2>Database Specification</h2>
<?php if ($T == 'install'): ?>
  <p>Specify the database you would like to use for this EnterTrack installation.  Check the box if you or your DB administrator have already created this database.</p>
<?php else: ?>
  <p>Specify the database used by your existing EnterTrack installation.</p>
<?php endif; ?>
<table width="100%">
<tr><td align="right" width="20%">Database Type:</td>
    <td width="80%"><select name="type">
  <?php
    if ($_SESSION['tests']['db']) {
      foreach ($_SESSION['tests']['db'] as $db_type) {
        echo "<option value=\"$db_type\"";
        if ($db_type == $S['type']) { echo " SELECTED"; }
        echo ">$db_type</option>\n";
      }
    } else {
      echo "<b>(no database extensions installed)</b>";
    }
  ?>
  </select><?php err("type"); ?></td></tr>

<tr><td align="right" width="20%">Hostname:</td>
    <td width="80%"><input type="text" name="host" size="40" value="<?php echo $S['host']; ?>">
  <?php err("host"); ?></td></tr>

<tr><td align="right" width="20%">Port:</td>
    <td width="80%"><input type="text" name="port" size="10" value="<?php echo $S['port']; ?>">
  (leave blank for default)
  <?php err("port"); ?></td></tr>

<tr><td align="right" width="20%">Database Name:</td>
    <td width="80%"><input type="text" name="name" size="30" value="<?php echo $S['name']; ?>">
  <?php err("name"); ?></td></tr>

<?php if ($T == 'install'): ?>
<tr><td align="right" width="20%">Already Created:</td>
    <td width="80%"><input type="checkbox" name="db_created" <?php if ($S['db_created']) { echo ' CHECKED'; } ?>>
  <?php err("db_created"); ?></td></tr>
<?php endif; ?>

</table>

<h2>Access Credentials</h2>
<?php if ($T == 'install'): ?>
<p>Enter a <i>new</i> database username and password that EnterTrack can use to access the database.  <em>Do not use your root password!</em>.  Check the box if you or your DB administrator have already created this user.</p>
<?php else: ?>
<p>Specify the database credentials used by your existing EnterTrack installation.</p>
<?php endif; ?>
<table width="100%">
<tr><td align="right" width="20%">Username:</td>
    <td width="80%"><input type="text" name="user" size="30" value="<?php echo $S['user']; ?>">
  <?php err("user"); ?></td></tr>

<tr><td align="right" width="20%">Password:</td>
    <td width="80%"><input type="password" name="pass" size="30" value="<?php echo $S['pass']; ?>">
  <?php err("pass"); ?></td></tr>

<?php if ($T == 'install'): ?>
<tr><td align="right" width="20%">Already Created:</td>
    <td width="80%"><input type="checkbox" name="user_created" <?php if ($S['user_created']) { echo ' CHECKED'; } ?>>
  <?php err("user_created"); ?></td></tr>
<?php endif; ?>

</table>
<input type="hidden" name="stage" value="config">
  <?php
  navigation_box(TRUE);
}

function database_config_submit()
{
  // reset all error flags
  $_SESSION['database']['error'] = array();
  $proceed = true;

  // Copy all POST results to the session
  $variables = array("type", "host", "port", "name", "user", "pass", "db_created", "user_created", "admin_user", "admin_pass");
  foreach ($variables as $variable) {
    if (isset($_POST[$variable])) {
      $_SESSION['database'][$variable] = $_POST[$variable];
    } else {
      $_SESSION['database'][$variable] = '';
    }
  }

  // Postgres doesn't like users with capital case o upper cases
  $_SESSION['database']['user'] = $_SESSION['database']['type'] == "pgsql" ? strtolower($_SESSION['database']['user']) : $_SESSION['database']['user']; 
  
  // shortcut
  $S = $_SESSION['database'];

  // Error-checking
  if (!$S['name']) {
    $_SESSION['database']['error']['name'] = 'You must specify a database name';
    $proceed = false;
  }

  if (preg_match('/[^a-zA-Z0-9_-]/', $S['name'])) {
    $_SESSION['database']['error']['name'] = 'The database name must consist of alphanumeric characters, dash, and underscore.';
    $proceed = false;
  }

  if (!$S['user']) {
    $_SESSION['database']['error']['user'] = 'You must specify a database username.';
    $proceed = false;
  }

  if ($S['user'] && preg_match('/[^a-zA-Z0-9_-]/', $S['user'])) {
    $_SESSION['database']['error']['user'] = 'The database username must consist of alphanumeric characters, dash, and underscore.';
    $proceed = false;
  }

  if ($S['user'] && !$S['pass']) {
    $_SESSION['database']['error']['pass'] = 'You must specify a password for this user';
    $proceed = false;
  }

  if ($S['admin_user'] && preg_match('/[^a-zA-Z0-9_-]/', $S['admin_user'])) {
    $_SESSION['database']['error']['admin_user'] = 'The database username must consist of alphanumeric characters, dash, and underscore.';
    $proceed = false;
  }

  if ($S['admin_user'] && !$S['admin_pass']) {
    $_SESSION['database']['error']['admin_pass'] = "If you specify an administrative user you must also specify a password";
    $proceed = false;
  }

  if ($proceed && $S['admin_user'] && $S['admin_pass']) {
    // try it out right away
    $err = try_db_login($S['type'], $S['host'], $S['port'], $S['admin_user'], $S['admin_pass']);
    if ($err) {
      $_SESSION['database']['error']['admin_user'] = "Could not authenticate with this username.  The error was: $err.";
      $proceed = false;
    }
  }

  // Return status
  if ($proceed) {
    switch ($S['setup_type']) {
      case 'install':
        // try logging in with any of the credentials we got; erroring out with any problems
        if ($S['user_created']) {
          // Just test the user
          if ($err = try_db_login($S['type'], $S['host'], $S['port'], $S['user'], $S['pass'])) {
            $_SESSION['database']['error']['user'] = "Could not authenticate with this username and password.  The error was: $err.";
            return 'config';
          }
        }

        if ($S['user_created'] && $S['db_created']) {
          if ($err = try_db_login($S['type'], $S['host'], $S['port'], $S['user'], $S['pass'], $S['name'])) {
            $_SESSION['database']['error']['name'] = "Could not connect to this database with the username and password below.  The error was: $err.";
            return 'config';
          }
        }

        // if we need to create the user or database, then test out this administrative user right now
        if (!$S['user_created'] || !$S['db_created']) {
          if (!$S['admin_user'] || !$S['admin_pass']) {
            $_SESSION['database']['error']['admin_user'] = "You must specify an administrative username and password to create a user or database";
            return 'config';
          }

          if ($err = try_db_login($S['type'], $S['host'], $S['port'], $S['admin_user'], $S['admin_pass'])) {
            $_SESSION['database']['error']['admin_user'] = "Could not authenticate with this username and password.  The error was: $err.";
            return 'config';
          }

          // tests out -- go to the page where we actually create the user and/or database
          return 'install_db_user';
        }

        // otherwise, get a root password to install the user and/or database
        return 'install_schema';
        break;

      case 'upgrade':
        // if not successful, bail with an error on the username
        if ($err = try_db_login($S['type'], $S['host'], $S['port'], $S['user'], $S['pass'])) {
          $_SESSION['database']['error']['user'] = "Could not authenticate with this username and password.  The error was: $err.";
          return 'config';
        }

        // try accessing the named database
        // if not successful, bail with an error on the database name
        if ($err = try_db_login($S['type'], $S['host'], $S['port'], $S['user'], $S['pass'], $S['name'])) {
          $_SESSION['database']['error']['name'] = "Could not connect to this database with the username and password below.  The error was: $err.";
          return 'config';
        }

        // If we got an admin user, try it out
        if ($S['admin_user'] && $S['admin_pass']) {
          if ($err = try_db_login($S['type'], $S['host'], $S['port'], $S['admin_user'], $S['admin_pass'], $S['name'])) {
            $_SESSION['database']['error']['admin_user'] = "Could not connect to the specified database with this username and password.  The error was: $err.";
            return 'config';
          }
        }

        // go to the database-backup stage
        return 'backup';
        break;

      default:
        return 'config'; // shouldn't happen
    }
  } else {
    return 'config'; // re-draw this config page with updated error messages
  }
}
/* }}} */

// upgrade pages

/* {{{ backup page */
function database_backup_page()
{
  // Some shortcuts to make the below more readable
  $S = $_SESSION['database'];

  // The forms on the page
  ?>
<h2>Database Backup</h2>
<p>This would be an opportune moment to backup your existing database.  For security reasons, this page will not generate the backup for you.  However, the following instructions may be helpful.</p>

<?php if ($S['type'] == 'mysql'): ?>
Use the <a href="http://dev.mysql.com/doc/mysql/en/mysqldump.html"><tt>mysqldump</tt> utility</a> to make a textual dump of your database contents.  From the shell prompt on the web server, use the following:
<pre><?php
echo "mysqldump ";
echo "--user='{$S['user']}' ";
echo "-p ";
if ($S['host'] && $S['host'] != 'localhost') { echo "-h '{$S['host']}' "; }
echo "'{$S['name']}' > /path/to/backup/file.sql";
?></pre>
<p>If you are not logged in to the web server, you may need to adjust the <tt>-h</tt> option above and/or modify the database permissions to allow access.  If you are running a Windows server, consult the <a href="http://dev.mysql.com/doc/mysql/en/disaster-prevention.html">MySQL documentation</a> for advice on making backups.</p>
<?php endif ?>

<?php if ($S['type'] == 'pgsql'): ?>
Use the <a href="http://www.postgresql.org/docs/8.1/static/app-pgdump.html"><tt>pg_dump</tt> utility</a> to make a textual backup of your database.  From the shell prompt on the web server, use the following:
<pre><?php
echo "pg_dump ";
echo "-U '{$S['user']}' ";
if ($S['host'] && $S['host'] != 'localhost') { echo "-h '{$S['host']}' "; }
echo "'{$S['name']}' > /path/to/backup/file.sql";
?></pre>
<p>If you are not logged in to the web server, you may need to adjust the <tt>-h</tt> option above and/or modify the database permissions to allow access.  If you are running a Windows server, consult the <a href="http://www.postgresql.org/docs/8.1/static/backup.html">PostgreSQL documentation</a> for advice on making backups.</p>
<?php endif ?>

<p>Click <i>Next</i> to begin the upgrade process.</p>
<input type="hidden" name="stage" value="backup">
  <?php
  navigation_box(TRUE);
}

function database_backup_submit()
{
  // unconditionally go on to the next phase.
  return 'schema_version';
}
/* }}} */

/* {{{ schema_version page */
function database_schema_version_page()
{
  // Some shortcuts to make the below more readable
  $S = $_SESSION['database'];

  // first guess the version
  $link = db_connect();
  if (!$link) { die("Could not connect to database"); } // this should have been caught earlier, so just die
  $schema_version = get_schema_version($link);
  $schema_version_name = get_schema_version_name($schema_version);
  $all_version_names = get_all_schema_version_names();
  setup_db_close($link);

  // set up the array of all names for display, in order
  if ($schema_version and !isset($$all_version_names[$schema_version])) {
    $all_version_names[$schema_version] = $schema_version_name;
  }

  ksort($all_version_names);

  // The forms on the page
  ?>
<h2>Existing Version Autodetection</h2>
<p>The setup wizard has attempted to automatically determine your installed version of EnterTrack.  Note that this determination is based only on an analysis of the database schema for key differences between released versions.  If you have changed your database schema, then this upgrade process <i>may</i> destroy your installation.  Double-check your backups, and give it a shot.</p>
<p>Please verify that the wizard has correctly determined the installed version, then click next to perform the upgrade.</p>
<table width="100%">
<tr><td align="right" width="20%">Existing Version:</td>
    <td width="80%"><select name="schema_version"><?php
  foreach ($all_version_names as $vers => $vers_name) {
    echo "<option value=\"$vers\"";
    if ($vers == $schema_version) { echo " SELECTED"; }
    echo ">$vers_name</option>\n";
  }
  ?></select>
<?php if (!$schema_version): ?>
  <span class="error">The wizard could not determine the installed version.  You may select a version here, but please be aware that it is unlikely to work!</span>
<?php endif; ?>
</td></tr>
</table>
<input type="hidden" name="stage" value="schema_version">
  <?php
  navigation_box(TRUE);
}

function database_schema_version_submit()
{
  if (isset($_POST['schema_version']) && $_POST['schema_version']) {
    $_SESSION['database']['schema_version'] = $_POST['schema_version'];
    return 'upgrade_schema';
  } else {
    return 'schema_version';
  }
}
/* }}} */

/* {{{ upgrade_schema page */
function database_upgrade_schema_page()
{
  // Some shortcuts to make the below more readable
  $S = $_SESSION['database'];

  // The forms on the page
  ?>
<h2>Upgrade Schema</h2>
<p>The setup wizard is upgrading the EnterTrack database Schema.</p>
<?php

  // update the schema
  $link = db_connect();
  if (!$link) { die("Could not connect to database"); } // this condition has been tested for; shouldn't happen

  $proceed = upgrade_schema($link, $S['schema_version']);
?>
</div>
<?php if ($proceed): ?>
<p>The schema has been upgraded successfully.</p>
<?php else: ?>
<p><span class="error">The schema upgrade failed; consult the messages above to determine the cause.</span></p>
<?php endif; ?>
<input type="hidden" name="stage" value="upgrade_schema">
  <?php
  navigation_box($proceed);
}

function database_upgrade_schema_submit()
{
  redirect("config.php");
}
/* }}} */

// install pages

/* {{{ install_db_user page */
function database_install_db_user_page()
{
  // Some shortcuts to make the below more readable
  $S = $_SESSION['database'];

  // The forms on the page
  ?>
<h2>Install Database and User</h2>
<p>The setup wizard is creating the user and database you specified.</p>
<div class="setup-steps">
<?php

  // Install the database user and/or the database

  // connect to the database server, but not to our (as-yet non-existent) database
  $link = setup_db_connect($S['type'], $S['host'], $S['port'], $S['admin_user'], $S['admin_pass']);
  if (!$link) { die("Could not connect to database"); }

  $proceed = true;

  // create the database
  if ($proceed && !$S['db_created']) {
    $sql = "CREATE database {$S['name']}";
    step_header(true);
    echo "<div class=\"title\">Create Database</div>\n";
    echo "<pre class=\"sql\">$sql</pre>\n";
    $res = setup_db_execute($link, $sql);
    if (!$res) {
      $err = setup_db_error($link);
      echo "<div class=\"error\">$err</div>\n";
      $proceed = false;
    }
    step_footer();
  }

  // create the user
  if ($proceed && !$S['user_created']) {
    step_header(true);
    echo "<div class=\"title\">Create User</div>\n";
    switch ($S['type']) {
      case 'mysql':
        // if the db host is "localhost", append "localhost" to the username (weird MySQL behavior..)
        $user_string = "'{$S['user']}'";
        if (!$S['host'] || $S['host'] == 'localhost') { $user_string .= "@localhost"; }

        $sql = "GRANT ALL on {$S['name']}.* to $user_string identified by '{$S['pass']}'";
        echo "<pre class=\"sql\">$sql</pre>\n";
        $res = setup_db_execute($link, $sql);
        if (!$res) {
          $err = setup_db_error($link);
          echo "<div class=\"error\">$err</div>\n";
          $proceed = false;
        }
        break;

      case 'pgsql':
        $sql = "CREATE USER {$S['user']} with password '{$S['pass']}'";
        echo "<pre class=\"sql\">$sql</pre>\n";
        $res = setup_db_execute($link, $sql);
        if (!$res) {
          $err = setup_db_error($link);
          echo "<div class=\"error\">$err</div>\n";
          $proceed = false;
        } else {
          // try a GRANT to see if it works
          $sql = "GRANT ALL on DATABASE {$S['name']} to {$S['user']}";
          echo "<pre class=\"sql\">$sql</pre>\n";
          $res = setup_db_execute($link, $sql);
          if (!$res) {
            $err = setup_db_error($link);
            echo "<div class=\"error\">$err</div>\n";
            $proceed = false;
          }
        }
    }
    step_footer();
  }

  // close the link if necessary
  if ($link) {
    setup_db_close($link);
  }

  // Double-check that we can access it now.
  if ($proceed) {
    step_header(true);
    echo "<div class=\"title\">Test New User</div>\n";
    // try accessing the named database
    // if not successful, bail with an error on the database name
    if ($err = try_db_login($S['type'], $S['host'], $S['port'], $S['user'], $S['pass'], $S['name'])) {
      echo "<div class=\"error\">Still could not connect to the database with the EnterTrack username and password.  This most often occurs when the database is not located on the web server, in which case the manual intervention of a database administrator is required.  The error was: $err.</div>";
      $proceed = false;
    } else {
      echo "OK";
    }
    step_footer();
  }
?>
</div>
<?php if ($proceed): ?>
<p>The database and user have been installed successfully.  Click "Next" to install the database schema.</p>
<?php else: ?>
<p><span class="error">The installation failed; consult the messages above to determine the cause.</span></p>
<?php endif; ?>
<input type="hidden" name="stage" value="install_db_user">
  <?php
  navigation_box($proceed);
}

function database_install_db_user_submit()
{
  return 'install_schema'; // move on
}
/* }}} */

/* {{{ install_schema page */
function database_install_schema_page()
{
  // Some shortcuts to make the below more readable
  $S = $_SESSION['database'];

  // The forms on the page
  ?>
<h2>Install Schema</h2>
<p>The setup wizard is installing the EnterTrack database Schema.</p>
<div class="setup-steps">
<?php

  // Install the schema
  $link = db_connect();
  if (!$link) { die("Could not connect to database"); } // this condition has been tested for; shouldn't happen

  $proceed = true;
  foreach ($_ENV['schema'] as $name => $table) {
    $sqls = sql_create_table($link, $table);
    step_header(true);
    echo "<div class=\"title\">$name</div>";
    foreach ($sqls as $sql) {
      echo "<pre class=\"sql\">$sql</pre>\n";
      if (!setup_db_execute($link, $sql)) {
        $proceed = false;
        echo "<div class=\"error\">";
        echo setup_db_error($link);
        echo "</div>\n";
        break;
      }
    }
    step_footer();

    if (!$proceed) { break; }
  }

  // add the appropriate schema version
  if ($proceed) {
    $v = CURRENT_SCHEMA_VERSION; // defined in setup/util/schema.php
    step_header(true);
    echo "<div class=\"title\">Setting schema version to $v</div>\n";
    $sql = "INSERT into schema_version values ($v)";
    echo "<pre class=\"sql\">$sql</pre>\n";
    if (!setup_db_execute($link, $sql)) {
      $proceed = false;
      echo "<div class=\"error\">";
      echo setup_db_error($link);
      echo "</div>\n";
    }
    step_footer();
  }

  if ($proceed) {
    setup_db_commit($link);
  }
?>
</div>
<?php if ($proceed): ?>
<p>The schema has been installed successfully.  Click "Next" to add data to the new installation.</p>
<?php else: ?>
<p><span class="error">The schema installation failed; consult the messages above to determine the cause.</span></p>
<?php endif; ?>
<input type="hidden" name="stage" value="install_schema">
  <?php
  navigation_box($proceed);
}

function database_install_schema_submit()
{
  return 'install_data_type'; // move on
}
/* }}} */

/* {{{ install_data_type page */
function database_install_data_type_page()
{
  // Some shortcuts to make the below more readable
  $S = $_SESSION['database'];

  // The forms on the page
  ?>
<h2>Install Data Type</h2>
<p>The setup wizard will now add some data to the database.  The wizard can add minimal data to create a functional system, which you can then customize.  Alternately, the demonstration data provides an instructive example to help you get started quickly with EnterTrack.</p>
<p>What sort of initial data would you like to install?</p>
<table width="100%">
<tr><td align="right" width="20%">Demonstration:</td><td><input type="radio" name="install_data_type" value="demo"<?php if ($S['install_data_type'] == 'demo') { echo ' CHECKED'; } ?>><?php err("install_data_type"); ?></td></tr>
<tr><td align="right" width="20%">Minimal:</td><td><input type="radio" name="install_data_type" value="minimal"<?php if ($S['install_data_type'] == 'minimal') { echo ' CHECKED'; } ?>></td></tr>
</table>
<input type="hidden" name="stage" value="install_data_type">
  <?php
  navigation_box(true);
}

function database_install_data_type_submit()
{
  // reset all error flags
  $_SESSION['database']['error'] = array();
  $proceed = true;

  if (isset($_POST['install_data_type'])) {
    $_SESSION['database']['install_data_type'] = $_POST['install_data_type'];
    return 'install_data';
  }

  $_SESSION['database']['error']['install_data_type'] = "Please specify a type of data installation.";
  return 'install_data_type';
}
/* }}} */

/* {{{ install_data page */
function database_install_data_page()
{
  // Some shortcuts to make the below more readable
  $S = $_SESSION['database'];

  // The forms on the page
  ?>
<h2>Install Data</h2>
<p>The setup wizard is installing the EnterTrack database data.</p>
<div class="setup-steps">
<?php

  // the queries to execute
  $minimal_sql = array(
    "INSERT INTO statuses (status,status_type) VALUES('Registered',1)",
    "INSERT INTO groups (name,notes) VALUES('Employees','This group is the members that maintain the issue tracker system.')",
    "INSERT INTO users (username,password,primary_email,admin) VALUES('admin','fe01ce2a7fbac8fafaed7c982a04e229','admin@localhost','t')",
    "INSERT INTO alternate_emails (userid, email) VALUES(1, 'admin@localhost')",
    "INSERT INTO group_users (userid,gid,perm_set) VALUES(1,1,1)",
    "INSERT INTO permissions (permission,system) VALUES('create_issues','t')",
    "INSERT INTO permissions (permission,system) VALUES('create_announcements','t')",
    "INSERT INTO permissions (permission,system) VALUES('view_private','t')",
    "INSERT INTO permissions (permission,system) VALUES('upload_files','t')",
    "INSERT INTO permissions (permission,system) VALUES('update_group','t')",
    "INSERT INTO permissions (permission,system) VALUES('technician','t')",
    "INSERT INTO permissions (permission,system) VALUES('move_issues','t')",
    "INSERT INTO permissions (permission,system) VALUES('edit_events','t')",
    "INSERT INTO permissions (permission,system) VALUES('assign_issues','t')",
    "INSERT INTO permissions (permission,user_perm,system) VALUES('product_manager','t','t')",
    "INSERT INTO permissions (permission,user_perm,system) VALUES('status_manager','t','t')",
    "INSERT INTO permissions (permission,user_perm,system) VALUES('category_manager','t','t')",
    "INSERT INTO permissions (permission,user_perm,system) VALUES('field_manager','t','t')",
    "INSERT INTO permissions (permission,user_perm,system) VALUES('cook_date','t','t')",
    "INSERT INTO permission_sets (name,description,permissions,system) VALUES ('Group Administrator','This permission set is for group administrators. It gives the user all available privileges for the group.','create_issues,create_announcements,view_private,upload_files,update_group,technician,move_issues,edit_events,field_manager,cook_date,assign_issues','t')",
    "INSERT INTO permission_sets (name,description,permissions,system) VALUES ('Client','This is the default client privilege set. It includes just enough permissions for a client to create issues and what would be required to update those issues with useful information.','create_issues,upload_files','t')",
    "INSERT INTO permission_sets (name,description,permissions,system) VALUES ('Privileged Client','This permission set is the same as the standard client permission set, with the addition of the permission to create announcements.','create_issues,create_announcements,upload_files','t')",
    "INSERT INTO permission_sets (name,description,permissions,system) VALUES ('Super Client','This permission set is the same as as the privileged client permission set, with the addition of the permission to view private events.','create_issues,create_announcements,view_private,upload_files,assign_issues','t')",
    "INSERT INTO permission_sets (name,description,permissions,system) VALUES ('Technician','This is the standard group technician permission set.','create_issues,create_announcements,view_private,upload_files,technician,assign_issues','t')",
    "INSERT INTO custom_field_types (typeid, type, tpl_path, name) VALUES (1, 'checkbox', 'default/tpl/fields/checkbox.tpl', 'checkbox')",
    "INSERT INTO custom_field_types (typeid, type, tpl_path, name) VALUES (2, 'date', 'default/tpl/fields/date.tpl', 'date')",
    "INSERT INTO custom_field_types (typeid, type, tpl_path, name) VALUES (3, 'select', 'default/tpl/fields/select.tpl', 'select')",
    "INSERT INTO custom_field_types (typeid, type, tpl_path, name) VALUES (4, 'select multiple', 'default/tpl/fields/select_multiple.tpl', 'select multiple')",
    "INSERT INTO custom_field_types (typeid, type, tpl_path, name) VALUES (5, 'text', 'default/tpl/fields/text.tpl', 'textfield')",
    "INSERT INTO custom_field_types (typeid, type, tpl_path, name) VALUES (6, 'textarea', 'default/tpl/fields/textarea.tpl', 'textbox')",
    "INSERT INTO motd values ('<b>Welcome to the EnterTrack artifact tracking system.</b>')",
  );

  $demo_sql = array(
    "INSERT INTO statuses (status,status_type) VALUES('Waiting on Tech',2)",
    "INSERT INTO statuses (status,status_type) VALUES('Waiting on Client',2)",
    "INSERT INTO statuses (status,status_type) VALUES('Closed by Tech',5)",
    "INSERT INTO statuses (status,status_type) VALUES('Closed by Client',5)",
    "INSERT INTO statuses (status,status_type) VALUES('Long Term',3)",
    "INSERT INTO statuses (status,status_type) VALUES('Stale',4)",
    "INSERT INTO statuses (status,status_type) VALUES('Auto Closed',6)",
    "INSERT INTO users (username,password,primary_email) VALUES('demo','fe01ce2a7fbac8fafaed7c982a04e229','demo@localhost')",
    "INSERT INTO alternate_emails (userid, email) VALUES(2, 'demo@localhost')",
    "INSERT INTO groups (name,notes) VALUES('Demo Group','Just a demo group')",
    "INSERT INTO categories (category) VALUES('Demo Category')",
    "INSERT INTO products (product) VALUES('Demo Product')",
    "INSERT INTO group_statuses (gid,sid) VALUES(2,1)",
    "INSERT INTO group_statuses (gid,sid) VALUES(2,2)",
    "INSERT INTO group_statuses (gid,sid) VALUES(2,3)",
    "INSERT INTO group_statuses (gid,sid) VALUES(2,4)",
    "INSERT INTO group_statuses (gid,sid) VALUES(2,5)",
    "INSERT INTO group_statuses (gid,sid) VALUES(2,6)",
    "INSERT INTO group_users (userid,gid,perm_set) VALUES(2,1,2)",
    "INSERT INTO group_users (userid,gid,perm_set) VALUES(1,2,1)",
    "INSERT INTO group_users (userid,gid,perm_set) VALUES(2,2,2)",
    "INSERT INTO group_categories (gid,cid) VALUES(1,1)",
    "INSERT INTO group_categories (gid,cid) VALUES(2,1)",
    "INSERT INTO group_products (gid,pid) VALUES(1,1)",
    "INSERT INTO group_products (gid,pid) VALUES(2,1)",
    "INSERT INTO group_statuses (gid,sid) VALUES(1,1)",
    "INSERT INTO group_statuses (gid,sid) VALUES(1,2)",
    "INSERT INTO group_statuses (gid,sid) VALUES(1,3)",
    "INSERT INTO group_statuses (gid,sid) VALUES(1,4)",
    "INSERT INTO group_statuses (gid,sid) VALUES(1,5)",
    "INSERT INTO group_statuses (gid,sid) VALUES(1,6)",
  );

  $query_steps = array("Adding minimal data" => $minimal_sql);
  if ($S['install_data_type'] == 'demo') {
    $query_steps['Adding demonstration data'] = $demo_sql;
  }

  // Install the data
  $link = db_connect();
  if (!$link) { die("Could not connect to database"); } // this condition has been tested for; shouldn't happen

  $proceed = true;
  foreach ($query_steps as $title => $queries) {
    step_header(true);
    echo "<div class=\"title\">$title</div>\n";
    foreach ($queries as $sql) {
      echo "<pre class=\"sql\">$sql</pre>\n";
      if (!setup_db_execute($link, $sql)) {
        $proceed = false;
        echo "<div class=\"error\">";
        echo setup_db_error($link);
        echo "</div>\n";
      }

      if (!$proceed) { break; }
    }
    step_footer();

    if (!$proceed) { break; }
  }
?>
</div>
<?php if ($proceed): ?>
<p>The data has been installed successfully.  Click "Next" to continue.</p>
<?php else: ?>
<p><span class="error">The data installation failed; consult the messages above to determine the cause.</span></p>
<?php endif; ?>
<input type="hidden" name="stage" value="install_data">
  <?php
  navigation_box($proceed);
}

function database_install_data_submit()
{
  redirect("config.php");
}
/* }}} */

////
// page utilities

/* {{{ Function err */
/**
 * Function to display errors.  Errors are taken from
 * $_SESSION['database']['error'][$key]
 *
 * @param string $key the field name to display the error for
 */
function err($key)
{
  if (isset($_SESSION['database']['error'][$key])) {
    echo "<span class=\"error\">{$_SESSION['database']['error'][$key]}</span>";
  }
}
/* }}} */

/* {{{ Function db_connect */
/**
 * Function to connect to the database.  Tries to use the administrative
 * username and password, falling back to the regular username and password.
 *
 * @return object the database link, or null on failure
 */
function db_connect()
{
  $S = $_SESSION['database'];

  if ($S['admin_user'] && $S['admin_pass']) {
    $link = setup_db_connect($S['type'], $S['host'], $S['port'], $S['admin_user'], $S['admin_pass'], $S['name']);
  } else {
    $link = setup_db_connect($S['type'], $S['host'], $S['port'], $S['user'], $S['pass'], $S['name']);
  }

  return $link;
}
/* }}} */

////
// page handling

// if the session doesn't exist, start over
if (!isset($_SESSION['database']) || $_SERVER['REQUEST_METHOD'] != 'POST') {
  // load up any existing ET config
  $config = new Configuration();

  // initialize a fresh new config
  $_SESSION['database'] = array(
    'setup_type' => ($config->original_version == 'none')?'install':'upgrade',
    'type' => $config->data['db']['type'],
    'host' => $config->data['db']['host'],
    'port' => $config->data['db']['port'],
    'name' => $config->data['db']['name'],
    'user' => $config->data['db']['user'],
    'pass' => $config->data['db']['pass'],
    'db_created' => null,
    'user_created' => null,
    'admin_user' => '',
    'admin_pass' => '',
    'schema_version' => null,
    'install_data_type' => 'demo',
    );

  // and start at the first stage
  $stage = 'setup_type';

// Otherwise, carry out an action if need be
} else {
  // Check to see if this was a request to start over
  if (isset($_POST['_start_over'])) {
    redirect("index.php");
  }

  // get the stage specified in the form
  if (!isset($_POST['stage'])) { die("stage not set"); }
  $stage = $_POST['stage'];

  // call out to the appropriate submit function, which returns
  // the new stage
  switch ($stage) {
    case 'setup_type': $stage = database_setup_type_submit(); break;
    case 'config': $stage = database_config_submit(); break;
    case 'install_get_admin': $stage = database_install_get_admin_submit(); break;
    case 'install_db_user': $stage = database_install_db_user_submit(); break;
    case 'install_schema': $stage = database_install_schema_submit(); break;
    case 'install_data_type': $stage = database_install_data_type_submit(); break;
    case 'install_data': $stage = database_install_data_submit(); break;
    case 'backup': $stage = database_backup_submit(); break;
    case 'schema_version': $stage = database_schema_version_submit(); break;
    case 'upgrade_schema': $stage = database_upgrade_schema_submit(); break;
    default: die("Bad stage " . htmlentities($stage));
  }
}

// Draw the install page
page_header("Setup :: Database", "Use this page to set up and configure your EnterTrack database.");
step_js();

switch ($stage) {
  case 'setup_type': database_setup_type_page(); break;
  case 'config': database_config_page(); break;
  case 'install_get_admin': database_install_get_admin_page(); break;
  case 'install_db_user': database_install_db_user_page(); break;
  case 'install_schema': database_install_schema_page(); break;
  case 'install_data_type': database_install_data_type_page(); break;
  case 'install_data': database_install_data_page(); break;
  case 'backup': database_backup_page(); break;
  case 'schema_version': database_schema_version_page(); break;
  case 'upgrade_schema': database_upgrade_schema_page(); break;
  default: die("Bad stage " . htmlentities($stage));
}

page_footer();

