<?php

/***
 * These functions implement the barest database abstraction layer, used only
 * by the database phase of the setup wizard.
 */

////
// Database Interface

/* {{{ Function try_db_login */
/**
 * Try to access the database
 *
 * Using the supplied username and password, and optionally database name,
 * try to access the database and return a boolean value indicating success
 *
 * @param string $type
 * @param string $host
 * @param string $port
 * @param string $user
 * @param string $pass
 * @param string $name (optional)
 * @return mixed false if no error, otherwise the error message
 */
function try_db_login($type, $host, $port, $user, $pass, $name=null)
{
  /* we use the low-level PHP builtin functions here, to avoid any complications
   * introduced by abstraction layers */
  switch ($type) {
    case 'mysql':
      $hostspec = $host . ($port? ":$port" : "");

      if ($link = @mysql_connect("$hostspec", $user, $pass)) {
        if ($name && !@mysql_select_db($name, $link)) {
          $result = mysql_error();
        } else {
          $result = false; // no error
        }
        mysql_close($link);
      } else {
        $result = mysql_error();
      }
      return $result;

    case 'pgsql':
      $connect_string  = array();
      if ($name) { $connect_string[] = "dbname={$name}"; }
            else { $connect_string[] = "dbname=template1"; } // dbname is required
      if ($host) { $connect_string[] = "host=$host"; }
      if ($port) { $connect_string[] = "port=$port"; }
      if ($user) { $connect_string[] = "user=$user"; }
      if ($pass) { $connect_string[] = "password=$pass"; }
      $connect_string = implode(' ', $connect_string);
      // use output buffering to get the error message here
      ob_start();
      $link = pg_connect($connect_string);
      $ob_error = strip_tags(ob_get_clean());

      if ($link) {
        pg_close($link);
        $result = false; // no error
      } else {
        $result = @pg_last_error();
        // sometimes PostegreSQL doesn't leave us an error code, so we have to
        // fall back to the one that PHP generated via output buffering
        if (!$result) { $result = $ob_error; }
      }
      return $result;

    default:
      return false;
  }
}
/* }}} */

/* {{{ Function setup_db_connect */
/**
 * Set up a new database connection
 *
 * @param string $type
 * @param string $host
 * @param string $port
 * @param string $user
 * @param string $pass
 * @param string $name (optional)
 * @return mixed the link or null in case of an error
 */
function setup_db_connect($type, $host, $port, $user, $pass, $name=null)
{
  switch ($_SESSION['database']['type']) {
    case 'mysql':
      $hostspec = $host . ($port? ":$port" : "");

      if ($link = @mysql_connect("$hostspec", $user, $pass)) {
        if ($name && !@mysql_select_db($name, $link)) {
          mysql_close($link);
          return null;
        }

        return $link;
      }
      return null;

    case 'pgsql':
      $result = false;
      $connect_string  = array();
      if ($name) { $connect_string[] = "dbname={$name}"; }
            else { $connect_string[] = "dbname=template1"; } // dbname is required
      if ($host) { $connect_string[] = "host=$host"; }
      if ($port) { $connect_string[] = "port=$port"; }
      if ($user) { $connect_string[] = "user=$user"; }
      if ($pass) { $connect_string[] = "password=$pass"; }
      $connect_string = implode(' ', $connect_string);
      $link = @pg_connect($connect_string);
      if ($link) {
        return $link;
      }
      return null;

    default:
      return null;
  }
}
/* }}} */

/* {{{ Function setup_db_execute */
/**
 * Execute a query on the given link
 *
 * @param object $link
 * @param string $sql
 * @return mixed the result from the databae layer
 */
function setup_db_execute($link, $sql)
{
  switch ($_SESSION['database']['type']) {
    case 'mysql':
      return mysql_query($sql, $link);

    case 'pgsql':
      return @pg_query($link, $sql);

    default:
      return null;
  }
}
/* }}} */

/* {{{ Function setup_db_last_id */
/**
 * Get the ID created by the last insert statement
 *
 * @param object $link
 * @param string $sequence -- the name of the sequenec to look at
 * @return mixed the result from the databae layer
 */
function setup_db_last_id($link, $sequence)
{
  switch ($_SESSION['database']['type']) {
    case 'mysql':
      return mysql_insert_id($link);

    case 'pgsql':
      $r = pg_query($link, "SELECT last_value FROM $sequence");
      if (!$r) { return null; }
      $row = pg_fetch_row($r);
      $id = $row[0];
      pg_free_result($r);
      return $id;

    default:
      return null;
  }
}
/* }}} */

/* {{{ Function setup_db_fetchall */
/**
 * Execute a query on the given link and return the results
 * as an array of associative arrays
 *
 * @param object $link
 * @param string $sql
 * @return array the results, or null in case of an error
 */
function setup_db_fetchall($link, $sql)
{
  switch ($_SESSION['database']['type']) {
    case 'mysql':
      $r = mysql_query($sql, $link);
      if (!$r) { return null; }

      $rv = array();
      while ($row = mysql_fetch_assoc($r)) { $rv[] = $row; }

      mysql_free_result($r);

      return $rv;

    case 'pgsql':
      $r = @pg_query($link, $sql);
      if (!$r) { return null; }

      $rv = pg_fetch_all($r);

      // pg_fetch_all returns 'false' on an empty set, so detect that
      if (!$rv && pg_num_rows($r) == 0) {
        $rv = array();
      }

      pg_free_result($r);

      return $rv;

    default:
      return null;
  }
}
/* }}} */

/* {{{ Function setup_db_column_exists */
/**
 * Determine whether or not the given column of the given table exists.
 *
 * Note: this will also return false if the table itself doesn't exist.
 *
 * @param object $link
 * @param string $table
 * @param string $column
 * @return string SQL
 */
function setup_db_column_exists($link, $table, $column)
{
  switch ($_SESSION['database']['type']) {
    case 'mysql':
      $r = @mysql_query("select $column from $table limit 1", $link);
      if ($r) {
        mysql_free_result($r);
        return true;
      }
      return false;

    case 'pgsql':
      $r = @pg_query($link, "select $column from $table limit 1");
      if ($r) {
        pg_free_result($r);
        return true;
      }
      return false;

    default:
      return false;
  }
}
/* }}} */

/* {{{ Function setup_db_error */
/**
 * Return the most recent error message from the database
 *
 * @param object $link
 * @return string the error message
 */
function setup_db_error($link)
{
  switch ($_SESSION['database']['type']) {
    case 'mysql':
      return mysql_error($link);

    case 'pgsql':
      return pg_last_error($link);

    default:
      return '(unkonwn)';
  }
}
/* }}} */

/* {{{ Function setup_db_close */
/**
 * Close a database connection
 *
 * @param object $link
 */
function setup_db_close($link)
{
  switch ($_SESSION['database']['type']) {
    case 'mysql':
      mysql_close($link);
      break;

    case 'pgsql':
      pg_close($link);
      break;
  }
}
/* }}} */

/* {{{ Function setup_db_commit */
/**
 * Commit the current transaction.
 *
 * @param object $link
 */
function setup_db_commit($link)
{
  switch ($_SESSION['database']['type']) {
    case 'mysql':
      return; // hah hah

    case 'pgsql':
      pg_query($link, "COMMIT");
      return;

    default:
      return null;
  }
}
/* }}} */

/* {{{ Function setup_db_escape */
/**
 * Escape a string for inclusion in a query (does not include quotes)
 *
 * @param object $link
 * @param string $string
 * @return string the escaped string
 */
function setup_db_escape($link, $string)
{
  switch ($_SESSION['database']['type']) {
    case 'mysql':
      return mysql_real_escape_string($string, $link);

    case 'pgsql':
      return pg_escape_string($string);

    default:
      return null;
  }
}
/* }}} */

////
// SQL generation

/* {{{ Function sql_add_column */
/**
 * Generate an SQL query to add a column
 *
 * @param object $link
 * @param string $table
 * @param string $column
 * @param string $column_spec the column object (see setup/database/schema.php)
 * @return array SQL statements
 */
function sqls_add_column($link, $table, $column, $column_spec)
{
  switch ($_SESSION['database']['type']) {
    case 'mysql':
      return array("ALTER table $table add column $column " . $column_spec->sql($_SESSION['database']['type']));

    case 'pgsql':
      $sqls = array();
      $sqls[] = "ALTER table $table add column $column " . $column_spec->sql_type($_SESSION['database']['type']);
      if ($column_spec->defaultvalue) {
        $sqls[] = "ALTER  table $table alter column $column set default '" . addslashes($column_spec->defaultvalue) . "'";
      }
      if ($column_spec->notnull) {
        $sqls[] = "UPDATE $table set $column = '" . addslashes($column_spec->defaultvalue) . "' where $column is NULL";
        $sqls[] = "ALTER  table $table alter column $column set not null";
      }
      return $sqls;
  }
}
/* }}} */

/* {{{ Function sql_set_not_null */
/**
 * Generate an SQL query to change whether it's 'NOT NULL'
 *
 * @param object $link
 * @param string $table
 * @param string $column
 * @param boolean $notnull
 * @return string SQL
 */
function sql_set_not_null($link, $table, $column, $colspec)
{
  switch ($_SESSION['database']['type']) {
    case 'mysql':
      return "ALTER table $table modify column $column " . $colspec->sql('mysql');

    case 'pgsql':
      if ($colspec->notnull) {
        return "ALTER table $table alter column $column set not null";
      } else {
        return "ALTER table $table alter column $column drop not null";
      }

    default:
      return false;
  }
}
/* }}} */

/* {{{ Function sql_drop_column */
/**
 * Generate an SQL query to drop a column
 *
 * @param object $link
 * @param string $table
 * @param string $column the column to drop
 * @return string SQL
 */
function sql_drop_column($link, $table, $column)
{
  return "ALTER table $table drop column $column";
}
/* }}} */

/* {{{ Function sql_drop_primary_key */
/**
 * Generate an SQL query to drop a (known) primary key from a table
 *
 * @param object $link
 * @param string $table
 * @param string $pkey_name name of the primary key (postgresql only)
 * @return string SQL
 */
function sql_drop_primary_key($link, $table, $pkey_name)
{
  switch ($_SESSION['database']['type']) {
    case 'mysql':
      return "ALTER table $table drop primary key";

    case 'pgsql':
      return "ALTER table $table drop constraint $pkey_name";

    default:
      return false;
  }
}
/* }}} */

/* {{{ Function sql_add_index */
/**
 * Generate an SQL query to add a new index to a table
 *
 * @param object $link
 * @param string $table
 * @param object $idx the Index object
 * @return string SQL
 */
function sql_add_index($link, $table, $idx)
{
  return $idx->sql($table);
}
/* }}} */

/* {{{ Function sql_rename_column */
/**
 * Generate sql to rename a column of a table
 *
 * @param object $link
 * @param string $table
 * @param string $old_name the old name
 * @param string $new_name the new name
 * @param string $column_spec the column object (see setup/database/schema.php)
 * @return string SQL
 */
function sql_rename_column($link, $table, $old_name, $new_name, $column_spec)
{
  switch ($_SESSION['database']['type']) {
    case 'mysql':
      return "ALTER table $table change $old_name $new_name " . $column_spec->sql('mysql');

    case 'pgsql':
      // pgsql doesn't need the column spec
      return "ALTER table $table rename column $old_name to $new_name";

    default:
      return false;
  }
}
/* }}} */

/* {{{ Function sql_create_table */
/**
 * Generate sql to create a table.  NOTE: This
 * function returns more than one statement in many cases!
 *
 * @param object $link
 * @param string $table_object the Table object (see setup/database/schema.php)
 * @return array of SQL statements to be executed in order
 */
function sql_create_table($link, $table_object)
{
  return $table_object->sql($_SESSION['database']['type']);
}
/* }}} */

/* {{{ Function sql_drop_table */
/**
 * Generate SQL to drop a table
 *
 * @param object $link
 * @param string $table_object the Table object (see setup/database/schema.php)
 * @return string SQL
 */
function sql_drop_table($link, $table_name)
{
  return "DROP table $table_name";
}
/* }}} */

/* {{{ Function sql_rename_table */
/**
 * Generate sql to rename a table
 *
 * @param object $link
 * @param string $old_name the old table name
 * @param string $new_name the new table name
 * @return string SQL
 */
function sql_rename_table($link, $old_name, $new_name)
{
  switch ($_SESSION['database']['type']) {
    case 'mysql':
      return "RENAME table $old_name to $new_name";

    case 'pgsql':
      return "ALTER table $old_name rename to $new_name";

    default:
      return false;
  }
}
/* }}} */
