<?php
/* {{{ Function screen_css */
/**
 * Generate the CSS used for on-screen display for this
 * theme.
 */
function screen_css() {
  // start building the rules
  $rules = array();

  $rules['html, body'] = array();
  $rules['html, body']['border-right'] = "0px !important";

  $rules['body'] = array();
  $rules['body']['margin'] = "0px";
  $rules['body']['margin-top'] = "0px";
  $rules['body']['margin-left'] = "0px";
  $rules['body']['margin-right'] = "0px";
  $rules['body']['padding'] = "0px";

  $rules['table'] = array();
  $rules['table']['border-collapse'] = "collapse";
  $rules['table']['empty-cells'] = "show";

  $rules['tr, td'] = array();
  $rules['tr, td']['vertical-align'] = "top";
  $rules['tr, td']['empty-cells'] = "show";
  $rules['tr, td']['border-collapse'] = "collapse";

  $rules['a, a:visited'] = array();
  $rules['a, a:visited']['background'] = "transparent";

  $rules['a:hover'] = array();
  $rules['a:hover']['background'] = "transparent";
  $rules['a:hover']['text-decoration'] = "underline !important";

  $rules['.buttons'] = array();
  $rules['.buttons']['width'] = "98%";

  $rules['.iconbar'] = array();
  $rules['.iconbar']['text-align'] = "right";
  $rules['.iconbar']['width'] = "99%";
  $rules['.iconbar']['margin-bottom'] = "4px";
  $rules['.iconbar']['padding'] = "2px";

  $rules['img'] = array();
  $rules['img']['behavior'] = "url('themes/default/pngbehavior.htc')";
  $rules['img']['vertical-align'] = "middle";

  $rules['.fixed'] = array();
  $rules['.fixed']['font-family'] = "courier";

  $rules['.header, .header a, .header a:visited'] = array();
  $rules['.header, .header a, .header a:visited']['vertical-align'] = "middle";
  $rules['.header, .header a, .header a:visited']['background-color'] = "black";
  $rules['.header, .header a, .header a:visited']['color'] = "white";
  $rules['.header, .header a, .header a:visited']['padding'] = "2px";

  $rules['.crumb'] = array();
  $rules['.crumb']['font-weight'] = "bold";
  $rules['.crumb']['padding'] = "2px";
  $rules['.crumb']['margin'] = "2px";
  $rules['.crumb']['vertical-align'] = "middle";

  $rules['.crumb a, .crumb a:visited'] = array();
  $rules['.crumb a, .crumb a:visited']['font-weight'] = "bold";
  $rules['.crumb a, .crumb a:visited']['padding'] = "2px";
  $rules['.crumb a, .crumb a:visited']['margin'] = "2px";
  $rules['.crumb a, .crumb a:visited']['vertical-align'] = "middle";

  $rules['.titlebar, .titlebar a, .titlebar a:visited'] = array();
  $rules['.titlebar, .titlebar a, .titlebar a:visited']['font-weight'] = "bold";

  $rules['.tablehead, .tablehead td, .tablehead a, .tablehead a:visited'] = array();
  $rules['.tablehead, .tablehead td, .tablehead a, .tablehead a:visited']['font-weight'] = "bold";

  $rules['.private'] = array();
  $rules['.private']['vertical-align'] = "top";
  $rules['.private']['background-repeat'] = "no-repeat";
  $rules['.private']['background-position'] = "center";

  $rules['.label'] = array();
  $rules['.label']['vertical-align'] = "top";

  $rules['.error'] = array();
  $rules['.error']['font-weight'] = "bold";

  $rules['.required'] = array();
  $rules['.required']['background'] = "transparent";
  $rules['.required']['font-weight'] = "bold";

  $rules['.noborders, .noborders td'] = array();
  $rules['.noborders, .noborders td']['border'] = "none";

  $rules['.leftnav'] = array();
  $rules['.leftnav']['padding'] = "0px";
  $rules['.leftnav']['margin'] = "0px";

  $rules['.leftnav td'] = array();
  $rules['.leftnav td']['border-top'] = "none";
  $rules['.leftnav td']['border-right'] = "none";
  $rules['.leftnav td']['border-left'] = "none";

  $rules['.rightnav'] = array();
  $rules['.rightnav']['padding'] = "0px";
  $rules['.rightnav']['margin'] = "0px";

  $rules['.rightnav td'] = array();
  $rules['.rightnav td']['border-top'] = "none";
  $rules['.rightnav td']['border-right'] = "none";
  $rules['.rightnav td']['border-left'] = "none";

  $rules['input, select, textarea'] = array();
  $rules['input, select, textarea']['margin'] = "2px";

  $rules['textarea'] = array();
  $rules['textarea']['width'] = "99%";
  $rules['textarea']['color'] = "black";
  $rules['textarea']['background-color'] = "white";

  $rules['.privateupdate'] = array();
  $rules['.privateupdate']['vertical-align'] = "top";
  $rules['.privateupdate']['color'] = "black";
  $rules['.privateupdate']['background-color'] = "white";
  $rules['.privateupdate']['background-image'] = "url("._URL_."images/private-event.png)";
  $rules['.privateupdate']['background-repeat'] = "no-repeat";
  $rules['.privateupdate']['background-position'] = "center";

  $rules['form'] = array();
  $rules['form']['display'] = "inline";

  $rules['.debug'] = array();
  $rules['.debug']['margin'] = "2px";
  $rules['.debug']['padding'] = "2px";

  $rules['.login'] = array();
  $rules['.login']['color'] = "black";
  $rules['.login']['background'] = "#cccccc";

  return $rules;
}
/* }}} */

/* {{{ Function user_css */
/**
 * Generate the CSS that is based on user-specifiable settings
 */
function user_css($theme) {
  // make all members of this array into local variables
  extract($theme);

  // start building the rules
  $rules = array();

  $rules['body'] = array();
  $rules['body']['font-size'] = "$font_size$font_unit";
  $rules['body']['color'] = $fgcolor;
  $rules['body']['font-family'] = $font_family;
  $rules['body']['background-color'] = $bgcolor;

  $rules['tr, td'] = array();
  $rules['tr, td']['font-size'] = "$font_size$font_unit";
  $rules['tr, td']['color'] = $fgcolor;
  $rules['tr, td']['font-family'] = $font_family;

  $rules['a, a:visited'] = array();
  $rules['a, a:visited']['color'] = $fgcolor;
  $rules['a, a:visited']['font-size'] = "$font_size$font_unit";
  $rules['a, a:visited']['text-decoration'] = $underline?'underline':'none';

  $rules['a:hover'] = array();
  $rules['a:hover']['color'] = "$link_hover !important";

  $rules['.buttons'] = array();
  $rules['.buttons']['color'] = $label_fgcolor;
  $rules['.buttons']['background'] = $label_bgcolor;
  $rules['.buttons']['border'] = "1px solid $border_color";

  $rules['.iconbar'] = array();
  $rules['.iconbar']['color'] = $data_fgcolor;
  $rules['.iconbar']['background'] = $data_bgcolor;
  $rules['.iconbar']['border'] = "1px solid $border_color";

  $rules['.crumb'] = array();
  $rules['.crumb']['font-size'] = "$font_size$font_unit";
  $rules['.crumb']['color'] = $subtitle_fgcolor;
  $rules['.crumb']['background'] = $subtitle_bgcolor;
  $rules['.crumb']['border-top'] = "1px solid $border_color";
  $rules['.crumb']['border-bottom'] = "1px solid $border_color";

  $rules['.crumb a, .crumb a:visited'] = array();
  $rules['.crumb a, .crumb a:visited']['font-size'] = "$font_size$font_unit";
  $rules['.crumb a, .crumb a:visited']['color'] = $subtitle_fgcolor;
  $rules['.crumb a, .crumb a:visited']['background'] = $subtitle_bgcolor;

  $rules['.titlebar, .titlebar a, .titlebar a:visited'] = array();
  $rules['.titlebar, .titlebar a, .titlebar a:visited']['background-color'] = $titlebar_bgcolor;
  $rules['.titlebar, .titlebar a, .titlebar a:visited']['color'] = $titlebar_fgcolor;
  $rules['.titlebar, .titlebar a, .titlebar a:visited']['font-size'] = "$font_size$font_unit";

  $rules['.subtitle, .subtitle a, .subtitle a:visited'] = array();
  $rules['.subtitle, .subtitle a, .subtitle a:visited']['background-color'] = $subtitle_bgcolor;
  $rules['.subtitle, .subtitle a, .subtitle a:visited']['color'] = $subtitle_fgcolor;
  $rules['.subtitle, .subtitle a, .subtitle a:visited']['font-size'] = "$font_size$font_unit";

  $rules['.tablehead, .tablehead td, .tablehead a, .tablehead a:visited'] = array();
  $rules['.tablehead, .tablehead td, .tablehead a, .tablehead a:visited']['color'] = $table_head_fgcolor;
  $rules['.tablehead, .tablehead td, .tablehead a, .tablehead a:visited']['background-color'] = $table_head_bgcolor;

  $rules['.private'] = array();
  $rules['.private']['color'] = $data_fgcolor;
  $rules['.private']['background-color'] = $data_bgcolor;
  $rules['.private']['background-image'] = "url('"._URL_.$_ENV['imgs']['private_event']."')";

  $rules['.label'] = array();
  $rules['.label']['color'] = $label_fgcolor;
  $rules['.label']['background-color'] = $label_bgcolor;

  $rules['.data'] = array();
  $rules['.data']['color'] = $data_fgcolor;
  $rules['.data']['background-color'] = $data_bgcolor;

  $rules['.row1, .row1 td, .row1 a, .row1 a:visited'] = array();
  $rules['.row1, .row1 td, .row1 a, .row1 a:visited']['color'] = $row1_fgcolor;
  $rules['.row1, .row1 td, .row1 a, .row1 a:visited']['background-color'] = $row1_bgcolor;

  $rules['.row2, .row2 td, .row2 a, .row2 a:visited'] = array();
  $rules['.row2, .row2 td, .row2 a, .row2 a:visited']['color'] = $row2_fgcolor;
  $rules['.row2, .row2 td, .row2 a, .row2 a:visited']['background-color'] = $row2_bgcolor;

  $rules['.menu, .menu a, .menu a:visited'] = array();
  $rules['.menu, .menu a, .menu a:visited']['color'] = $menu_fgcolor;
  $rules['.menu, .menu a, .menu a:visited']['background-color'] = $menu_bgcolor;

  $rules['.submenu, .submenu a, .submenu a:visited'] = array();
  $rules['.submenu, .submenu a, .submenu a:visited']['color'] = $submenu_fgcolor;
  $rules['.submenu, .submenu a, .submenu a:visited']['background-color'] = $submenu_bgcolor;

  $rules['.error'] = array();
  $rules['.error']['color'] = $error_fgcolor;
  $rules['.error']['background-color'] = $error_bgcolor;

  $rules['.required'] = array();
  $rules['.required']['color'] = $error_fgcolor;

  $rules['.borders, .borders td'] = array();
  $rules['.borders, .borders td']['border'] = "1px solid $border_color";

  $rules['.leftnav'] = array();
  $rules['.leftnav']['border-right'] = "1px solid $border_color";

  $rules['.leftnav td'] = array();
  $rules['.leftnav td']['border-bottom'] = "1px solid $border_color";

  $rules['.rightnav'] = array();
  $rules['.rightnav']['border-left'] = "1px solid $border_color";

  $rules['.rightnav td'] = array();
  $rules['.rightnav td']['border-bottom'] = "1px solid $border_color";

  $rules['input, select, textarea'] = array();
  $rules['input, select, textarea']['font-size'] = "$font_size$font_unit";
  $rules['input, select, textarea']['font-family'] = $font_family;
  $rules['input, select, textarea']['border'] = "1px solid $border_color";

  $rules['.footer'] = array();
  $rules['.footer']['font-size'] = "$font_size$font_unit";
  $rules['.footer']['color'] = $fgcolor;
  $rules['.footer']['background-color'] = $bgcolor;
  $rules['.footer']['font-family'] = $font_family;
  $rules['.footer']['border-top'] = "1px solid $border_color";

  $rules['.debug'] = array();
  $rules['.debug']['color'] = $data_fgcolor;
  $rules['.debug']['background-color'] = $data_bgcolor;
  $rules['.debug']['font-size'] = "$font_size$font_unit";

  $rules['.login'] = array();
  $rules['.login']['border'] = "1px solid $border_color";

  return $rules;
}
/* }}} */

/* {{{ Function print_css */
/**
 * Generate the CSS used for printing for this theme.
 */
function print_css() {
  // start building the rules
  $rules = array();

  $rules['body'] = array();
  $rules['body']['font-family'] = "monospace";
  $rules['body']['font-size'] = "10pt";
  $rules['body']['color'] = "black";
  $rules['body']['background-color'] = "white";
  $rules['body']['margin'] = "2pt";

  $rules['table'] = array();
  $rules['table']['empty-cells'] = "show";
  $rules['table']['border-collapse'] = "collapse";

  $rules['table.report'] = array();
  $rules['table.report']['font-family'] = "courier";
  $rules['table.report']['font-size'] = "10pt";
  $rules['table.report']['width'] = "100%";
  $rules['table.report']['border-width'] = "1px";
  $rules['table.report']['border-collapse'] = "collapse";

  $rules['td'] = array();
  $rules['td']['color'] = "black";
  $rules['td']['background-color'] = "white";
  $rules['td']['font-size'] = "10pt";
  $rules['td']['vertical-align'] = "top";
  $rules['td']['border-collapse'] = "collapse";

  $rules['pre'] = array();
  $rules['pre']['font-family'] = "helvetica";

  $rules['a, a:visited'] = array();
  $rules['a, a:visited']['color'] = "black";
  $rules['a, a:visited']['text-decoration'] = "none";

  $rules['pre.report'] = array();
  $rules['pre.report']['font-family'] = "courier";
  $rules['pre.report']['font-size'] = "10pt";

  $rules['.header, .header a, .header a:visited'] = array();
  $rules['.header, .header a, .header a:visited']['display'] = "none";

  $rules['.crumb'] = array();
  $rules['.crumb']['display'] = "none";

  $rules['.iconbar'] = array();
  $rules['.iconbar']['display'] = "none";

  $rules['.titlebar'] = array();
  $rules['.titlebar']['text-align'] = "center";
  $rules['.titlebar']['font-weight'] = "bold";

  $rules['.titlebar a, titlebar a:visited'] = array();
  $rules['.titlebar a, titlebar a:visited']['text-align'] = "center";
  $rules['.titlebar a, titlebar a:visited']['font-weight'] = "bold";

  $rules['.subtitle'] = array();
  $rules['.subtitle']['display'] = "none";

  $rules['.tablehead, .tablehead td, .tablehead a, .tablehead a:visited'] = array();
  $rules['.tablehead, .tablehead td, .tablehead a, .tablehead a:visited']['font-weight'] = "bold";

  $rules['.borders'] = array();
  $rules['.borders']['border'] = "1px solid black";
  $rules['.borders']['border-collapse'] = "collapse";

  $rules['.borders td'] = array();
  $rules['.borders td']['border'] = "1px solid black";

  $rules['.noborders td'] = array();
  $rules['.noborders td']['border'] = "none";

  $rules['.private'] = array();
  $rules['.private']['background'] = "url(images/private.gif) no-repeat";
  $rules['.private']['background-position'] = "center";

  $rules['.label'] = array();
  $rules['.label']['font-weight'] = "bold";

  $rules['.error'] = array();
  $rules['.error']['font-weight'] = "bold";

  $rules['.leftnav'] = array();
  $rules['.leftnav']['display'] = "none";

  $rules['.rightnav'] = array();
  $rules['.rightnav']['display'] = "none";

  $rules['input, select, textarea'] = array();
  $rules['input, select, textarea']['font-size'] = "10pt";
  $rules['input, select, textarea']['border'] = "0px solid black";

  $rules['textarea'] = array();
  $rules['textarea']['font-size'] = "10pt";
  $rules['textarea']['width'] = "99%";

  $rules['.footer'] = array();
  $rules['.footer']['display'] = "none";

  $rules['form'] = array();
  $rules['form']['display'] = "inline";

  $rules['.noprint'] = array();
  $rules['.noprint']['display'] = "none";

  $rules['.pagebreak'] = array();
  $rules['.pagebreak']['display'] = "block";
  $rules['.pagebreak']['page-break-after'] = "always";

  return $rules;
}
/* }}} */
